# -*- coding: utf-8 -*-
"""
Created on Tue Sep 24 14:23:34 2024

@author: inagle
"""


import matplotlib
import matplotlib.pyplot as pl
from matplotlib.lines import Line2D
from scipy.ndimage import gaussian_filter1d
import pandas
import io

import codecs
from apMethods_ReadRheo_Datacollection_mt import *


# Martin Krzywinski's 24-color, color-blind-friendly palette
color_blind_palette = {
    "orange": "#E69F00",  # Orange
    "skyblue":"#56B4E9",  # Sky Blue
    "g": "#009E73",  # Green
    "y":"#F0E442",  # Yellow
    "b": "#0072B2",  # Blue
    "darkorange":"#D55E00",  # Dark Orange
    "pink":"#CC79A7",  # Pink
    "grey":"#999999",  # Grey
    "lightyellow":"#F7CB4D",  # Light Yellow
    "lightblue":"#ADD8E6",  # Light Blue
    "lightgreen":"#90EE90",  # Light Green
    "lightpink":"#FFB6C1",  # Light Pink
    "darkred":"#A52A2A",  # Brown
    "purple":"#800080",  # Purple
    "olive":"#808000",  # Olive
    "orange":"#FFA500",  # Orange
    "steelblue":"#4682B4",  # Steel Blue
    "brown":"#8B4513",  # Saddle Brown
    "cadetblue": "#5F9EA0",  # Cadet Blue
    "r":"#CD5C5C"   # Indian Red
}


#Set default color cycle
def resolve_color(color):
    """Resolve shorthand color names to Tableau hex codes."""
    return color_blind_palette.get(color, color)  # Default to original if shorthand not found

# Override the default plot function to use custom color
original_plot = pl.plot

def patched_plot(*args, **kwargs):
    if 'color' in kwargs:
        kwargs['color'] = resolve_color(kwargs['color'])
    return original_plot(*args, **kwargs)

pl.plot = patched_plot 



#This function calculates the average for n samples. 
# It is first required to construct an array containing all the arrays of each sample.
# The function has as variables :
# 1) data that is an array containing the of n x l x c, with n the number of different samples to average,
# l the number or lines in the array, or number of measuring points, and c the number of columns. 
# 2) Title corresponds to the title of the column we would like to plot. 
# For example, Title = Storage Modulus or Title = Loss Modulus

        
def average (data,Title:str):
    # Title = Type + " Modulus"   
    #n number of samples averaged
    n = np.size(data,0)
    S = np.zeros(np.size(data,1))
    for i in range (n) :
        S = S + np.array(data[i][Title], dtype='float64')
    avg = S/n
    return avg 

def standard_deviation (data,Title:str):
    # Title = Type + " Modulus" 
    #n number of samples averaged
    n = np.size(data,0)
    #l number of lines or measuring points per sample
    l = np.size(data,1)
    Q = np.zeros(l)
    for i in range (n) :
        Q = Q + np.square(np.array(data[i][Title], dtype='float64')-average(data,Title))
    SD = np.sqrt(Q/n)
    return SD 

# Plots 
# Plot polymerization curve from data. Mean +/- SD. Data has to be an array of all polymerisation measurements, corresponding to a 12 x 1080 array. (1h30 polymerization and point every 5s)
# G' ( Title = 'Storage Modulus') or G" (Title = 'Loss Modulus') as a function of time.
# Curve and points of given color c, given shape s, and a given label l 
def plotPolymerisation(data,Title:str,l:str,c:str,s:str):
    time_pol = np.array(data[0]['Time'].str.replace(',', ''), dtype='float64') ## adding .str.replace(',', '') removed the commas
    # if np.size(data,0)==1:
    #     pl.plot(time_pol, np.array(data[0][Title],dtype='float64'), ls='-', color = c, label=l,marker=s, ms=1,linewidth=2)
    
    data_avg = average(data,Title)
    data_SD = standard_deviation(data,Title)
    pl.plot(time_pol, data_avg, ls='-', color = c, label=l,marker=s, ms=1, linewidth = 3)
    pl.errorbar(time_pol, data_avg, ls = 'None', yerr = data_SD, ecolor=c,elinewidth=0.05)
    
    pl.xlabel(r't (s)', fontsize=30)
    if Title == 'Storage Modulus':
        pl.ylabel(r'$G^{\prime}$ $(Pa)$', fontsize=30)
        pl.ylim (0,150)
    elif Title == 'Loss Modulus' :
        pl.ylabel(r'$G^{\prime\prime}$ $(Pa)$', fontsize=30) 
        pl.ylim (0,30)
        
    for spine in pl.gca().spines.values():
        spine.set_linewidth(1.5) 
        
    pl.xticks(fontsize=24)
    pl.yticks(fontsize=24)
    pl.tick_params(axis='both', which='major', length=8, width=2)
    pl.legend(loc='upper left',fontsize=20, frameon=False) ##add ncol = 2 for MDAMB
    # pl.tight_layout()
    pl.xlim(0,6000)
    

    
# Plots 
#Plot polymerization curve from data. Plots all the individual curves.
#Data has to be an array of all polymerisation measurements, corresponding to a 12 x 1080 array. (1h30 polymerization and point every 5s)
#G' ( Title = 'Storage Modulus') or G" (Title = 'Loss Modulus') as a function of time.
#Curve and points of given color c, given shape s, and a given label l 
def plotPolymerisation_all(data,Title:str,l:str,c:str,s:str):
    time_pol = np.array(data[0]['Time'].str.replace(',', ''), dtype='float64') ## adding .str.replace(',', '') removed the commas
    
    for i in range (np.size(data,0)):
        pl.plot(time_pol, np.array(data[i][Title], dtype='float64'), color = c,marker=s, ms=1,linewidth = 1.5)
        
    if Title == 'Storage Modulus':
        pl.xlabel(r'$t$ $(s)$', fontsize=30)
        pl.tight_layout()
        pl.ylabel(r'$G^{\prime}$ $(Pa)$', fontsize=30)
        pl.ylim (0,150)
       
    elif Title == 'Loss Modulus' :
        pl.ylabel(r'$G^{\prime\prime}$ $(Pa)$', fontsize=30) 
        pl.ylim (0,30)
        
    for spine in pl.gca().spines.values():
        spine.set_linewidth(1.5)
        
    pl.xticks(fontsize=24)
    pl.yticks(fontsize=24)
    pl.legend(loc='upper left',fontsize=20, frameon=False)
    pl.tick_params(axis='both', which='major', length=8, width=2)
    pl.xlim(0,6000)  
    
    
# Plots 
#Returns the final value of G' of G'' at the end of polymerization. Data has to be an array of all polymerisation measurements, corresponding to a 12 x 1080 array. (1h30 polymerization and point every 5s)
#G' ( Title = 'Storage Modulus') or G" (Title = 'Loss Modulus')
#Curve and points of given color c, given shape s, and a given label l 
def finalPol(data,Title:str):
    time_pol = np.array(data[0]['Time'].str.replace(',', ''), dtype='float64') ## adding .str.replace(',', '') removed the commas
    # if np.size(data,0)==1:
    #     pl.plot(time_pol, np.array(data[0][Title],dtype='float64'), ls='-', color = c, label=l,marker=s, ms=1,linewidth=2)
    
    data_avg = average(data,Title)
    final_value = data_avg[len(data_avg)-1]
    #final value of the Storage of teh Loss modulus 
    return final_value
      
# Plots 
#Plot polymerization normalized curve from data1 (mean and SD). Normalized data by the average final G' or G" value of data2. Data has to be an array of all polymerisation measurements, corresponding to a 12 x 1080 array. (1h30 polymerization and point every 5s)
#G' ( Title = 'Storage Modulus') or G" (Title = 'Loss Modulus') as a function of time.
#Curve and points of given color c, given shape s, and a given label l 
def plotPolymerisation_norm(data1,data2,Title:str,l:str,c:str,s:str):
    time_pol = np.array(data1[0]['Time'].str.replace(',', ''), dtype='float64') ## adding .str.replace(',', '') removed the commas
    # if np.size(data,0)==1:
    #     pl.plot(time_pol, np.array(data[0][Title],dtype='float64'), ls='-', color = c, label=l,marker=s, ms=1,linewidth=2)
    
    data_avg = average(data1,Title)
    data_SD = standard_deviation(data1,Title)
    data_avg_norm = data_avg/finalPol(data2, Title)
    data_SD_norm = data_SD/finalPol(data2, Title)
    
    pl.plot(time_pol, data_avg_norm, ls='-', color = c, label=l,marker=s, ms=1, linewidth = 3)
    pl.errorbar(time_pol, data_avg_norm, ls = 'None', yerr = data_SD_norm, ecolor=c,elinewidth=0.05)
    
    pl.xlabel(r'$t$ $(s)$', fontsize = 30)
    if Title == 'Storage Modulus':
        pl.ylabel(r'$G^{\prime}$ / $G^{\prime}_{final}$ (-)', fontsize=30)
        pl.ylim (0,1.2)
    elif Title == 'Loss Modulus' :
        pl.ylabel(r'$G^{\prime\prime}$/$G^{\prime\prime}_{final}$ (-)', fontsize = 30)
        pl.ylim (0,1.2)

    for spine in pl.gca().spines.values():
            spine.set_linewidth(1.5)
        
        
    pl.tick_params(axis='both', which='major', length=8, width=2)
    pl.xticks(fontsize=24)
    pl.yticks(fontsize=24)
        
    
    pl.legend(loc='lower right',fontsize=20, frameon=False)
    # pl.tight_layout()
    pl.xlim(0,6000)    

 
def plotPolymerisation_normpeak(data, Title: str, l: str, c: str, s: str):
    # adding .str.replace(',', '') removed the commas
    time_pol = np.array(data[0]['Time'].str.replace(',', ''), dtype='float64')
    # if np.size(data,0)==1:
    #     pl.plot(time_pol, np.array(data[0][Title],dtype='float64'), ls='-', color = c, label=l,marker=s, ms=1,linewidth=2)
    data_norm = []
    for i in range(np.size(data, 0)):
        data_norm.append(np.array(
            data[i][Title], dtype='float64')/max(np.array(data[i][Title], dtype='float64')))
        # Renormalizing each polymerization curve bu the maximal G' or G" value
    data_avg_norm = np.average(data_norm, axis=0)
    data_SD_norm = np.std(data_norm, axis=0)

    pl.plot(time_pol, data_avg_norm, ls='-', color=c,
            label=l, marker=s, ms=1, linewidth=3)
    pl.errorbar(time_pol, data_avg_norm, ls='None',
                yerr=data_SD_norm, ecolor=c, elinewidth=0.05)
    
    pl.xticks(fontsize=24)
    pl.yticks(fontsize=24)
    pl.xlabel(r'$t$ $(s)$',fontsize=30)
    if Title == 'Storage Modulus':
        pl.ylabel(r'$G^{\prime}$/$G^{\prime}_{\text{max}}$(-)',fontsize=30)
        pl.ylim(0, 1.2)
    elif Title == 'Loss Modulus':
        pl.ylabel(r'$Normalized G^{\prime\prime}$',fontsize=30)
        pl.ylim(0, 1.2)
        
    for spine in pl.gca().spines.values():
                spine.set_linewidth(1.5)
                
    pl.legend(loc='lower right',fontsize=20, frameon=False)
    # pl.tight_layout()
    pl.xlim(0, 6000)
    
# Plot frequency sweep from data. G' ( Title = 'Storage Modulus') or G" (Title = 'Loss Modulus') as a function of frequency.
# Here data plotted until the 6th point corrrsponding to 1 Hz. 
# Curve and points of given color c, given shape s, and a given label l

# def plotFreqSweep(data, Title: str, l: str, c: str, s: str):
#     data_avg = average(data, Title)
#     data_SD = standard_deviation(data, Title)
#     freq = np.array(data[0]['Frequency'], dtype='float64')
    
#     pl.plot(freq[0:6], data_avg[0:6], ls='-', color=c, label=l, marker=s, ms=6, linewidth=3)
#     pl.errorbar(freq[0:6], data_avg[0:6], ls='None', yerr=data_SD[0:6], ecolor=c, elinewidth=0.8) 
#     pl.loglog()
#     pl.xlabel(r'$f$ $(Hz)$', fontsize=30)
    
#     if Title == 'Storage Modulus':
#         pl.ylabel(r'$G^{\prime}$ $(Pa)$', fontsize=30)
#     elif Title == 'Loss Modulus':
#         pl.ylabel(r'$G^{\prime\prime}$ $(Pa)$', fontsize=30)
        
#     for spine in pl.gca().spines.values():
#             spine.set_linewidth(1.5)
            
    
#     # Ensure legend is shown only once
#     pl.legend(loc='lower right', fontsize=20, frameon=False)
    
#     pl.xticks(fontsize=24)
#     pl.yticks(fontsize=24)
#     pl.tick_params(axis='both', which='major', length=8, width=2)  # Make ticks longer and thicker
#     pl.tick_params(axis='both', which='minor', length=6, width=2)  # Adjust minor ticks

#     # pl.tight_layout()
#     pl.ylim(1, 150)

def plotFreqSweep(data, Title: str, l: str, c: str, s: str):
    data_avg = average(data, Title)
    data_SD = standard_deviation(data, Title)
    freq = np.array(data[0]['Frequency'], dtype='float64')

    marker_style = 'v' if Title == 'Loss Modulus' else s

    pl.plot(freq[0:6], data_avg[0:6], ls='-', color=c, marker=marker_style, ms=8, linewidth=3)
    pl.errorbar(freq[0:6], data_avg[0:6], ls='None', yerr=data_SD[0:6], ecolor=c, elinewidth=0.8)
    pl.loglog()
    pl.xlabel(r'$f$ $(Hz)$', fontsize=30)

    pl.ylabel(r'$G^{\prime}$, $G^{\prime\prime}$ $(Pa)$', fontsize=30)

    # Show legend with only the colored line for Storage Modulus
    if Title == 'Storage Modulus':
        pl.plot([], [], ls='-', color=c, label=l, linewidth=3)
        pl.legend(loc='lower right', fontsize=20, frameon=False)

    for spine in pl.gca().spines.values():
        spine.set_linewidth(1.5)

    pl.xticks(fontsize=24)
    pl.yticks(fontsize=24)
    pl.tick_params(axis='both', which='major', length=8, width=2)  # Make ticks longer and thicker
    pl.tick_params(axis='both', which='minor', length=6, width=2)  # Adjust minor ticks

    # pl.tight_layout()
    pl.ylim(1, 150)



    
##THIS IS THE OLD DIFF MOD PLOT. KEEP IT TO COMPARE WITH THE TRUNCATED ONE  
#Plot differential modulus from data as a function of the stress (or strain). Mean (+/- SD).
#Curve and points of given color c, given shape s, and a given label l    
# def plotDiffMod(data, l: str, c: str, s: str):
#     all_diffMod = []
#     all_stress = []
#     all_strain = []
    
#     for dataset in data:  # Assuming data is a list of dictionaries
#         strain = np.array(dataset['Strain'], dtype='float64')
#         stress = np.array(dataset['Shear Stress'], dtype='float64')

#         # Calculate differential modulus for each dataset
#         diffMod = np.gradient(stress, strain / 100.0)
        
#         all_diffMod.append(diffMod)
#         all_stress.append(stress)
#         all_strain.append(strain)

#     # Convert lists to arrays for easy averaging
#     all_diffMod = np.array(all_diffMod)
#     all_stress = np.array(all_stress)
#     all_strain = np.array(all_strain)

#     # Calculate average and standard deviation across datasets
#     avg_diffMod = np.mean(all_diffMod, axis=0)
#     diffMod_SD = np.std(all_diffMod, axis=0)


#     # Plot the average differential modulus
#     pl.plot(np.mean(all_stress, axis=0), avg_diffMod, ls='-', color=c, label=l, marker=s, ms=6, linewidth=3)

#     # Plot the standard deviation as error bars
#     # pl.errorbar(np.mean(all_stress, axis=0), avg_diffMod, yerr=diffMod_SD, fmt='none', ecolor=c, elinewidth=0.8)
#     # pl.errorbar(np.mean(all_strain, axis=0), avg_diffMod, yerr=diffMod_SD, fmt='none', ecolor=c, elinewidth=0.8)

#     pl.xlabel(r'$\sigma$ $(Pa)$', fontsize = 24) 
#     pl.ylabel(r'$K$ $(Pa)$',fontsize = 24) 
#     pl.loglog()
#     pl.xticks(fontsize=20)
#     pl.yticks(fontsize=20)
#     pl.legend(loc='upper left',fontsize=16, frameon=False)

#     pl.tick_params(axis='both', which='major', length=7, width=1)  # Make ticks longer and thicker
#     pl.tick_params(axis='both', which='minor', length=5, width=1)
#     pl.xlim(0.1, 100)
#     pl.ylim(10, 300)
    
    
def plotDiffMod(data, l: str, c: str, s: str):
    all_diffMod = []
    all_stress = []
    
    # Calculate the differential modulus (K) for each dataset
    for dataset in data:  
        strain = np.array(dataset['Strain'], dtype='float64')
        stress = np.array(dataset['Shear Stress'], dtype='float64')

        # Compute differential modulus (K)
        diffMod = np.gradient(stress, strain / 100.0)
        
        all_diffMod.append(diffMod)
        all_stress.append(stress)

    # Convert lists to arrays for averaging
    all_diffMod = np.array(all_diffMod)
    all_stress = np.array(all_stress)

    # Ensure no NaN values in the datasets
    if np.any(np.isnan(all_diffMod)) or np.any(np.isnan(all_stress)):
        print("Warning: NaN values detected in the data.")
        all_diffMod = np.nan_to_num(all_diffMod)
        all_stress = np.nan_to_num(all_stress)

    # Compute average differential modulus and stress
    avg_diffMod = np.mean(all_diffMod, axis=0)
    avg_stress = np.mean(all_stress, axis=0)
    
    # Compute standard deviation of stress and differential modulus
    std_diffMod = np.std(all_diffMod, axis=0)
    std_stress = np.std(all_stress, axis=0)

    # Search for the peak starting from 1 Pa onwards
    # Find the first index where stress is greater than 1 Pa
    start_index = np.argmax(avg_stress > 1)

    # Ensure the peak index is found in the region of interest
    peak_index_avg = np.argmax(avg_diffMod[start_index:]) + start_index

  
   # Truncate the data at the peak index for the average curve
    avg_diffMod_truncated = avg_diffMod[:peak_index_avg + 1]
    avg_stress_truncated = avg_stress[:peak_index_avg + 1]
    std_diffMod_truncated = std_diffMod[:peak_index_avg + 1]
    std_stress_truncated = std_stress[:peak_index_avg + 1]

    # Plot only the average curve up to the peak
    pl.plot(avg_stress_truncated, avg_diffMod_truncated, ls='-', color=c, label=l, marker=s, ms=6, linewidth=3)
    
    # Plot the standard deviation as a shaded region around the average curve
    pl.fill_between(avg_stress_truncated, avg_diffMod_truncated - std_diffMod_truncated, avg_diffMod_truncated + std_diffMod_truncated, color=c, alpha=0.1)

     
# Draw the edges with a darker color
    pl.plot(avg_stress_truncated, avg_diffMod_truncated - std_diffMod_truncated, color=c, alpha=0.2, linewidth=1.5)
    pl.plot(avg_stress_truncated, avg_diffMod_truncated + std_diffMod_truncated, color=c, alpha=0.5, linewidth=1.5)
    
    for spine in pl.gca().spines.values():
        spine.set_linewidth(1.5)
        
    pl.xlabel(r'$\sigma$ $(Pa)$', fontsize=30)
    pl.ylabel(r'$K$ $(Pa)$', fontsize=30)
    pl.loglog()
    pl.xticks(fontsize=24)
    pl.yticks(fontsize=24)
    pl.legend(loc='lower right', fontsize=20, frameon=False)
    pl.tick_params(axis='both', which='major', length=8, width=2)
    pl.tick_params(axis='both', which='minor', length=6, width=2)
    pl.xlim(0.1, 100)
    pl.ylim(10, 300)


 
##This is the new function, with truncation at the rupture point    
def plotDiffMod_all(data, l: str, c: str, s: str):
    all_diffMod = []
    all_stress = []
    all_strain = []
    
    for dataset in data:  # Assuming data is a list of dictionaries
        strain = np.array(dataset['Strain'], dtype='float64')
        stress = np.array(dataset['Shear Stress'], dtype='float64')

        # Calculate differential modulus for each dataset
        diffMod = np.gradient(stress, strain / 100.0)
        
        all_diffMod.append(diffMod)
        all_stress.append(stress)
        all_strain.append(strain)

    # Convert lists to arrays for easy averaging
    all_diffMod = np.array(all_diffMod)
    all_stress = np.array(all_stress)
    all_strain = np.array(all_strain)
    
    fig, ax = pl.subplots(figsize=(8, 6)) 
    
    for i in range(np.size(data, 0)):
        # Search for the peak starting from 1 Pa onwards
        start_index = np.argmax(all_stress[i] > 1)
        peak_index = np.argmax(all_diffMod[i][start_index:]) + start_index

        # Truncate the individual dataset at the peak
        truncated_diffMod = all_diffMod[i][:peak_index + 1]
        truncated_stress = all_stress[i][:peak_index + 1]

        # Plot the individual curve up to the peak
        pl.plot(truncated_stress, truncated_diffMod, ls='-', color=c, marker=s, ms=1.5, linewidth=1.5)
    for spine in pl.gca().spines.values():
        spine.set_linewidth(1.5)
        
    ax.set_xlabel(r'$\sigma$ $(Pa)$', fontsize=30)
    ax.set_ylabel(r'$K$ $(Pa)$', fontsize=30)
    ax.xaxis.set_tick_params(labelsize=24)
    ax.yaxis.set_tick_params(labelsize=24)
    pl.xticks(fontsize=24)
    pl.yticks(fontsize=24)
    pl.loglog()
    pl.legend(loc='upper left', fontsize=20, frameon=False)
    pl.tick_params(axis='both', which='major', length=8, width=2)
    pl.tick_params(axis='both', which='minor', length=6, width=2)
    pl.xlim(0.1, 100)
    pl.ylim(10, 1000)

# This function returns the average value of the differential modulus of one condition (data containing all the arrays of each sample of the condition)
# at 0.1 Pa stress. (point 11 of the array)   
def DiffMod_init (data):
    all_diffMod = []
    all_stress = []
    all_strain = []
    
    for dataset in data:  # Assuming data is a list of dictionaries
        strain = np.array(dataset['Strain'], dtype='float64')
        stress = np.array(dataset['Shear Stress'], dtype='float64')

        # Calculate differential modulus for each dataset
        diffMod = np.gradient(stress, strain / 100.0)
        
        all_diffMod.append(diffMod)
        all_stress.append(stress)
        all_strain.append(strain)

    # Convert lists to arrays for easy averaging
    all_diffMod = np.array(all_diffMod)
    all_stress = np.array(all_stress)
    all_strain = np.array(all_strain)

    # Calculate average and standard deviation across datasets
    avg_diffMod = np.mean(all_diffMod, axis=0)
    diffMod_init = avg_diffMod[11]
    return diffMod_init 

#Plot differential modulus from data1 as a function of the stress. Differential modulus of data1 normalized by the average value at the initial value 0.1 Pa (point 11) of data 2.
#Curve and points of given color c, given shape s, and a given label l    
def plotDiffMod_norm(data1, data2, l: str, c: str, s: str):
    all_diffMod = []
    all_stress = []
    all_strain = []
    
    for dataset in data1:  # Assuming data is a list of dictionaries
        strain = np.array(dataset['Strain'], dtype='float64')
        stress = np.array(dataset['Shear Stress'], dtype='float64')
        
        # Calculate differential modulus for each dataset
        diffMod = np.gradient(stress, strain / 100.0)
        
        all_diffMod.append(diffMod)
        all_stress.append(stress)
        all_strain.append(strain)

    for spine in pl.gca().spines.values():
        spine.set_linewidth(1.5)

    # Convert lists to arrays for easier computations
    all_diffMod = np.array(all_diffMod)
    all_stress = np.array(all_stress)
    all_strain = np.array(all_strain)

    # Ensure no NaN values in the datasets
    if np.any(np.isnan(all_diffMod)) or np.any(np.isnan(all_stress)):
        print("Warning: NaN values detected in the data.")
        all_diffMod = np.nan_to_num(all_diffMod)
        all_stress = np.nan_to_num(all_stress)

    # Compute average differential modulus and stress
    avg_diffMod = np.mean(all_diffMod, axis=0)
    avg_stress = np.mean(all_stress, axis=0)
    
    # Normalize by K(0.1 Pa) of data2
    avg_diffMod_norm = avg_diffMod / DiffMod_init(data2)
    diffMod_SD_norm = np.std(all_diffMod, axis=0) / DiffMod_init(data2)

    # Search for the peak starting from 1 Pa onwards
    start_index = np.argmax(avg_stress > 1)  # Find the first index where stress > 1 Pa

    # Ensure the peak index is found in the region of interest
    peak_index_avg = np.argmax(avg_diffMod_norm[start_index:]) + start_index

    # Truncate the data at the peak index
    avg_diffMod_norm_truncated = avg_diffMod_norm[:peak_index_avg + 1]
    avg_stress_truncated = avg_stress[:peak_index_avg + 1]
    diffMod_SD_norm_truncated = diffMod_SD_norm[:peak_index_avg + 1]

    # Plot the average differential modulus
    pl.plot(avg_stress_truncated, avg_diffMod_norm_truncated, ls='-', color=c, label=l, marker=s, ms=6, linewidth=3)
    
    # Plot the shaded region for the standard deviation
    pl.fill_between(avg_stress_truncated, avg_diffMod_norm_truncated - diffMod_SD_norm_truncated, avg_diffMod_norm_truncated + diffMod_SD_norm_truncated, color=c, alpha=0.1)
   

# Draw the edges with a darker color
    pl.plot(avg_stress_truncated, avg_diffMod_norm_truncated - diffMod_SD_norm_truncated, color=c, alpha=0.2, linewidth=1.5)
    pl.plot(avg_stress_truncated, avg_diffMod_norm_truncated + diffMod_SD_norm_truncated, color=c, alpha=0.5, linewidth=1.5)


    pl.xlabel(r'$\sigma$ $(Pa)$', fontsize=30) 
    pl.ylabel(r'$K$ / $K_{0}$ (-)', fontsize=30) 
    pl.tick_params(axis='both', which='major', length=8, width=2)
    pl.tick_params(axis='both', which='minor', length=6, width=2)
    pl.xticks(fontsize=24)
    pl.yticks(fontsize=24)
    
    pl.legend(loc='lower right', fontsize=20, frameon=False) ##for integrin plot ncol=2 and fontsize =15
    pl.loglog()
  
    pl.tight_layout()
    pl.xlim(0.1, 50)
    pl.ylim(0.1, 10)      


# #%%
# ##Plot of polymerization, Frequency sweep and stress ramp for CTL, 0.4, 4, 20% cells - G'
# pl.figure(figsize=(8, 6))
# plotPolymerisation(CTL_pol, 'Storage Modulus', 'CTL', 'grey', 'o')     
# plotPolymerisation(MDA_0pt1percent_pol_tot, 'Storage Modulus','0.4%', 'g', 'o')
# plotPolymerisation(MDA_1percent_pol , 'Storage Modulus','4%', 'b', 'o')
# plotPolymerisation(MDA_20percent_pol, "Storage Modulus", '20%', 'r', 'o') 
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Figure 1/MDAMB231 Rheology/Gprime-MDAMB231.png', dpi=300, bbox_inches='tight')
# pl.show()

# # %%
# ##Plot of polymerization, Frequency sweep and stress ramp for CTL, 0.4, 4, 20% cells - G''
# pl.figure(figsize=(8, 6))
# plotPolymerisation(CTL_pol, 'Loss Modulus', 'CTL', 'grey', 'o')
# plotPolymerisation(MDA_0pt1percent_pol_tot, 'Loss Modulus','0.4%', 'g', 'o')
# plotPolymerisation(MDA_1percent_pol , 'Loss Modulus','4%', 'b', 'o')
# plotPolymerisation(MDA_20percent_pol, "Loss Modulus", '20%', 'r', 'o') 
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/Gdoubleprime-MDAMB231.png', dpi=300, bbox_inches='tight')
# pl.show()

# # %%
# # # %%
# #Frequency sweep plots for CTL, 0.4%,4%,20% MDAMB231 cells - G' and G''
# pl.figure(figsize=(8, 6))
# plotFreqSweep(CTL_freqsweep, 'Storage Modulus', 'CTL', 'grey', 'o')
# plotFreqSweep(CTL_freqsweep, 'Loss Modulus', 'CTL', 'grey', 'o')
# plotFreqSweep(MDA_0pt1percent_freqsweep_tot, 'Storage Modulus','0.4%', 'g', 'o')
# plotFreqSweep(MDA_0pt1percent_freqsweep_tot , 'Loss Modulus','0.4%', 'g', 'o')
# plotFreqSweep(MDA_1percent_freqsweep , 'Storage Modulus','4%', 'b', 'o')
# plotFreqSweep(MDA_1percent_freqsweep , 'Loss Modulus','4%', 'b', 'o')
# plotFreqSweep(MDA_20percent_freqsweep, 'Storage Modulus', '20%','r', 'o')
# plotFreqSweep(MDA_20percent_freqsweep, 'Loss Modulus', '20%','r', 'o')

# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/Freqsweep-MDAMB231.png', dpi=300, bbox_inches='tight')
# pl.show()

# # %%

# ##Differential modulus K for CTL, 0.4%,4%,20% MDAMB231 cells 
# pl.figure(figsize=(8, 6))
# plotDiffMod(CTL_stressramp, 'CTL', 'grey', 'o')
# plotDiffMod(MDA_0pt1percent_stressramp_tot, '0.4%', 'g', 'o')
# plotDiffMod(MDA_1percent_stressramp ,'4%', 'b', 'o')
# plotDiffMod(MDA_20percent_stressramp, '20%','r', 'o') 
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Figure 1/MDAMB231 Rheology/K-MDAMB231.png', dpi=300, bbox_inches='tight')
# pl.show()


# #%%
# #Stress ramp for CTL, 0.4%,4%,20% MDAMB231 cells 
# pl.figure(figsize=(8, 6))
# plotStressRamp(CTL_stressramp, 'CTL', 'grey', 'o')
# plotStressRamp(MDA_0pt1percent_stressramp_tot , '0.4%', 'g', 'o')
# plotStressRamp(MDA_1percent_stressramp ,'4%', 'b', 'o')
# plotStressRamp(MDA_20percent_stressramp, '20%','r', 'o') 
# # pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/Stressramp-MDAMB231.png', dpi=300, bbox_inches='tight')
# pl.show()

# # %%


# # ##HDF PLOTS 

# # %%
# ##Plots with HDF 0.4, 4 and CTL with HDF media - G'
# pl.figure(figsize= (8,6))
# plotPolymerisation(CTL_HDF_pol, 'Storage Modulus', 'CTL', 'grey', 'o')
# plotPolymerisation(HDF_04percent_pol, 'Storage Modulus', ' 0.4%', 'g', 'o')
# plotPolymerisation(HDF_4percent_pol, 'Storage Modulus', '4% ', 'b', 'o')
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Figure 1/HDF Rheology/Gprime-HDF.png', dpi=300, bbox_inches='tight')
# pl.show()


# # %%
# #Plots with HDF 0.4, 4 and CTL with HDF media - G''
# pl.figure(figsize= (8,6))
# plotPolymerisation(CTL_HDF_pol, 'Loss Modulus', 'CTL', 'grey', 'o')
# plotPolymerisation(HDF_04percent_pol, 'Loss Modulus', ' 0.4%', 'g', 'o')
# plotPolymerisation(HDF_4percent_pol, 'Loss Modulus', '4% ', 'b', 'o')
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/Gdoubleprime-HDF.png', dpi=300, bbox_inches='tight')
# pl.show()

# # # %%
# ##Frequency sweep plots for CTL, 0.4%,4% HDF cells - G' and G''
# pl.figure(figsize= (8,6))
# plotFreqSweep(CTL_HDF_freqsweep, 'Storage Modulus', 'CTL', 'grey', 'o')
# plotFreqSweep(CTL_HDF_freqsweep, 'Loss Modulus', 'CTL', 'grey', 'o')
# plotFreqSweep(HDF_04percent_freqsweep, 'Storage Modulus', ' 0.4%', 'g', 'o')
# plotFreqSweep(HDF_04percent_freqsweep, 'Loss Modulus', ' 0.4%', 'g', 'o')
# plotFreqSweep(HDF_4percent_freqsweep, 'Storage Modulus', '4% ', 'b', 'o')
# plotFreqSweep(HDF_4percent_freqsweep, 'Loss Modulus', '4% ', 'b', 'o')


# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/Freqsweep-HDF.png', dpi=300, bbox_inches='tight')
# pl.show()
# # # # %%

# # %%
# #Differential modulus K for CTL, 0.4%,4% HDF cells 
# pl.figure(figsize= (8,6))
# plotDiffMod(CTL_HDF_stressramp, 'CTL', 'grey', 'o')
# plotDiffMod(HDF_04percent_stressramp, ' 0.4%', 'g', 'o')
# plotDiffMod(HDF_4percent_stressramp, '4%', 'b', 'o')
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Figure 1/HDF Rheology/K-HDF.png', dpi=300, bbox_inches='tight')
# pl.show()


# # %%
# # ##Stress ramp for CTL, 0.4%,4% HDF cells (for this one I will chage the axis of the stress ramp function to x= 60 and y = 90)
# pl.figure(figsize= (8,6))
# plotStressRamp(CTL_HDF_stressramp, 'CTL', 'grey', 'o')
# plotStressRamp(HDF_04percent_stressramp, ' 0.4%', 'g', 'o')
# plotStressRamp(HDF_4percent_stressramp, '4%', 'b', 'o')
# # pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/Stressramp-HDF.png', dpi=300, bbox_inches='tight')
# pl.show()
# # # %%

# # %%
# ##Supplementary Figures

# # %%
# ##No influence of the gap - CTL collagen and 20% combined with 0.32 excluded - G'(for different gaps the legend fontsize is 18)
# pl.figure(figsize= (8,6))
# plotPolymerisation(CTL_pol, 'Storage Modulus', 'CTL - 0.16 mm', 'grey', 'o')
# plotPolymerisation(CTL_g048_pol, 'Storage Modulus', 'CTL - 0.48 mm', 'purple', 'o')
# plotPolymerisation(MDA_20percent_pol, 'Storage Modulus', '20% - 0.16 mm','r', 'o')
# plotPolymerisation(MDA_10percent_g048_pol, 'Storage Modulus', '20% - 0.48 mm', 'brown', 'o')
# # pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/Different gaps CTL and 20percent.png', dpi=300, bbox_inches='tight')
# pl.show()
# # %%

# #No influence of the gap - CTL collagen and 20% combined - K 
# pl.figure(figsize= (8,6))
# plotDiffMod(CTL_stressramp, 'CTL - 0.16 mm', 'grey', 'o')
# plotDiffMod(CTL_g048_stressramp, 'CTL - 0.48 mm', 'purple', 'o')
# plotDiffMod(MDA_20percent_stressramp, '20% - 0.16 mm','r', 'o')
# plotDiffMod(MDA_10percent_g048_stressramp, '20% - 0.48 mm','brown', 'o')
# # pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/Different gaps CTL and 20percent - K .png', dpi=300, bbox_inches='tight')
# pl.show()

# # %%
# ##Static vs dynamic polymerization - CTL collagen and 20% -  G' from frequency sweeps
# pl.figure(figsize= (8,6))
# plotFreqSweep(CTL_freqsweep,'Storage Modulus','CTL','grey','o')
# plotFreqSweep(CTL_static_freqsweep,'Storage Modulus','CTL - static','purple','o')

# plotFreqSweep(MDA_20percent_freqsweep ,'Storage Modulus','20%','r','o')
# plotFreqSweep(MDA_10percent_static_freqsweep,'Storage Modulus','20% - static','brown','o')
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/Stat vs dyn polym - CTL and 20percent - Gprime.png', dpi=300, bbox_inches='tight')
# pl.show()

# # %%
# #Static vs dynamic polymerization - CTL collagen and 20% -  Stress ramp
# pl.figure(figsize= (8,6))
# plotStressRamp(CTL_stressramp,'CTL','grey','o') 
# plotStressRamp(CTL_static_stressramp,'CTL - static','purple','o')
# plotStressRamp(MDA_20percent_stressramp,'20%','r','o') 
# plotStressRamp(MDA_10percent_static_stressramp,'20% - static','brown','o')
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/Stat vs dyn polym - CTL and 20percent - Stressramp.png', dpi=300, bbox_inches='tight')
# pl.show()

# #%%
# pl.figure(figsize= (8,6))
# plotDiffMod(CTL_stressramp,'CTL','grey','o') 
# plotDiffMod(CTL_static_stressramp,'CTL - static','purple','o')
# plotDiffMod(MDA_20percent_stressramp,'20%','r','o') 
# plotDiffMod(MDA_10percent_static_stressramp,'20% - static','brown','o')
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/Stat vs dyn polym - CTL and 20percent - K.png', dpi=300, bbox_inches='tight')
# pl.show()

# #%%
# #Plot all individual curves for Polymerization curves -MDAMB231 - G'
# pl.figure(figsize= (8,6))
# plotPolymerisation_all(CTL_pol, 'Storage Modulus','CTL', 'grey', 'o')
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/SinglecurvesMDA - Gprime - CTL.png', dpi=300, bbox_inches='tight')
# pl.show()


# pl.figure(figsize= (8,6))
# plotPolymerisation_all(MDA_0pt1percent_pol_tot, 'Storage Modulus','0.4%', 'g', 'o')
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/SinglecurvesMDA - Gprime - 0.4.png', dpi=300, bbox_inches='tight')
# pl.show()


# pl.figure(figsize= (8,6))
# plotPolymerisation_all(MDA_1percent_pol, 'Storage Modulus','4%', 'b', 'o')
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/SinglecurvesMDA - Gprime - 4.png', dpi=300, bbox_inches='tight')
# pl.show()

# pl.figure(figsize= (8,6))
# plotPolymerisation_all(MDA_20percent_pol, 'Storage Modulus','20%', 'r', 'o')
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/SinglecurvesMDA - Gprime - 20.png', dpi=300, bbox_inches='tight')
# pl.show()

# # %%
# # %%
# #Plot all individual curves for Polymerization curves - HDF - G'
# pl.figure(figsize= (8,6))
# plotPolymerisation_all(CTL_HDF_pol, 'Storage Modulus','CTL', 'grey', 'o')
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/SinglecurvesHDF - Gprime - CTL.png', dpi=300, bbox_inches='tight')
# pl.show()


# pl.figure(figsize= (8,6))
# plotPolymerisation_all(HDF_04percent_pol, 'Storage Modulus','0.4%', 'g', 'o')
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/SinglecurvesHDF - Gprime - 0.4.png', dpi=300, bbox_inches='tight')
# pl.show()


# pl.figure(figsize= (8,6))
# plotPolymerisation_all(HDF_4percent_pol, 'Storage Modulus','4%', 'b', 'o')
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/SinglecurvesHDF - Gprime - 4.png', dpi=300, bbox_inches='tight')
# pl.show()

# # # %%

# # %%
# ##Plot all individual curves for Differential modulus curves - K - MDAMB231 
# pl.figure(figsize= (8,6))
# plotDiffMod_all(CTL_stressramp,'CTL', 'grey', 'o') 
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/SinglecurvesMDA - K - CTL.png', dpi=300, bbox_inches='tight')
# pl.show()

# pl.figure(figsize= (8,6))
# plotDiffMod_all(MDA_0pt1percent_stressramp_tot,'0.4%', 'g', 'o') 
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/SinglecurvesMDA - K - 0.4.png', dpi=300, bbox_inches='tight')
# pl.show()
 
# pl.figure(figsize= (8,6))
# plotDiffMod_all(MDA_1percent_stressramp,'4%', 'b', 'o') 
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/SinglecurvesMDA - K - 4.png', dpi=300, bbox_inches='tight')
# pl.show()

# pl.figure(figsize= (8,6))
# plotDiffMod_all(MDA_20percent_stressramp,'20%', 'r', 'o') 
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/SinglecurvesMDA - K - 20.png', dpi=300, bbox_inches='tight')
# pl.show()
 
# # %%
# # # %%

# ##Plot all individual curves for Differential modulus curves - K - HDF
# pl.figure(figsize= (8,6))
# plotDiffMod_all(CTL_HDF_stressramp,'CTL', 'grey', 'o') 
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/SinglecurvesHDF - K - CTL.png', dpi=300, bbox_inches='tight')
# pl.show()

# pl.figure(figsize= (8,6))
# plotDiffMod_all(HDF_04percent_stressramp,'0.4%', 'g', 'o') 
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/SinglecurvesHDF - K - 0.4.png', dpi=300, bbox_inches='tight')
# pl.show()
 
# pl.figure(figsize= (8,6))
# plotDiffMod_all(HDF_4percent_stressramp,'4%', 'b', 'o') 
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/SinglecurvesHDF - K - 4.png', dpi=300, bbox_inches='tight')
# pl.show()



#%%
#For onset time --> functions_onset_time code 

# Strain and stress at rupture plots --> code functions_rupture_strain_stress

#%%
#For standard rheometers with a bottom coverslip 
pl.figure(figsize= (8,6))
CTL_glass_pol = [CTL_glass_pol_020224]
CTL_glass_freqsweep = [CTL_glass_freqsweep_020224]
CTL_glass_stressramp = [CTL_glass_stressramp_020224] 


# plotPolymerisation(CTL_pol, 'Storage Modulus', 'CTL', 'k', 'o')     
pl.figure(figsize= (8,6))
plotPolymerisation_normpeak(CTL_pol, 'Storage Modulus','CTL', 'grey', 'o')
plotPolymerisation_normpeak(CTL_glass_pol, 'Storage Modulus', 'CTL - glass coverslip', 'k', 'o')
pl.legend(loc='best', fontsize=20, frameon=False)
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/CTL_glasscoverslip_polymerization.png', dpi=300, bbox_inches='tight')
pl.show()

pl.figure(figsize= (8,6))
plotDiffMod_norm(CTL_stressramp,CTL_stressramp,'CTL', 'grey', 'o') 
plotDiffMod_norm(CTL_glass_stressramp,CTL_glass_stressramp,'CTL - glass coverslip', 'k', 'o') 
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/CTL_glasscoverslipstressramp.png', dpi=300, bbox_inches='tight')
pl.show()



