# -*- coding: utf-8 -*-
"""
Created on Tue Sep 24 14:23:34 2024

@author: inagle
"""

import numpy as np
import matplotlib
import matplotlib.pyplot as pl
import pandas
import io

import codecs
from apMethods_ReadRheo_Datacollection import *

import matplotlib
import matplotlib.pyplot as pl
from scipy.ndimage import gaussian_filter1d

# Martin Krzywinski's 24-color, color-blind-friendly palette
color_blind_palette = {
    "orange": "#E69F00",  # Orange
    "skyblue":"#56B4E9",  # Sky Blue
    "g": "#009E73",  # Green
    "y":"#F0E442",  # Yellow
    "b": "#0072B2",  # Blue
    "darkorange":"#D55E00",  # Dark Orange
    "pink":"#CC79A7",  # Pink
    "grey":"#999999",  # Grey
    "lightyellow":"#F7CB4D",  # Light Yellow
    "lightblue":"#ADD8E6",  # Light Blue
    "lightgreen":"#90EE90",  # Light Green
    "lightpink":"#FFB6C1",  # Light Pink
    "darkred":"#A52A2A",  # Brown
    "purple":"#800080",  # Purple
    "olive":"#808000",  # Olive
    "orange":"#FFA500",  # Orange
    "steelblue":"#4682B4",  # Steel Blue
    "brown":"#8B4513",  # Saddle Brown
    "cadetblue": "#5F9EA0",  # Cadet Blue
    "r":"#CD5C5C"   # Indian Red
}


# Set default color cycle
def resolve_color(color):
    """Resolve shorthand color names to Tableau hex codes."""
    return color_blind_palette.get(color, color)  # Default to original if shorthand not found

# Override the default plot function to use custom color
original_plot = pl.plot

def patched_plot(*args, **kwargs):
    if 'color' in kwargs:
        kwargs['color'] = resolve_color(kwargs['color'])
    return original_plot(*args, **kwargs)

pl.plot = patched_plot 




#This function calculates the average for n samples. 
# It is first required to construct an array containing all the arrays of each sample.
# The function has as variables :
# 1) data that is an array containing the of n x l x c, with n the number of different samples to average,
# l the number or lines in the array, or number of measuring points, and c the number of columns. 
# 2) Title corresponds to the title of the column we would like to plot. 
# For example, Title = Storage Modulus or Title = Loss Modulus

        
def average (data,Title:str):
    # Title = Type + " Modulus"   
    #n number of samples averaged
    n = np.size(data,0)
    S = np.zeros(np.size(data,1))
    for i in range (n) :
        S = S + np.array(data[i][Title], dtype='float64')
    avg = S/n
    return avg 

def standard_deviation (data,Title:str):
    # Title = Type + " Modulus" 
    #n number of samples averaged
    n = np.size(data,0)
    #l number of lines or measuring points per sample
    l = np.size(data,1)
    Q = np.zeros(l)
    for i in range (n) :
        Q = Q + np.square(np.array(data[i][Title], dtype='float64')-average(data,Title))
    SD = np.sqrt(Q/n)
    return SD 

# Plots 
#Plot polymerization curve from data. Mean +/- SD. Data has to be an array of all polymerisation measurements, corresponding to a 12 x 1080 array. (1h30 polymerization and point every 5s)
#G' ( Title = 'Storage Modulus') or G" (Title = 'Loss Modulus') as a function of time.
#Curve and points of given color c, given shape s, and a given label l 
def plotPolymerisation(data,Title:str,l:str,c:str,s:str):
    time_pol = np.array(data[0]['Time'].str.replace(',', ''), dtype='float64') ## adding .str.replace(',', '') removed the commas
    # if np.size(data,0)==1:
    #     pl.plot(time_pol, np.array(data[0][Title],dtype='float64'), ls='-', color = c, label=l,marker=s, ms=1,linewidth=2)
    
    data_avg = average(data,Title)
    data_SD = standard_deviation(data,Title)
    pl.plot(time_pol, data_avg, ls='-', color = c, label=l,marker=s, ms=1, linewidth = 3)
    pl.errorbar(time_pol, data_avg, ls = 'None', yerr = data_SD, ecolor=c,elinewidth=0.05)
    
    pl.xlabel(r'$t$ $(s)$', fontsize=22)
    if Title == 'Storage Modulus':
        pl.ylabel(r'$G^{\prime}$ $(Pa)$', fontsize=22)
        pl.ylim (0,130)
    elif Title == 'Loss Modulus' :
        pl.ylabel(r'$G^{\prime\prime}$ $(Pa)$', fontsize=22) 
        pl.ylim (0,30)
        

    pl.xticks(fontsize=18)
    pl.yticks(fontsize=18)
    pl.legend(loc='lower right',fontsize=14, frameon=False)
    # pl.tight_layout()
    pl.xlim(0,6000)

    
# Plots 
#Plot polymerization curve from data. Plots all the individual curves.
#Data has to be an array of all polymerisation measurements, corresponding to a 12 x 1080 array. (1h30 polymerization and point every 5s)
#G' ( Title = 'Storage Modulus') or G" (Title = 'Loss Modulus') as a function of time.
#Curve and points of given color c, given shape s, and a given label l 
def plotPolymerisation_all(data,Title:str,l:str,c:str,s:str):
    time_pol = np.array(data[0]['Time'].str.replace(',', ''), dtype='float64') ## adding .str.replace(',', '') removed the commas
    
    for i in range (np.size(data,0)):
        pl.plot(time_pol, gaussian_filter1d(np.array(data[i][Title], dtype='float64'),20), color = c,marker=s, ms=0,linewidth = 5)
        #Smoothing of the G' values with a gaussian filter and sigma = 2
    if Title == 'Storage Modulus':
        pl.xlabel(r'Time (s)', fontsize=26)
        pl.tight_layout()
        pl.ylabel(r'$G^{\prime}$ $(Pa)$', fontsize=26)
        pl.ylim (0,90)
       
    elif Title == 'Loss Modulus' :
        pl.ylabel(r'$G^{\prime\prime}$ $(Pa)$', fontsize=26) 
        pl.ylim (0,30)
    
    pl.xticks(fontsize=22)
    pl.yticks(fontsize=22)
    pl.legend(loc='lower right',fontsize=14, frameon=False)
    pl.legend(loc='lower right', fontsize=14, frameon=False)
    pl.xlim(0,6000)  
      
    

    
#Plot frequency sweep from data. G' ( Title = 'Storage Modulus') or G" (Title = 'Loss Modulus') as a function of frequency.
# Here data plotted until the 6th point corrrsponding to 1 Hz. 
# Curve and points of given color c, given shape s, and a given label l

def plotFreqSweep (data,Title:str,l:str,c:str, s:str):
    data_avg = average(data,Title)
    data_SD = standard_deviation(data,Title)
    freq = np.array(data[0]['Frequency'],dtype='float64')
    pl.plot(freq[0:6], data_avg[0:6],ls='-', color = c, label=l,marker=s, ms=6, linewidth = 3)
    pl.errorbar(freq[0:6], data_avg[0:6], ls = 'None', yerr = data_SD[0:6], ecolor=c,elinewidth=0.8) 
    pl.loglog()
    pl.xlabel(r'Frequency (Hz)', fontsize=22)
    if Title == 'Storage Modulus':
        pl.ylabel(r'$G^{\prime}$ $(Pa)$', fontsize=22)
    elif Title == 'Loss Modulus' :
        pl.ylabel(r'$G^{\prime}$, $G^{\prime\prime}$ $(Pa)$', fontsize=22)
    pl.legend(loc='lower right', fontsize=14, frameon=False)
    
    pl.xticks(fontsize=18)
    pl.yticks(fontsize=18)
    # pl.tight_layout()
    pl.ylim(10,120)
    
  

#Plot stress ramp from data. Strain as a function of the applied stress.
#Curve and points of given color c, given shape s, and a given label l    
def plotStressRamp (data,l:str,c:str, s:str):
    data_avg = average(data,'Strain')
    data_SD = standard_deviation(data,'Strain')
    stress = np.array(data[0]['Shear Stress'],dtype='float64')
    pl.plot(data_avg[0:40],stress[0:40],ls='-', color = c, label=l,marker=s, ms=6, linewidth = 3)
    pl.errorbar(data_avg[0:40],stress[0:40], ls = 'None', yerr = data_SD[0:40], ecolor=c,elinewidth=0.8) 
    pl.xlabel(r'Strain (%)')
    pl.ylabel(r'Shear Stress (Pa)')
    pl.legend(loc='best')
    pl.tight_layout()
    pl.xlim(0,25)
    pl.ylim(0,70)
    pl.xlabel(r'Strain (%)', fontsize = 22)
    pl.ylabel(r'Shear Stress (Pa)', fontsize = 22)
    pl.legend(loc='upper right', fontsize=14, frameon=False)
    
    pl.xticks(fontsize=18)
    pl.yticks(fontsize=18)
    # pl.tight_layout()
    pl.xlim(0,30)
    pl.ylim(0,30)
    
#Plot differential modulus from data as a function of the stress (or strain). Mean (+/- SD).
#Curve and points of given color c, given shape s, and a given label l    
def plotDiffMod(data, l: str, c: str, s: str):
    all_diffMod = []
    all_stress = []
    all_strain = []
    
    for dataset in data:  # Assuming data is a list of dictionaries
        strain = np.array(dataset['Strain'], dtype='float64')
        stress = np.array(dataset['Shear Stress'], dtype='float64')

        # Calculate differential modulus for each dataset
        diffMod = np.gradient(stress, strain / 100.0)
        
        all_diffMod.append(diffMod)
        all_stress.append(stress)
        all_strain.append(strain)

    # Convert lists to arrays for easy averaging
    all_diffMod = np.array(all_diffMod)
    all_stress = np.array(all_stress)
    all_strain = np.array(all_strain)

    # Calculate average and standard deviation across datasets
    avg_diffMod = np.mean(all_diffMod, axis=0)
    diffMod_SD = np.std(all_diffMod, axis=0)

    # fig, ax = pl.subplots(figsize=(8, 6)) 
    # Plot the average differential modulus
    pl.plot(np.mean(all_stress, axis=0), avg_diffMod, ls='-', color=c, label=l, marker=s, ms=6, linewidth=3)
    # pl.plot(np.mean(all_strain, axis=0), avg_diffMod, ls='-', color=c, label=l, marker=s, ms=5, linewidth=1)

    # Plot the standard deviation as error bars
    # pl.errorbar(np.mean(all_stress, axis=0), avg_diffMod, yerr=diffMod_SD, fmt='none', ecolor=c, elinewidth=0.8)
    # pl.errorbar(np.mean(all_strain, axis=0), avg_diffMod, yerr=diffMod_SD, fmt='none', ecolor=c, elinewidth=0.8)

    pl.xlabel(r'Stress (Pa)', fontsize=22) 
    pl.ylabel(r'K (Pa)',fontsize=22) 
    pl.loglog()
    pl.xticks(fontsize=18)
    pl.yticks(fontsize=18)
    pl.legend(loc='upper left',fontsize=14, frameon=False)
    pl.tight_layout()
    pl.xlim(0.1, 100)
    pl.ylim(10, 300)
    
    

#Plot differential modulus from data as a function of the stress (or strain). Plot all individual curves.
#Curve and points of given color c, given shape s, and a given label l 
def plotDiffMod_all(data, l: str, c: str, s: str):
    all_diffMod = []
    all_stress = []
    all_strain = []
    
    for dataset in data:  # Assuming data is a list of dictionaries
        strain = np.array(dataset['Strain'], dtype='float64')
        stress = np.array(dataset['Shear Stress'], dtype='float64')

        # Calculate differential modulus for each dataset
        diffMod = np.gradient(stress, strain / 100.0)
        
        all_diffMod.append(diffMod)
        all_stress.append(stress)
        all_strain.append(strain)

    # Convert lists to arrays for easy averaging
    all_diffMod = np.array(all_diffMod)
    all_stress = np.array(all_stress)
    all_strain = np.array(all_strain)
    
    fig, ax = pl.subplots(figsize=(8, 6)) 
    for i in range (np.size(data,0)):
        pl.plot(all_stress[i],all_diffMod[i], ls='-', color=c, marker=s, ms=2, linewidth=1)


    ax.set_xlabel(r'Stress (Pa)',fontsize=22)   # Changing x-axis title font size
    ax.set_ylabel(r'K (Pa)',fontsize=22)    # Changing y-axis title font size
    ax.xaxis.set_tick_params(labelsize=18)   # Changing x-axis ticks font size
    ax.yaxis.set_tick_params(labelsize=18)   # Changing y-axis ticks font size
    # pl.xlabel(r'Stress (Pa)') 
    # pl.ylabel(r'K (Pa)') 
    pl.loglog()
    # pl.legend(loc='best')
    pl.tight_layout()
    pl.xlim(0.1, 100)
    pl.ylim(10, 1000)
    
    



##Plot of polymerization for one example MDA-MB-231 4% in the rheoconfoncal - Experiment from the 18/12/2024
MDA_4percent_pol_18122024_array=[MDA_4percent_pol_18122024]
pl.figure(figsize= (12,6))
plotPolymerisation(MDA_4percent_pol_18122024_array, 'Storage Modulus','CTL', 'b', 'o')
pl.savefig('C:/Users/inagle/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Figure 2/Polymerization_curve_4percentMDA_18122024_smoothened.png', dpi=300, bbox_inches='tight')


## For the polymerization curve over time for the rheoconfocal 4% samples (N=4) cf Supp Figure 2 for the code 