# -*- coding: utf-8 -*-
"""
Created on Thu Jan 30 11:26:14 2025

@author: mtavasso
"""

import numpy as np
import matplotlib
import matplotlib.pyplot as pl
import pandas
import io
from scipy.ndimage import gaussian_filter1d
import seaborn as sns
from scipy.stats import mannwhitneyu

import codecs
from apMethods_ReadRheo_Datacollection_mt import *



# Martin Krzywinski's 24-color, color-blind-friendly palette
color_blind_palette = {
    "orange": "#E69F00",  # Orange
    "skyblue":"#56B4E9",  # Sky Blue
    "g": "#009E73",  # Green
    "y":"#F0E442",  # Yellow
    "b": "#0072B2",  # Blue
    "darkorange":"#D55E00",  # Dark Orange
    "pink":"#CC79A7",  # Pink
    "grey":"#999999",  # Grey
    "lightyellow":"#F7CB4D",  # Light Yellow
    "lightblue":"#ADD8E6",  # Light Blue
    "lightgreen":"#90EE90",  # Light Green
    "lightpink":"#FFB6C1",  # Light Pink
    "darkred":"#A52A2A",  # Brown
    "purple":"#800080",  # Purple
    "olive":"#808000",  # Olive
    "orange":"#FFA500",  # Orange
    "steelblue":"#4682B4",  # Steel Blue
    "brown":"#8B4513",  # Saddle Brown
    "cadetblue": "#5F9EA0",  # Cadet Blue
    "r":"#CD5C5C"   # Indian Red
}


# Set default color cycle
def resolve_color(color):
    """Resolve shorthand color names to Tableau hex codes."""
    return color_blind_palette.get(color, color)  # Default to original if shorthand not found

# Override the default plot function to use custom color
original_plot = pl.plot

def patched_plot(*args, **kwargs):
    if 'color' in kwargs:
        kwargs['color'] = resolve_color(kwargs['color'])
    return original_plot(*args, **kwargs)

pl.plot = patched_plot 




#This function calculates the average for n samples. 
# It is first required to construct an array containing all the arrays of each sample.
# The function has as variables :
# 1) data that is an array containing the of n x l x c, with n the number of different samples to average,
# l the number or lines in the array, or number of measuring points, and c the number of columns. 
# 2) Title corresponds to the title of the column we would like to plot. 
# For example, Title = Storage Modulus or Title = Loss Modulus

        
def average (data,Title:str):
    # Title = Type + " Modulus"   
    #n number of samples averaged
    n = np.size(data,0)
    S = np.zeros(np.size(data,1))
    for i in range (n) :
        S = S + np.array(data[i][Title], dtype='float64')
    avg = S/n
    return avg 

def standard_deviation (data,Title:str):
    # Title = Type + " Modulus" 
    #n number of samples averaged
    n = np.size(data,0)
    #l number of lines or measuring points per sample
    l = np.size(data,1)
    Q = np.zeros(l)
    for i in range (n) :
        Q = Q + np.square(np.array(data[i][Title], dtype='float64')-average(data,Title))
    SD = np.sqrt(Q/n)
    return SD 

# Plots 
#Plot polymerization curve from data. Mean +/- SD. Data has to be an array of all polymerisation measurements, corresponding to a 12 x 1080 array. (1h30 polymerization and point every 5s)
#G' ( Title = 'Storage Modulus') or G" (Title = 'Loss Modulus') as a function of time.
#Curve and points of given color c, given shape s, and a given label l 
def plotPolymerisation(data,Title:str,l:str,c:str,s:str):
    time_pol = np.array(data[0]['Time'].str.replace(',', ''), dtype='float64') ## adding .str.replace(',', '') removed the commas
    # if np.size(data,0)==1:
    #     pl.plot(time_pol, np.array(data[0][Title],dtype='float64'), ls='-', color = c, label=l,marker=s, ms=1,linewidth=2)
    
    data_avg = average(data,Title)
    data_SD = standard_deviation(data,Title)
    pl.plot(time_pol, data_avg, ls='-', color = c, label=l,marker=s, ms=1, linewidth = 3)
    pl.errorbar(time_pol, data_avg, ls = 'None', yerr = data_SD, ecolor=c,elinewidth=0.05)
    
    pl.xlabel(r't (s)', fontsize=30)
    if Title == 'Storage Modulus':
        pl.ylabel(r'$G^{\prime}$ $(Pa)$', fontsize=30)
        pl.ylim (0,150)
    elif Title == 'Loss Modulus' :
        pl.ylabel(r'$G^{\prime\prime}$ $(Pa)$', fontsize=30) 
        pl.ylim (0,30)
        
    for spine in pl.gca().spines.values():
        spine.set_linewidth(1.5) 
        
    pl.xticks(fontsize=24)
    pl.yticks(fontsize=24)
    pl.tick_params(axis='both', which='major', length=8, width=2)
    pl.legend(loc='upper left',fontsize=20, frameon=False) ##add ncol = 2 for MDAMB
    # pl.tight_layout()
    pl.xlim(0,6000)
    

    
# Plots 
#Plot polymerization curve from data. Plots all the individual curves.
#Data has to be an array of all polymerisation measurements, corresponding to a 12 x 1080 array. (1h30 polymerization and point every 5s)
#G' ( Title = 'Storage Modulus') or G" (Title = 'Loss Modulus') as a function of time.
#Curve and points of given color c, given shape s, and a given label l 
def plotPolymerisation_all(data,Title:str,l:str,c:str,s:str):
    time_pol = np.array(data[0]['Time'].str.replace(',', ''), dtype='float64') ## adding .str.replace(',', '') removed the commas
    
    for i in range (np.size(data,0)):
        pl.plot(time_pol, np.array(data[i][Title], dtype='float64'), color = c,marker=s, ms=1,linewidth = 1.5)
        
    if Title == 'Storage Modulus':
        pl.xlabel(r'$t$ $(s)$', fontsize=30)
        pl.tight_layout()
        pl.ylabel(r'$G^{\prime}$ $(Pa)$', fontsize=30)
        pl.ylim (0,150)
       
    elif Title == 'Loss Modulus' :
        pl.ylabel(r'$G^{\prime\prime}$ $(Pa)$', fontsize=30) 
        pl.ylim (0,30)
        
    for spine in pl.gca().spines.values():
        spine.set_linewidth(1.5)
        
    pl.xticks(fontsize=24)
    pl.yticks(fontsize=24)
    pl.legend(loc='upper left',fontsize=20, frameon=False)
    pl.tick_params(axis='both', which='major', length=8, width=2)
    pl.xlim(0,6000)   
      
    
    
# Plots 
# Returns the final value of G' of G'' at the end of polymerization. Data has to be an array of all polymerisation measurements, corresponding to a 12 x 1080 array. (1h30 polymerization and point every 5s)
# G' ( Title = 'Storage Modulus') or G" (Title = 'Loss Modulus')
# Curve and points of given color c, given shape s, and a given label l 
def finalPol(data,Title:str):
    time_pol = np.array(data[0]['Time'].str.replace(',', ''), dtype='float64') ## adding .str.replace(',', '') removed the commas
    # if np.size(data,0)==1:
    #     pl.plot(time_pol, np.array(data[0][Title],dtype='float64'), ls='-', color = c, label=l,marker=s, ms=1,linewidth=2)
    
    data_avg = average(data,Title)
    final_value = data_avg[len(data_avg)-1]
    #final value of the Storage of teh Loss modulus 
    return final_value


# Plots 
#Plot polymerization normalized curve from data1 (mean and SD). Normalized data by the average final G' or G" value of data2. Data has to be an array of all polymerisation measurements, corresponding to a 12 x 1080 array. (1h30 polymerization and point every 5s)
#G' ( Title = 'Storage Modulus') or G" (Title = 'Loss Modulus') as a function of time.
#Curve and points of given color c, given shape s, and a given label l 
def plotPolymerisation_norm(data1,data2,Title:str,l:str,c:str,s:str):
    time_pol = np.array(data1[0]['Time'].str.replace(',', ''), dtype='float64') ## adding .str.replace(',', '') removed the commas
    # if np.size(data,0)==1:
    #     pl.plot(time_pol, np.array(data[0][Title],dtype='float64'), ls='-', color = c, label=l,marker=s, ms=1,linewidth=2)
    
    data_avg = average(data1,Title)
    data_SD = standard_deviation(data1,Title)
    data_avg_norm = data_avg/finalPol(data2, Title)
    data_SD_norm = data_SD/finalPol(data2, Title)
    
    pl.plot(time_pol, data_avg_norm, ls='-', color = c, label=l,marker=s, ms=1, linewidth = 3)
    pl.errorbar(time_pol, data_avg_norm, ls = 'None', yerr = data_SD_norm, ecolor=c,elinewidth=0.05)
    
    pl.xlabel(r'$t$ $(s)$', fontsize = 30)
    if Title == 'Storage Modulus':
        pl.ylabel(r'$G^{\prime}$ / $G^{\prime}_{final} (CTL)$ (-)', fontsize=30)
        pl.ylim (0,1.2)
    elif Title == 'Loss Modulus' :
        pl.ylabel(r'$G^{\prime\prime}$/$G^{\prime\prime}_{final} (CTL)$ (-)', fontsize = 30)
        pl.ylim (0,1.2)

    for spine in pl.gca().spines.values():
            spine.set_linewidth(1.5)
        
        
    pl.tick_params(axis='both', which='major', length=8, width=2)
    pl.xticks(fontsize=24)
    pl.yticks(fontsize=24)
        
    
    pl.legend(loc='lower right',fontsize=20, frameon=False)
    # pl.tight_layout()
    pl.xlim(0,6000)  
    

 
def plotPolymerisation_normpeak(data, Title: str, l: str, c: str, s: str):
    # adding .str.replace(',', '') removed the commas
    time_pol = np.array(data[0]['Time'].str.replace(',', ''), dtype='float64')
    # if np.size(data,0)==1:
    #     pl.plot(time_pol, np.array(data[0][Title],dtype='float64'), ls='-', color = c, label=l,marker=s, ms=1,linewidth=2)
    data_norm = []
    for i in range(np.size(data, 0)):
        data_norm.append(np.array(
            data[i][Title], dtype='float64')/max(np.array(data[i][Title], dtype='float64')))
        # Renormalizing each polymerization curve bu the maximal G' or G" value
    data_avg_norm = np.average(data_norm, axis=0)
    data_SD_norm = np.std(data_norm, axis=0)

    pl.plot(time_pol, data_avg_norm, ls='-', color=c,
            label=l, marker=s, ms=1, linewidth=3)
    pl.errorbar(time_pol, data_avg_norm, ls='None',
                yerr=data_SD_norm, ecolor=c, elinewidth=0.05)
    
    pl.xticks(fontsize=24)
    pl.yticks(fontsize=24)
    pl.xlabel(r'$t$ $(s)$',fontsize=30)
    if Title == 'Storage Modulus':
        pl.ylabel(r'$G^{\prime}$/$G^{\prime}_{\text{max}}$(-)',fontsize=30)
        pl.ylim(0, 1.2)
    elif Title == 'Loss Modulus':
        pl.ylabel(r'$Normalized G^{\prime\prime}$',fontsize=30)
        pl.ylim(0, 1.2)
        
    for spine in pl.gca().spines.values():
                spine.set_linewidth(1.5)
                
    pl.legend(loc='lower right',fontsize=20, frameon=False)
    # pl.tight_layout()
    pl.xlim(0, 6000)
    
    

    
#Plot frequency sweep from data. G' ( Title = 'Storage Modulus') or G" (Title = 'Loss Modulus') as a function of frequency.
# Here data plotted until the 6th point corrrsponding to 1 Hz. 
# Curve and points of given color c, given shape s, and a given label l

# def plotFreqSweep (data,Title:str,l:str,c:str, s:str):
#     data_avg = average(data,Title)
#     data_SD = standard_deviation(data,Title)
#     freq = np.array(data[0]['Frequency'],dtype='float64')
#     pl.plot(freq[0:6], data_avg[0:6],ls='-', color = c, label=l,marker=s, ms=5, linewidth = 1)
#     pl.errorbar(freq[0:6], data_avg[0:6], ls = 'None', yerr = data_SD[0:6], ecolor=c,elinewidth=0.8) 
#     pl.loglog()
#     pl.xlabel(r'Frequency (Hz)')
#     if Title == 'Storage Modulus':
#         pl.ylabel(r'$G^{\prime}$ $(Pa)$')
#     elif Title == 'Loss Modulus' :
#         pl.ylabel(r'$G^{\prime}$, $G^{\prime\prime}$ $(Pa)$')
#     pl.legend(loc='upper right')
#     pl.tight_layout()
#     pl.ylim(1,150)
    

##Frequency sweep plotting both G' and G''
def plotFreqSweep(data, Title: str, l: str, c: str, s: str):
    data_avg = average(data, Title)
    data_SD = standard_deviation(data, Title)
    freq = np.array(data[0]['Frequency'], dtype='float64')

    marker_style = 's' if Title == 'Loss Modulus' else s

    pl.plot(freq[0:6], data_avg[0:6], ls='-', color=c, marker=marker_style, ms=6, linewidth=3)
    pl.errorbar(freq[0:6], data_avg[0:6], ls='None', yerr=data_SD[0:6], ecolor=c, elinewidth=0.8)
    pl.loglog()
    pl.xlabel(r'$f$ $(Hz)$', fontsize=30)

    pl.ylabel(r'$G^{\prime}$, $G^{\prime\prime}$ $(Pa)$', fontsize=30)

    # Show legend with only the colored line for Storage Modulus
    if Title == 'Storage Modulus':
        pl.plot([], [], ls='-', color=c, label=l, linewidth=3)
        pl.legend(loc='lower right', fontsize=20, frameon=False)

    for spine in pl.gca().spines.values():
        spine.set_linewidth(1.5)

    pl.xticks(fontsize=24)
    pl.yticks(fontsize=24)
    pl.tick_params(axis='both', which='major', length=8, width=2)  # Make ticks longer and thicker
    pl.tick_params(axis='both', which='minor', length=6, width=2)  # Adjust minor ticks

    # pl.tight_layout()
    pl.ylim(1, 150)
    

#Plot stress ramp from data. Strain as a function of the applied stress.
#Curve and points of given color c, given shape s, and a given label l    
def plotStressRamp (data,l:str,c:str, s:str):
    data_avg = average(data,'Strain')
    data_SD = standard_deviation(data,'Strain')
    stress = np.array(data[0]['Shear Stress'],dtype='float64')
    pl.plot(data_avg[0:40],stress[0:40],ls='-', color = c, label=l,marker=s, ms=5, linewidth = 1)
    pl.errorbar(data_avg[0:40],stress[0:40], ls = 'None', yerr = data_SD[0:40], ecolor=c,elinewidth=0.8) 
    pl.xlabel(r'Strain (%)')
    pl.ylabel(r'Shear Stress (Pa)')
    pl.legend(loc='best')
    pl.tight_layout()
    pl.xlim(0,25)
    pl.ylim(0,70)
    pl.xlabel(r'Strain (%)')#,fontsize=14
    pl.ylabel(r'Shear Stress (Pa)') #,fontsize=14
    pl.legend(loc='best')
    pl.tight_layout()
    pl.xlim(0,40)
    pl.ylim(0,30)
    
#Plot differential modulus from data as a function of the stress (or strain). Mean (+/- SD).
#Curve and points of given color c, given shape s, and a given label l    
def plotDiffMod(data, l: str, c: str, s: str):
    all_diffMod = []
    all_stress = []
    
    # Calculate the differential modulus (K) for each dataset
    for dataset in data:  
        strain = np.array(dataset['Strain'], dtype='float64')
        stress = np.array(dataset['Shear Stress'], dtype='float64')

        # Compute differential modulus (K)
        diffMod = np.gradient(stress, strain / 100.0)
        
        all_diffMod.append(diffMod)
        all_stress.append(stress)

    # Convert lists to arrays for averaging
    all_diffMod = np.array(all_diffMod)
    all_stress = np.array(all_stress)

    # Ensure no NaN values in the datasets
    if np.any(np.isnan(all_diffMod)) or np.any(np.isnan(all_stress)):
        print("Warning: NaN values detected in the data.")
        all_diffMod = np.nan_to_num(all_diffMod)
        all_stress = np.nan_to_num(all_stress)

    # Compute average differential modulus and stress
    avg_diffMod = np.mean(all_diffMod, axis=0)
    avg_stress = np.mean(all_stress, axis=0)
    
    # Compute standard deviation of stress and differential modulus
    std_diffMod = np.std(all_diffMod, axis=0)
    std_stress = np.std(all_stress, axis=0)

    # Search for the peak starting from 1 Pa onwards
    # Find the first index where stress is greater than 1 Pa
    start_index = np.argmax(avg_stress > 1)

    # Ensure the peak index is found in the region of interest
    peak_index_avg = np.argmax(avg_diffMod[start_index:]) + start_index

  
   # Truncate the data at the peak index for the average curve
    avg_diffMod_truncated = avg_diffMod[:peak_index_avg + 1]
    avg_stress_truncated = avg_stress[:peak_index_avg + 1]
    std_diffMod_truncated = std_diffMod[:peak_index_avg + 1]
    std_stress_truncated = std_stress[:peak_index_avg + 1]

    # Plot only the average curve up to the peak
    pl.plot(avg_stress_truncated, avg_diffMod_truncated, ls='-', color=c, label=l, marker=s, ms=6, linewidth=3)
    
    # Plot the standard deviation as a shaded region around the average curve
    pl.fill_between(avg_stress_truncated, avg_diffMod_truncated - std_diffMod_truncated, avg_diffMod_truncated + std_diffMod_truncated, color=c, alpha=0.1)

   
    for spine in pl.gca().spines.values():
        spine.set_linewidth(1.5)
        
    pl.xlabel(r'$\sigma$ $(Pa)$', fontsize=30)
    pl.ylabel(r'$K$ $(Pa)$', fontsize=30)
    pl.loglog()
    pl.xticks(fontsize=24)
    pl.yticks(fontsize=24)
    pl.legend(loc='lower right', fontsize=16, frameon=False)
    pl.tick_params(axis='both', which='major', length=8, width=2)
    pl.tick_params(axis='both', which='minor', length=6, width=2)
    pl.xlim(0.1, 100)
    pl.ylim(10, 300)

    
    

##This is the new function, with truncation at the rupture point    
def plotDiffMod_all(data, l: str, c: str, s: str):
    all_diffMod = []
    all_stress = []
    all_strain = []
    
    for dataset in data:  # Assuming data is a list of dictionaries
        strain = np.array(dataset['Strain'], dtype='float64')
        stress = np.array(dataset['Shear Stress'], dtype='float64')

        # Calculate differential modulus for each dataset
        diffMod = np.gradient(stress, strain / 100.0)
        
        all_diffMod.append(diffMod)
        all_stress.append(stress)
        all_strain.append(strain)

    # Convert lists to arrays for easy averaging
    all_diffMod = np.array(all_diffMod)
    all_stress = np.array(all_stress)
    all_strain = np.array(all_strain)
    
    fig, ax = pl.subplots(figsize=(8, 6)) 
    
    for i in range(np.size(data, 0)):
        # Search for the peak starting from 1 Pa onwards
        start_index = np.argmax(all_stress[i] > 1)
        peak_index = np.argmax(all_diffMod[i][start_index:]) + start_index

        # Truncate the individual dataset at the peak
        truncated_diffMod = all_diffMod[i][:peak_index + 1]
        truncated_stress = all_stress[i][:peak_index + 1]

        # Plot the individual curve up to the peak
        pl.plot(truncated_stress, truncated_diffMod, ls='-', color=c, marker=s, ms=1.5, linewidth=1.5)
    for spine in pl.gca().spines.values():
        spine.set_linewidth(1.5)
        
    ax.set_xlabel(r'$\sigma$ $(Pa)$', fontsize=30)
    ax.set_ylabel(r'$K$ $(Pa)$', fontsize=30)
    ax.xaxis.set_tick_params(labelsize=24)
    ax.yaxis.set_tick_params(labelsize=24)
    pl.xticks(fontsize=24)
    pl.yticks(fontsize=24)
    pl.loglog()
    pl.legend(loc='upper left', fontsize=20, frameon=False)
    pl.tick_params(axis='both', which='major', length=8, width=2)
    pl.tick_params(axis='both', which='minor', length=6, width=2)
    pl.xlim(0.1, 100)
    pl.ylim(10, 1000)

# This function returns the average value of the differential modulus of one condition (data containing all the arrays of each sample of the condition)
# at 0.1 Pa stress. (point 11 of the array)   
def DiffMod_init (data):
    all_diffMod = []
    all_stress = []
    all_strain = []
    
    for dataset in data:  # Assuming data is a list of dictionaries
        strain = np.array(dataset['Strain'], dtype='float64')
        stress = np.array(dataset['Shear Stress'], dtype='float64')

        # Calculate differential modulus for each dataset
        diffMod = np.gradient(stress, strain / 100.0)
        
        all_diffMod.append(diffMod)
        all_stress.append(stress)
        all_strain.append(strain)

    # Convert lists to arrays for easy averaging
    all_diffMod = np.array(all_diffMod)
    all_stress = np.array(all_stress)
    all_strain = np.array(all_strain)

    # Calculate average and standard deviation across datasets
    avg_diffMod = np.mean(all_diffMod, axis=0)
    diffMod_init = avg_diffMod[11]
    return diffMod_init 

#Plot differential modulus from data1 as a function of the stress. Differential modulus of data1 normalized by the average value at the initial value 0.1 Pa (point 11) of data 2.
#Curve and points of given color c, given shape s, and a given label l    
def plotDiffMod_norm(data1, data2, l: str, c: str, s: str):
    all_diffMod = []
    all_stress = []
    all_strain = []
    
    for dataset in data1:  # Assuming data is a list of dictionaries
        strain = np.array(dataset['Strain'], dtype='float64')
        stress = np.array(dataset['Shear Stress'], dtype='float64')
        
        # Calculate differential modulus for each dataset
        diffMod = np.gradient(stress, strain / 100.0)
        
        all_diffMod.append(diffMod)
        all_stress.append(stress)
        all_strain.append(strain)

    for spine in pl.gca().spines.values():
        spine.set_linewidth(1.5)

    # Convert lists to arrays for easier computations
    all_diffMod = np.array(all_diffMod)
    all_stress = np.array(all_stress)
    all_strain = np.array(all_strain)

    # Ensure no NaN values in the datasets
    if np.any(np.isnan(all_diffMod)) or np.any(np.isnan(all_stress)):
        print("Warning: NaN values detected in the data.")
        all_diffMod = np.nan_to_num(all_diffMod)
        all_stress = np.nan_to_num(all_stress)

    # Compute average differential modulus and stress
    avg_diffMod = np.mean(all_diffMod, axis=0)
    avg_stress = np.mean(all_stress, axis=0)
    
    # Normalize by K(0.1 Pa) of data2
    avg_diffMod_norm = avg_diffMod / DiffMod_init(data2)
    diffMod_SD_norm = np.std(all_diffMod, axis=0) / DiffMod_init(data2)

    # Search for the peak starting from 1 Pa onwards
    start_index = np.argmax(avg_stress > 1)  # Find the first index where stress > 1 Pa

    # Ensure the peak index is found in the region of interest
    peak_index_avg = np.argmax(avg_diffMod_norm[start_index:]) + start_index

    # Truncate the data at the peak index
    avg_diffMod_norm_truncated = avg_diffMod_norm[:peak_index_avg + 1]
    avg_stress_truncated = avg_stress[:peak_index_avg + 1]
    diffMod_SD_norm_truncated = diffMod_SD_norm[:peak_index_avg + 1]

    # Plot the average differential modulus
    pl.plot(avg_stress_truncated, avg_diffMod_norm_truncated, ls='-', color=c, label=l, marker=s, ms=6, linewidth=3)
    
    # Plot the shaded region for the standard deviation
    pl.fill_between(avg_stress_truncated, avg_diffMod_norm_truncated - diffMod_SD_norm_truncated, avg_diffMod_norm_truncated + diffMod_SD_norm_truncated, color=c, alpha=0.1)
   

# Draw the edges with a darker color
    pl.plot(avg_stress_truncated, avg_diffMod_norm_truncated - diffMod_SD_norm_truncated, color=c, alpha=0.2, linewidth=1.5)
    pl.plot(avg_stress_truncated, avg_diffMod_norm_truncated + diffMod_SD_norm_truncated, color=c, alpha=0.5, linewidth=1.5)


    pl.xlabel(r'$\sigma$ $(Pa)$', fontsize=30) 
    pl.ylabel(r'$K$ / $K_{0} (CTL)$ (-)', fontsize=30) 
    pl.tick_params(axis='both', which='major', length=8, width=2)
    pl.tick_params(axis='both', which='minor', length=6, width=2)
    pl.xticks(fontsize=24)
    pl.yticks(fontsize=24)
    
    pl.legend(loc='lower right', fontsize=20, frameon=False) ##for integrin plot ncol=2 and fontsize =15
    pl.loglog()
  
    pl.tight_layout()
    pl.xlim(0.1, 50)
    pl.ylim(0.1, 10)   


# Function that gives as an output the list of onset times for an array containing all the arrays of samples for one condition.
# The onset time is defined by the intersection between the tangent of the curve were the first derivative is maximal and the t axis. 
# The input arguments are "data" the array containing all the arrays of each sample of the condition and sigma the value used for the gaussian filtering.
# The function can plot the storage modulus as a function of time, the first derivative as a function of time, the second derivative as a function of time
# with the onset time point visible to verify the computation. 
def onset_time(data,sigma):
    #Defining the time axis
    time = np.array(data[0][:]['Time'], dtype='float64')
    dt = time[1]-time[0]
    # Initializing the list that will contain all teh calculated onset times
    onset_time = np.zeros(np.size(data,0))
    for i in range (np.size(data,0)) :
        #Load the data
        sample = np.array(data[i][:]['Storage Modulus'], dtype='float64')
        #Smooth the curve
        sample_smooth = gaussian_filter1d(sample, sigma)
        #Compute first derivative
        dy = np.gradient(sample_smooth)
        #Compute second derivative
        d2y = np.gradient(dy)
        # Find the max of the first derivative and the corresponding index ind_tM and time tM
        ind_tM = np.argmax(dy)
        tM = ind_tM *dt
        #Equation of the tangent to the curve where the first derivative is maximal.
        tang = dy[ind_tM]/dt*(time-tM)+sample_smooth[ind_tM]
        #Finding the intersection of that tangent with the abcissa axis (t axis). It gives a value t1 corresponding to the onset time.
        ind_t1=0
        for j in range (1,len(dy)):
            if tang [j-1]*tang[j]< 0:  # Check if the product of consecutive y values is negative
                ind_t1 = j-1
        t1 = ind_t1*dt
        onset_time[i] = t1
    
       
    return onset_time
            




#%%  

##Plots with PAA beads (4 and 20%) and relative CTL of 12/11 and 14/11 - G'
pl.figure(figsize=(8, 6))
plotPolymerisation(CTL_beads_pol, 'Storage Modulus', 'CTL', 'grey', 'o')  
plotPolymerisation(PAbeads_4percent_pol, 'Storage Modulus', '4% MPs ', 'pink', 'o')
plotPolymerisation(PAbeads_20percent_pol, 'Storage Modulus', '20% MPs ', 'purple', 'o')
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Figure 3/Others - not normalised/Gprime beads - 4 and 20percent- CTLbeads.png', dpi=300, bbox_inches='tight')
pl.show()

#%%

#%%
##Plots with PAA beads (4 and 20%) and all CTL - G'
pl.figure(figsize=(8, 6))
plotPolymerisation(CTL_pol, 'Storage Modulus', 'CTL', 'grey', 'o')  
plotPolymerisation(PAbeads_4percent_pol, 'Storage Modulus', '4% MPs ', 'pink', 'o')
plotPolymerisation(PAbeads_20percent_pol, 'Storage Modulus', '20% MPs ', 'purple', 'o')
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Figure 3/Others - not normalised/Gprime beads - 4 and 20percent.png', dpi=300, bbox_inches='tight')
pl.show()
#%%

#%%
# Plots  with PAA beads (4 and 20%) and relative CTL of 12/11 and 14/11  - K
pl.figure(figsize=(8, 6))
plotDiffMod(CTL_beads_stressramp, 'CTL', 'grey', 'o')
plotDiffMod(PAbeads_4percent_stressramp, '4% MPs', 'pink', 'o')
plotDiffMod(PAbeads_20percent_stressramp, '20% MPs', 'purple', 'o')
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Figure 3/Others - not normalised/K beads - 4 and 20percent- CTLbeads.png', dpi=300, bbox_inches='tight')
pl.show()
#%%


#%%
#Plots  with PAA beads (4 and 20%) and all CTL  - K
pl.figure(figsize=(8, 6))
plotDiffMod(CTL_stressramp, 'CTL', 'grey', 'o')
plotDiffMod(PAbeads_4percent_stressramp, '4% MPs', 'pink', 'o')
plotDiffMod(PAbeads_20percent_stressramp, '20% MPs', 'purple', 'o')
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Figure 3/Others - not normalised/K beads - 4 and 20percent.png', dpi=300, bbox_inches='tight')
pl.show()
#%%



#%%
## Plot polymerization for 4% and 20% beads normalized by the final value of the corresponding CTL of those days
pl.figure(figsize=(8, 6))
plotPolymerisation_norm(CTL_beads_pol,CTL_beads_pol, 'Storage Modulus','CTL', 'grey', 'o')
plotPolymerisation_norm(PAbeads_4percent_pol,CTL_beads_pol, 'Storage Modulus','4% MPs', 'pink', 'o')
plotPolymerisation_norm(PAbeads_20percent_pol,CTL_beads_pol, 'Storage Modulus','20% MPs', 'purple', 'o')
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Figure 3/Gprime beads normal- 4 and 20percent- CTLbeads.png', dpi=300, bbox_inches='tight')
pl.show()
#%%

# #%%
# ##Plot of differential modulus normalized by differential modulus at 0.1 Pa of internal CTL. For beads and 4% and 20%.
pl.figure(figsize=(8, 6))
plotDiffMod_norm(CTL_beads_stressramp,CTL_beads_stressramp,'CTL', 'grey', 'o')
plotDiffMod_norm(PAbeads_4percent_stressramp,CTL_beads_stressramp, '4% MPs', 'pink', 'o')
plotDiffMod_norm(PAbeads_20percent_stressramp,CTL_beads_stressramp, '20% MPs', 'purple', 'o')
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Figure 3/K beads normal - all.png', dpi=300, bbox_inches='tight')
pl.show()
# #%%

# ##Experiments with integrin blocking antibody


# #%%
##Plots for 4 and 20% cells with anti-beta1 integrin blockin antibody and relative CTL of 13/12 - G'
pl.figure(figsize=(8, 6))
plotPolymerisation(CTL_pol_integblock, 'Storage Modulus','CTL- p5d2', 'grey', 'o')
plotPolymerisation(MDA_4percent_integblock_pol, 'Storage Modulus', r'4% - anti-$\beta$1 integrin', 'skyblue', 'o')
plotPolymerisation(MDA_20percent_integblock_pol, 'Storage Modulus', r'20% - anti-$\beta$1 integrin', 'orange', 'o')
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Figure 3/Others - not normalised/Gprime p5d2 - 4 and 20percent- CTLp5d2.png', dpi=300, bbox_inches='tight')
pl.show()

#%%

#%%
##Plots for 4 and 20% cells with anti-beta1 integrin blockin antibody and all CTL - G'
pl.figure(figsize=(8, 6))
plotPolymerisation(CTL_pol, 'Storage Modulus','CTL- p5d2', 'grey', 'o')
plotPolymerisation(MDA_4percent_integblock_pol, 'Storage Modulus', r'4% - anti-$\beta$1 integrin', 'skyblue', 'o')
plotPolymerisation(MDA_20percent_integblock_pol, 'Storage Modulus', r'20% - anti-$\beta$1 integrin', 'orange', 'o')
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Figure 3/Others - not normalised/Gprime p5d2 - 4 and 20percent.png', dpi=300, bbox_inches='tight')
pl.show()

#%%

#%%
## Plot polymerization for 4% and 20% with anti-beta1 integrin blockin antibody normalized by the final value of the corresponding CTL of those days
pl.figure(figsize=(8, 6))
plotPolymerisation_norm(CTL_pol_integblock, CTL_pol_integblock, 'Storage Modulus','CTL', 'grey', 'o')
plotPolymerisation_norm(MDA_4percent_integblock_pol, CTL_pol_integblock, 'Storage Modulus', r'4% - anti-$\beta$1 integrin', 'skyblue', 'o')
plotPolymerisation_norm(MDA_20percent_integblock_pol, CTL_pol_integblock, 'Storage Modulus',r'20% - anti-$\beta$1 integrin', 'orange', 'o')
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Figure 3/Gprime p5d2 normal - 4 and 20percent.png', dpi=300, bbox_inches='tight')
pl.show()

#%%
## Plot polymerization for 4% and 20% with anti-beta1 integrin blockin antibody + 4% and 20% 
##normalized by the final value of the corresponding CTL of those days
pl.figure(figsize=(8, 6))
plotPolymerisation_norm(CTL_pol_integblock, CTL_pol_integblock, 'Storage Modulus','CTL', 'grey', 'o')
plotPolymerisation_norm(MDA_1percent_pol, CTL_pol, 'Storage Modulus','4%', 'b', 'o')
plotPolymerisation_norm(MDA_4percent_integblock_pol, CTL_pol_integblock, 'Storage Modulus', r'4% - anti-$\beta$1 integrin', 'skyblue', 'o')
plotPolymerisation_norm(MDA_20percent_pol, CTL_pol, 'Storage Modulus','20%', 'r', 'o')
plotPolymerisation_norm(MDA_20percent_integblock_pol, CTL_pol_integblock, 'Storage Modulus',r'20% - anti-$\beta$1 integrin', 'orange', 'o')
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Figure 3/Others - not normalised/Gprime p5d2 normal - all.png', dpi=300, bbox_inches='tight')
pl.show()

#%%


##Plots for 4 and 20% cells with anti-beta1 integrin blockin antibody and relative CTL of 13/12 - K
pl.figure(figsize=(8, 6))
plotDiffMod(CTL_stressramp_integblock,'CTL', 'grey', 'o')
plotDiffMod(MDA_4percent_integblock_stressramp, r'4% - anti-$\beta$1 integrin', 'skyblue', 'o')
plotDiffMod(MDA_20percent_integblock_stressramp, r'20% - anti-$\beta$1 integrin', 'orange', 'o')
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Figure 3/Others - not normalised/K p5d2 - 4 and 20percent- CTLp5d2.png', dpi=300, bbox_inches='tight')
pl.show()

#%%


##Plots for 4 and 20% cells with anti-beta1 integrin blockin antibody and all CTL - K
pl.figure(figsize=(8, 6))
plotDiffMod(CTL_stressramp,'CTL', 'grey', 'o')
plotDiffMod(MDA_4percent_integblock_stressramp, r'4% - anti $\beta$1 integrin', 'skyblue', 'o')
plotDiffMod(MDA_20percent_integblock_stressramp, r'20% - anti $\beta$1 integrin', 'orange', 'o')
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Figure 3/Others - not normalised/K p5d2 - 4 and 20percent.png', dpi=300, bbox_inches='tight')
pl.show()
#%%


#%%


##Plots for 4 and 20% cells with anti-beta1 integrin blockin antibody and all CTL - K
pl.figure(figsize=(8, 6))
plotDiffMod(CTL_stressramp_integblock,'CTL', 'grey', 'o')
plotDiffMod(MDA_1percent_stressramp,'4%', 'b', 'o')
plotDiffMod(MDA_4percent_integblock_stressramp, r'4% - anti $\beta$1 integrin', 'skyblue', 'o')
plotDiffMod(MDA_20percent_stressramp,'20%', 'r', 'o')
plotDiffMod(MDA_20percent_integblock_stressramp, r'20% - anti $\beta$1 integrin', 'orange', 'o')
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Figure 3/Others - not normalised/K p5d2 - all - CTLp5d2.png', dpi=300, bbox_inches='tight')
pl.show()
#%%



#%%
#Plot of differential modulus normalized by differential modulus at 0.1 Pa of internal CTL. For control conditions and P5D2, 4% and 20%.
pl.figure(figsize=(8, 6))
plotDiffMod_norm(CTL_stressramp_integblock,CTL_stressramp_integblock,'CTL', 'grey', 'o')
plotDiffMod_norm(MDA_1percent_stressramp,CTL_stressramp,'4%', 'b', 'o')
plotDiffMod_norm(MDA_4percent_integblock_stressramp,CTL_stressramp_integblock, r'4% - anti-$\beta$1 integrin', 'skyblue', 'o')
plotDiffMod_norm(MDA_20percent_stressramp,CTL_stressramp,'20%', 'r', 'o')
plotDiffMod_norm(MDA_20percent_integblock_stressramp,CTL_stressramp_integblock,r'20% - anti-$\beta$1 integrin', 'orange', 'o')
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Figure 3/K p5d2 normal - all.png', dpi=300, bbox_inches='tight')
pl.show()
#%%

##Plot of differential modulus normalized by differential modulus at 0.1 Pa of internal CTL and 0.4% old, new and old + new 

# plotDiffMod_norm(MDA_0pt1percent_stressramp,MDA_0pt1percent_stressramp,'0.4% cells ', 'y', 'o')
# plotDiffMod_norm(MDA_0pt1percent_stressramp_new,MDA_0pt1percent_stressramp_new,'0.4% cells - new  ', 'goldenrod', 'o')
# plotDiffMod_norm(MDA_0pt1percent_stressramp_tot,MDA_0pt1percent_stressramp_tot,'0.4% cells - all  ', 'gold', 'o')


##%%Plot of individual curves for corresponding CTL, 4% and 20% integrin treated cells - Gprime
pl.figure(figsize=(8, 6))
plotPolymerisation_all(CTL_pol_integblock, 'Storage Modulus','CTL', 'grey', 'o')
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 3/SinglecurvesInteg - Gprime - CTL.png', dpi=300, bbox_inches='tight')
pl.show()


pl.figure(figsize=(8, 6))
plotPolymerisation_all(MDA_4percent_integblock_pol, 'Storage Modulus',r'4% - anti-$\beta$1 integrin', 'skyblue', 'o')
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 3/SinglecurvesInteg - Gprime - 4.png', dpi=300, bbox_inches='tight')
pl.show()

pl.figure(figsize=(8, 6))
plotPolymerisation_all(MDA_20percent_integblock_pol, 'Storage Modulus', r'20% - anti-$\beta$1 integrin', 'orange', 'o')
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 3/SinglecurvesInteg - Gprime - 20.png', dpi=300, bbox_inches='tight')
pl.show()




# %%Plot of individual curves for corresponding CTL, 4% and 20% integrin treated cells - K

pl.figure(figsize=(8, 6))
plotDiffMod_all(CTL_stressramp_integblock,'CTL', 'grey', 'o')
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 3/SinglecurvesInteg - K - CTL.png', dpi=300, bbox_inches='tight')
pl.show() 

pl.figure(figsize=(8, 6))
plotDiffMod_all(MDA_4percent_integblock_stressramp, '4%', 'skyblue', 'o')
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 3/SinglecurvesInteg - K - 4.png', dpi=300, bbox_inches='tight')
pl.show()

pl.figure(figsize=(8, 6))
plotDiffMod_all(MDA_20percent_integblock_stressramp, r'4% - anti-$\beta$1 integrin', 'orange', 'o')
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 3/SinglecurvesInteg - K - 20.png', dpi=300, bbox_inches='tight')
pl.show()


# %%%%Plot of individual curves for corresponding CTL, 4% and 20% PAA beads - Gprime
pl.figure(figsize=(8, 6))
plotPolymerisation_all(CTL_beads_pol, 'Storage Modulus','CTL', 'grey', 'o')
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 3/SinglecurvesBeads - Gprime - CTL.png', dpi=300, bbox_inches='tight')
pl.show()


pl.figure(figsize=(8, 6))
plotPolymerisation_all(PAbeads_4percent_pol, 'Storage Modulus','4% MPs', 'pink', 'o')
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 3/SinglecurvesBeads - Gprime - 4.png', dpi=300, bbox_inches='tight')
pl.show()

pl.figure(figsize=(8, 6))
plotPolymerisation_all(PAbeads_20percent_pol, 'Storage Modulus', '20% MPs', 'purple', 'o')
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 3/SinglecurvesBeads - Gprime - 20.png', dpi=300, bbox_inches='tight')
pl.show()



# %%
pl.figure(figsize=(8, 6))
plotDiffMod_all(CTL_beads_stressramp,'CTL', 'grey', 'o')
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 3/SinglecurvesBeads - K - CTL.png', dpi=300, bbox_inches='tight')
pl.show()


pl.figure(figsize=(8, 6))
plotDiffMod_all(PAbeads_4percent_stressramp,'4% MPs', 'pink', 'o')
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 3/SinglecurvesBeads - K - 4.png', dpi=300, bbox_inches='tight')
pl.show()

pl.figure(figsize=(8, 6))
plotDiffMod_all(PAbeads_20percent_stressramp,'20% MPs', 'purple', 'o')
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 3/SinglecurvesBeads - K - 20.png', dpi=300, bbox_inches='tight')
pl.show()

# #%% Onset time boxplots
sigma = 2
# MDA-MB-231 cells
#Excluding the first two samples because it took some time to launch the samples
CTL_onset_time = onset_time(CTL_pol[2:],sigma)

MDA_4percent_onset_time = onset_time(MDA_1percent_pol,sigma)
#Excluding the first two samples because the polymerization curves look weird with two inflexion points
MDA_20percent_onset_time = onset_time(MDA_20percent_pol[2:],sigma)

# PAA beads
CTL_beads_pol_onset_time = onset_time(CTL_beads_pol,sigma)
PAbeads_4percent_pol_onset_time = onset_time(PAbeads_4percent_pol,sigma)
PAbeads_20percent_pol_onset_time = onset_time(PAbeads_20percent_pol,sigma)

# Cells with P5D2 anti-beta 1 integrin blocking antibody 
CTL_integblock_onset_time = onset_time(CTL_pol_integblock,sigma)
MDA_4percent_integblock_onset_time = onset_time(MDA_4percent_integblock_pol,sigma)
MDA_20percent_integblock_onset_time = onset_time(MDA_20percent_integblock_pol,sigma)

#Making a table with all the onset times of the internal CTL of beads and integrin blocked experiments
CTL_total_onset_time = np.concatenate((CTL_beads_pol_onset_time,CTL_integblock_onset_time))

##Table with CTL - PA beads 4% - PA beads 20% - 4% cells - 20% cells - 4% integ block - 20% integ block 
##all = [CTL_total_onset_time,
        ## PAbeads_4percent_pol_onset_time,
        ## PAbeads_20percent_pol_onset_time,
        ## MDA_4percent_onset_time,
        ## MDA_20percent_onset_time,
        ## MDA_4percent_integblock_onset_time,
        ## MDA_20percent_integblock_onset_time]


##Table with CTL - PA beads 4% - PA beads 20% - 4% integ block - 20% integ block 
all = [CTL_total_onset_time,
       PAbeads_4percent_pol_onset_time,
       PAbeads_20percent_pol_onset_time,
       MDA_4percent_integblock_onset_time,
       MDA_20percent_integblock_onset_time]

# Custom color palette
palette_colors = [
    color_blind_palette["grey"],  # CTL
    color_blind_palette["pink"],     # 4% beads
    color_blind_palette["purple"],     # 20% beads
    color_blind_palette["skyblue"],     # 4% cells - P5D2
    color_blind_palette["orange"],     # 20% cells - P5D2
]
    
#Perform pairwise Mann-Whitney U tests (Note:Comparing only CTL with other conditions with cells!!!)
p_values = []
for i in range(1, len(all)):  # Start from 1 to skip comparing CTL with itself
    _, p = mannwhitneyu(all[0], all[i], alternative='two-sided')
    p_values.append(p)

# Function to map p-value to annotation text
def get_p_value_annotation(p_value):
    if p_value < 0.001:
        return "***"
    elif p_value < 0.01:
        return "**"
    elif p_value < 0.05:
        return "*"
    else:
        return "ns"
# print(p_values)

# Plot
pl.figure(figsize=(8, 6))
boxprops = {'linewidth': 2}         # Thicker box lines
whiskerprops = {'linewidth': 2}     # Thicker whisker lines
capprops = {'linewidth': 3}         # Thicker cap lines
medianprops = {'color': 'black', 'linewidth': 3}  # Thicker median line
for spine in pl.gca().spines.values():
        spine.set_linewidth(1.5)
        

# Create the boxplot and swarmplot
ax = sns.boxplot(data=all, palette= palette_colors, width=0.5, 
                  boxprops=boxprops, whiskerprops=whiskerprops, 
                  capprops=capprops, medianprops=medianprops, 
                  showfliers=False)
sns.swarmplot(data=all, color="black", size = 6)


# Add labels
ax.set_xticks(range(len(all)))  # Set tick positions
ax.set_xticklabels(
    ['CTL', '4% MPs', '20% MPs',r'4% anti-$\beta$1 integrin', r'20% anti-$\beta$1 integrin'],
    fontsize=30,
    rotation=30,  # Adjust rotation (try 45 if needed)
    ha="right"    # Align text to the right
)
ax.set_ylabel(r'$t_{onset}$ (s)', fontsize=30)
pl.tick_params(axis='both', which='major', length=8, width=2)  # Make ticks longer and thicker
pl.tick_params(axis='both', which='minor', length=6, width=2) 
pl.yticks(fontsize=24)
pl.xticks(fontsize=24)


#Set the range of the plot
ax.set_ylim(0,1400)

# Annotate statistical significance
y_max = max([np.max(data) for data in all])  # Find the maximum value in the dataset
line_height = y_max + 1  # Starting height for annotation lines
spacing = 50 # Spacing between annotation lines

for i, p_value in enumerate(p_values):
    annotation = get_p_value_annotation(p_value)
    x1, x2 = 0, i + 1  # Positions of the two groups being compared
    y = line_height + i * spacing  # Adjust line height for each comparison
    ax.plot([x1, x1, x2, x2], [y, y + 0.5, y + 0.5, y], color="black", lw=1.5)  # Draw line
    ax.text((x1 + x2) / 2, y + 0.5, annotation, ha="center", va="bottom", fontsize=18, color="black")  # Add annotation
    line_height += spacing  # Increment the base line height for the next pair
    
# Finalize plot
pl.tight_layout()
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Figure 3/Onsettimes_CTL_PAbeads_Integ.png', dpi=300, bbox_inches='tight')
pl.show()




#%% Strain and stress at rupture plots 
#==> cf code functions_rupture_strain_stress
