# -*- coding: utf-8 -*-
"""
Created on Thu Mar 13 14:30:09 2025

@author: inagle
"""


import pandas as pd
import matplotlib.pyplot as pl
import csv


#%% Histogram of cell diameter for HDFs 
# Define file path
file_path_HDF = "C:/Users/inagle/OneDrive - Delft University of Technology/Rheomicroscopy/Flow cytometry HDF - MDA/MDA - HDF flow cytometry - analyzed by Irène/20240527 - HDF flow cytometry Charlie_reanalyzed_GROUPED_run1and2_AR0pt7.csv"  # Change this to your actual file path

# Detect the delimiter
def detect_delimiter(file_path):
    with open(file_path, 'r', encoding='utf-8') as f:
        sample = f.readline()
        dialect = csv.Sniffer().sniff(sample)
    return dialect.delimiter

# Load the CSV file with detected delimiter
delimiter = detect_delimiter(file_path_HDF)
df = pd.read_csv(file_path_HDF, delimiter=delimiter, encoding="utf-8")

# Rename columns correctly
df.columns = ["Object_Number", "Diameter_M04"]

# Drop the first row if it contains column names
df = df.iloc[1:].reset_index(drop=True)

# Convert "Diameter_M04" to numeric format (replacing commas with dots)
df["Diameter_M04"] = df["Diameter_M04"].str.replace(",", ".").astype(float)

# Compute the average and standard deviation of values above 10 (because some outliers are around 0-5 um)
filtered_values = df[df["Diameter_M04"] > 10]["Diameter_M04"]
average = filtered_values.mean()
std_dev = filtered_values.std()
print(f"Average (values > 10): {average:.2f}")
print(f"Standard Deviation (values > 10): {std_dev:.2f}")

# Plot the histogram
pl.figure(figsize=(8, 6))
pl.hist(filtered_values, bins=35, color='paleturquoise', edgecolor='black', alpha=0.7,density=True,range=(10, 45))
pl.xlabel("Cell diameter (μm)",fontsize=26,labelpad=15)
pl.ylabel("Normalized frequency (-)",fontsize=26,labelpad=15)
pl.ylim(0, 0.2)
pl.xticks(fontsize=22)
pl.yticks(fontsize=22)
pl.tick_params(axis='both', which='major', length=8, width=2)
# Show the plot
pl.show()

# Compute the average and standard deviation of values above 10 (because some outliers are around 0-5 um)
filtered_values = df[df["Diameter_M04"] > 10]["Diameter_M04"]
average = filtered_values.mean()
std_dev = filtered_values.std()
print(f"Average (values > 10): {average:.2f}")
print(f"Standard Deviation (values > 10): {std_dev:.2f}")



#%% Histogram of cell diameter for MDA-MB-231 Lifeact-GFP cells

# Define file path
file_path_MDA = "C:/Users/inagle/OneDrive - Delft University of Technology/Rheomicroscopy/Flow cytometry HDF - MDA/MDA - HDF flow cytometry - analyzed by Irène/20240319_MDA-MB-231.csv"  # Change this to your actual file path

# Detect the delimiter
def detect_delimiter(file_path):
    with open(file_path, 'r', encoding='utf-8') as f:
        sample = f.readline()
        dialect = csv.Sniffer().sniff(sample)
    return dialect.delimiter

# Load the CSV file with detected delimiter
delimiter = detect_delimiter(file_path_MDA)
df = pd.read_csv(file_path_MDA, delimiter=delimiter, encoding="utf-8")

# Rename columns correctly
df.columns = ["Object_Number", "Diameter_M04"]

# Drop the first row if it contains column names
df = df.iloc[1:].reset_index(drop=True)

# Convert "Diameter_M04" to numeric format (replacing commas with dots)
df["Diameter_M04"] = df["Diameter_M04"].str.replace(",", ".").astype(float)

# Plot the histogram
pl.figure(figsize=(8, 6))
pl.hist(df["Diameter_M04"], bins=35, color='paleturquoise', edgecolor='black', alpha=0.7,density=True,range=(10, 45))
pl.xlabel("Cell diameter (μm)",fontsize=26,labelpad=15)
pl.ylabel("Normalized frequency (-)",fontsize=26,labelpad=15)
pl.ylim(0, 0.2)
pl.xticks(fontsize=22)
pl.yticks(fontsize=22)
pl.tick_params(axis='both', which='major', length=8, width=2)
# Show the plot
pl.show()

 