# -*- coding: utf-8 -*-
"""
Created on Wed Jan 29 17:10:51 2025

@author: inagle
"""

"""
Created on Tue Sep 24 14:23:34 2024

@author: inagle
"""


# Martin Krzywinski's 24-color, color-blind-friendly palette
import numpy as np
import matplotlib
import matplotlib.pyplot as pl
import pandas as pd
import io
import codecs
from apMethods_ReadRheo_Datacollection_mt import *
import matplotlib
import matplotlib.pyplot as pl
from scipy.ndimage import gaussian_filter1d
import seaborn as sns
from scipy.stats import mannwhitneyu
import csv
color_blind_palette = {
    "orange": "#E69F00",  # Orange
    "skyblue":"#56B4E9",  # Sky Blue
    "g": "#009E73",  # Green
    "y":"#F0E442",  # Yellow
    "b": "#0072B2",  # Blue
    "darkorange":"#D55E00",  # Dark Orange
    "pink":"#CC79A7",  # Pink
    "grey":"#999999",  # Grey
    "lightyellow":"#F7CB4D",  # Light Yellow
    "lightblue":"#ADD8E6",  # Light Blue
    "lightgreen":"#90EE90",  # Light Green
    "lightpink":"#FFB6C1",  # Light Pink
    "darkred":"#A52A2A",  # Brown
    "purple":"#800080",  # Purple
    "olive":"#808000",  # Olive
    "orange":"#FFA500",  # Orange
    "steelblue":"#4682B4",  # Steel Blue
    "brown":"#8B4513",  # Saddle Brown
    "cadetblue": "#5F9EA0",  # Cadet Blue
    "r":"#CD5C5C"   # Indian Red
}


# Set default color cycle
def resolve_color(color):
    """Resolve shorthand color names to Tableau hex codes."""
    return color_blind_palette.get(color, color)  # Default to original if shorthand not found


# Override the default plot function to use custom color
original_plot = pl.plot


def patched_plot(*args, **kwargs):
    if 'color' in kwargs:
        kwargs['color'] = resolve_color(kwargs['color'])
    return original_plot(*args, **kwargs)


pl.plot = patched_plot

#%%
# This function calculates the average for n samples.
# It is first required to construct an array containing all the arrays of each sample.
# The function has as variables :
# 1) data that is an array containing the of n x l x c, with n the number of different samples to average,
# l the number or lines in the array, or number of measuring points, and c the number of columns.
# 2) Title corresponds to the title of the column we would like to plot.
# For example, Title = Storage Modulus or Title = Loss Modulus


def average(data, Title: str):
    # Title = Type + " Modulus"
    # n number of samples averaged
    n = np.size(data, 0)
    S = np.zeros(np.size(data, 1))
    for i in range(n):
        S = S + np.array(data[i][Title], dtype='float64')
    avg = S/n
    return avg


def standard_deviation(data, Title: str):
    # Title = Type + " Modulus"
    # n number of samples averaged
    n = np.size(data, 0)
    # l number of lines or measuring points per sample
    l = np.size(data, 1)
    Q = np.zeros(l)
    for i in range(n):
        Q = Q + np.square(np.array(data[i][Title],
                          dtype='float64')-average(data, Title))
    SD = np.sqrt(Q/n)
    return SD

# Plots
# Plot polymerization curve from data. Mean +/- SD. Data has to be an array of all polymerisation measurements, corresponding to a 12 x 1080 array. (1h30 polymerization and point every 5s)
# G' ( Title = 'Storage Modulus') or G" (Title = 'Loss Modulus') as a function of time.
# Curve and points of given color c, given shape s, and a given label l


def plotPolymerisation(data, Title: str, l: str, c: str, s: str):
    # adding .str.replace(',', '') removed the commas
    time_pol = np.array(data[0]['Time'].str.replace(',', ''), dtype='float64')
    # if np.size(data,0)==1:
    #     pl.plot(time_pol, np.array(data[0][Title],dtype='float64'), ls='-', color = c, label=l,marker=s, ms=1,linewidth=2)

    data_avg = average(data, Title)
    data_SD = standard_deviation(data, Title)
    pl.plot(time_pol, data_avg, ls='-', color=c,
            label=l, marker=s, ms=1, linewidth=3)
    pl.errorbar(time_pol, data_avg, ls='None',
                yerr=data_SD, ecolor=c, elinewidth=0.05)

    pl.xlabel(r'Time (s)', fontsize=22)
    if Title == 'Storage Modulus':
        pl.ylabel(r'$G^{\prime}$ $(Pa)$', fontsize=22)
        pl.ylim(0, 130)
    elif Title == 'Loss Modulus':
        pl.ylabel(r'$G^{\prime\prime}$ $(Pa)$', fontsize=22)
        pl.ylim(0, 30)

    pl.xticks(fontsize=18)
    pl.yticks(fontsize=18)
    pl.legend(loc='lower right', fontsize=14, frameon=False)
    # pl.tight_layout()
    pl.xlim(0, 6000)


# Plots
# Plot polymerization curve from data. Plots all the individual curves.
# Data has to be an array of all polymerisation measurements, corresponding to a 12 x 1080 array. (1h30 polymerization and point every 5s)
# G' ( Title = 'Storage Modulus') or G" (Title = 'Loss Modulus') as a function of time.
# Curve and points of given color c, given shape s, and a given label l
def plotPolymerisation_all(data, Title: str, l: str, c: str, s: str):
    # adding .str.replace(',', '') removed the commas
    time_pol = np.array(data[0]['Time'].str.replace(',', ''), dtype='float64')

    for i in range(np.size(data, 0)):
        pl.plot(time_pol, gaussian_filter1d(np.array(
            data[i][Title], dtype='float64'), 20), color=c, marker=s, ms=0, linewidth=5)
        # Smoothing of the G' values with a gaussian filter and sigma = 2
    if Title == 'Storage Modulus':
        pl.xlabel(r'Time (s)', fontsize=26)
        pl.tight_layout()
        pl.ylabel(r'$G^{\prime}$ $(Pa)$', fontsize=26)
        pl.ylim(0, 90)

    elif Title == 'Loss Modulus':
        pl.ylabel(r'$G^{\prime\prime}$ $(Pa)$', fontsize=26)
        pl.ylim(0, 30)

    pl.xticks(fontsize=22)
    pl.yticks(fontsize=22)
    pl.legend(loc='lower right', fontsize=14, frameon=False)
    pl.legend(loc='lower right', fontsize=14, frameon=False)
    pl.xlim(0, 6000)

# Plots
# Plot polymerization normalized curve from data, mean and SD. Normalized data by the peak G' or G" value of data1. Data has to be an array of all polymerisation measurements, corresponding to a 12 x 1080 array. (1h30 polymerization and point every 5s)
# G' ( Title = 'Storage Modulus') or G" (Title = 'Loss Modulus') as a function of time.
# Curve and points of given color c, given shape s, and a given label l


def plotPolymerisation_normpeak(data, Title: str, l: str, c: str, s: str):
    # adding .str.replace(',', '') removed the commas
    time_pol = np.array(data[0]['Time'].str.replace(',', ''), dtype='float64')
    # if np.size(data,0)==1:
    #     pl.plot(time_pol, np.array(data[0][Title],dtype='float64'), ls='-', color = c, label=l,marker=s, ms=1,linewidth=2)
    data_norm = []
    for i in range(np.size(data, 0)):
        data_norm.append(np.array(
            data[i][Title], dtype='float64')/max(np.array(data[i][Title], dtype='float64')))
        # Renormalizing each polymerization curve bu the maximal G' or G" value
    data_avg_norm = np.average(data_norm, axis=0)
    data_SD_norm = np.std(data_norm, axis=0)

    pl.plot(time_pol, data_avg_norm, ls='-', color=c,
            label=l, marker=s, ms=1, linewidth=3)
    pl.errorbar(time_pol, data_avg_norm, ls='None',
                yerr=data_SD_norm, ecolor=c, elinewidth=0.05)
    
    pl.xticks(fontsize=18)
    pl.yticks(fontsize=18)
    pl.xlabel(r'$t$ $(s)$',fontsize=22)
    if Title == 'Storage Modulus':
        pl.ylabel(r'Normalized $G^{\prime}$ (-)',fontsize=22)
        pl.ylim(0, 1.2)
    elif Title == 'Loss Modulus':
        pl.ylabel(r'$Normalized G^{\prime\prime}$',fontsize=22)
        pl.ylim(0, 1.2)
    # pl.legend(loc='best',fontsize=18, frameon=False)
    # pl.tight_layout()
    pl.xlim(0, 6000)


# Function that gives as an output the list of onset times for an array containing all the arrays of samples for one condition.
# The onset time is defined by the intersection between the tangent of the curve were the first derivative is maximal and the t axis.
# The input arguments are "data" the array containing all the arrays of each sample of the condition and sigma the value used for the gaussian filtering.
# The function can plot the storage modulus as a function of time, the first derivative as a function of time, the second derivative as a function of time
# with the onset time point visible to verify the computation.
def onset_time(data, sigma):
    # Defining the time axis
    time = np.array(data[0][:]['Time'], dtype='float64')
    dt = time[1]-time[0]
    # Initializing the list that will contain all teh calculated onset times
    onset_time = np.zeros(np.size(data, 0))
    for i in range(np.size(data, 0)):
        # Load the data
        sample = np.array(data[i][:]['Storage Modulus'], dtype='float64')
        # Smooth the curve
        sample_smooth = gaussian_filter1d(sample, sigma)
        # Compute first derivative
        dy = np.gradient(sample_smooth)
        # Compute second derivative
        d2y = np.gradient(dy)
        # Find the max of the first derivative and the corresponding index ind_tM and time tM
        ind_tM = np.argmax(dy)
        tM = ind_tM * dt
        # Equation of the tangent to the curve where the first derivative is maximal.
        tang = dy[ind_tM]/dt*(time-tM)+sample_smooth[ind_tM]
        # Finding the intersection of that tangent with the abcissa axis (t axis). It gives a value t1 corresponding to the onset time.
        ind_t1 = 0
        for j in range(1, len(dy)):
            if tang[j-1]*tang[j] < 0:  # Check if the product of consecutive y values is negative
                ind_t1 = j-1
        t1 = ind_t1*dt
        onset_time[i] = t1

        # Plotting the curve with the tangent, its intersection to the t axis and with the tangent to the Plateau.
        pl.figure()
        pl.plot(time, sample_smooth)
        pl.plot(time, tang, 'k--', lw=1)
        pl.title('Sample ' + str(i))
        # pl.plot(time_pol,tang2, 'k--', lw = 1)
        pl.plot(tM, sample_smooth[ind_tM], marker='o', color='y')
        pl.plot(t1, 0, marker='o', color='g')
        # pl.plot(t2,tang[ind_t2],marker='o', color = 'r')
        pl.ylim(0, 120)
        pl.xlabel(r'Time (s)')
        pl.ylabel(r'$G^{\prime}$ $(Pa)$')

        # Plotting the first derivative and the max of the first derivative at tM
        pl.figure()
        pl.plot(time, dy)
        pl.plot(tM, dy[ind_tM], marker='o', color='y')

        # Plotting the second derivative and the max of the first derivative at tM corresponding to where the second derivative is equal to zero
        pl.figure()
        pl.plot(time, d2y)
        pl.plot(tM, d2y[ind_tM], marker='o', color='y')

    return onset_time

# Function that gives as an output the list of onset times for an array containing all the arrays of samples for one condition.
# This function is specifically made for the intensity curves. So each sample is composed of two columns, the first one is the time in s, the second is the intensity
# The onset time is defined by the intersection between the tangent of the curve were the first derivative is maximal and the t axis.
# The input arguments are "data" the array containing all the arrays of each sample of the condition and sigma the value used for the gaussian filtering.
# The function can plot the storage modulus as a function of time, the first derivative as a function of time, the second derivative as a function of time
# with the onset time point visible to verify the computation.
def onset_time_intensity(data, sigma):
    # Process all the samples from the condition so that they have the same length
    # Find the smallest array length
    min_length = min(arr.shape[1] for arr in data)

    # Trim all arrays to the smallest length
    # Defining the time axis
    time = data[0][0][0:min_length]
    intensity = [arr[1][:min_length] for arr in data]

    dt = time[1]-time[0]
    # Initializing the list that will contain all teh calculated onset times
    onset_time = np.zeros(np.size(intensity, 0))
    for i in range(np.size(intensity, 0)):
        # Load the data
        sample = intensity[i]
        # Smooth the curve
        sample_smooth = gaussian_filter1d(sample, sigma)
        # Compute first derivative
        dy = np.gradient(sample_smooth)
        # Compute second derivative
        d2y = np.gradient(dy)
        # Find the max of the first derivative and the corresponding index ind_tM and time tM
        ind_tM = np.argmax(dy)
        tM = ind_tM * dt
        # Equation of the tangent to the curve where the first derivative is maximal.
        tang = dy[ind_tM]/dt*(time-tM)+sample_smooth[ind_tM]
        # Finding the intersection of that tangent with the abcissa axis (t axis). It gives a value t1 corresponding to the onset time.
        ind_t1 = 0
        for j in range(1, len(dy)):
            if tang[j-1]*tang[j] < 0:  # Check if the product of consecutive y values is negative
                ind_t1 = j-1
        t1 = ind_t1*dt
        onset_time[i] = t1

        # Plotting the curve with the tangent, its intersection to the t axis and with the tangent to the Plateau.
        pl.figure()
        pl.plot(time, sample_smooth)
        pl.plot(time, tang, 'k--', lw=1)
        pl.title('Sample ' + str(i))
        # pl.plot(time_pol,tang2, 'k--', lw = 1)
        pl.plot(tM, sample_smooth[ind_tM], marker='o', color='y')
        pl.plot(t1, 0, marker='o', color='g')
        # pl.plot(t2,tang[ind_t2],marker='o', color = 'r')
        # pl.ylim(0, 120)
        pl.xlabel(r'Time (s)')
        pl.ylabel(r'Intensity (a.u.)')

        # Plotting the first derivative and the max of the first derivative at tM
        pl.figure()
        pl.plot(time, dy)
        pl.plot(tM, dy[ind_tM], marker='o', color='y')

        # Plotting the second derivative and the max of the first derivative at tM corresponding to where the second derivative is equal to zero
        pl.figure()
        pl.plot(time, d2y)
        pl.plot(tM, d2y[ind_tM], marker='o', color='y')

    return onset_time

# Function to read the csv UTF-8 files containing the Time as a function of Intensity over polymerization.
# The file needs to be saved as csv UTF-8 file.
# The column 'Time' is calculated from the first column that corresponds to the slice number (manually done in the Excell file)
# The column 'Int' is calculated from the "RawIntDen" column and substracting the value of the first point (manually done in the Excell file)
# The two extracted columns need to be named "Time" and "Int"
# All the numbers need to be converted to "General" type in Excell
# Make sure not to leave any empty boxes, otherwise this will lead to an error
def readInt(csv_file):
    time = []
    intensity = []
    with open(csv_file, mode='r', newline='', encoding='utf-8-sig') as file:  # utf-8-sig removes BOM
        # Specify semicolon delimiter
        reader = csv.DictReader(file, delimiter=";")

        for row in reader:
            time.append(float(row["Time"]))
            intensity.append(float(row["Int"]))
    data = [np.array(time), np.array(intensity)]
    return np.array(data)

# Plots
# Plot polymerization normalized curve from data1. Plots mean and SD. Normalized data by the average final intensity.
# Data has to to be an array of all intensity measurements (one column for time, one column for Intensity)
# corresponding to a N x 2 x ~270 or 540 array. (1h30 polymerization and imaging every 10s or 20s)
# Curve and points of given color c, given shape s, and a given label l


def plotInt_norm(data, l: str, c: str, s: str):
    # Process all the samples from the condition so that they have the same length
    # Find the smallest array length
    min_length = min(arr.shape[1] for arr in data)

    # Trim all arrays to the smallest length
    time = data[0][0][0:min_length]
    intensity = [arr[1][:min_length] for arr in data]

    intensity_norm = []
    for i in range(np.size(intensity, 0)):
        intensity_norm.append(intensity[i][:]/max(intensity[i][:]))
        # Renormalizing each polymerization curve by the maximal intensity value
    intensity_avg_norm = np.average(intensity_norm, axis=0)
    intensity_SD_norm = np.std(intensity_norm, axis=0)

    pl.plot(time, intensity_avg_norm, ls='-', color=c,
            label=l, marker=s, ms=1, linewidth=3)
    pl.errorbar(time, intensity_avg_norm, ls='None',
                yerr=intensity_SD_norm, ecolor=c, elinewidth=0.1)

    pl.xlabel(r'$t$ $(s)$',fontsize=22)
    pl.ylabel(r'Normalized intensity (-)',fontsize=22)
    pl.ylim(-0.1, 1.2)
    pl.xticks(fontsize=18)
    pl.yticks(fontsize=18)
    pl.legend(loc='best',fontsize=18, frameon=False)
    # pl.tight_layout()
    pl.xlim(0, 6000)


# # Load the CSV files with the measured intensities corresponding to the rheology measurements
# CTL samples
# Defining the path names
path_CTL_200224 = 'C:/Users/inagle/OneDrive - Delft University of Technology/Rheomicroscopy/Confocal_PBlab/20240220 - MDAMB231 in collagen I 4 mgmL - confocal/CTL_collagenI_4mgmL_second_round/Results_Polymerisation_1h30_dt=10s_gap0pt32_2.csv'
path_CTL_180924 = 'C:/Users/inagle/OneDrive - Delft University of Technology/Rheomicroscopy/Confocal_PBlab/20240918 - CTL atelocollagen I 4mgmL - confocal/Results_20240918_CTL_atelocollagenI_4mg_mL_polym_dt10s.csv'
path_CTL_270924 = 'C:/Users/inagle/OneDrive - Delft University of Technology/Rheomicroscopy/Confocal_PBlab/20240927 - CTL atelocollagen I 4mgmL - confocal/Results_20240927_CTL_atelocollagenI_4mg_mL_polym_dt10s.csv'
path_CTL_111024 = 'C:/Users/inagle/OneDrive - Delft University of Technology/Rheomicroscopy/Confocal_PBlab/20241011 - MDAMB231 in collagen I 4 mgmL - confocal/Results_20241011_CTL_atelocollagenI_4mg_mL_polym_dt10s.csv'

# Table containing all the samples the CTL condition (each sample is composed of two columns, one for time and one for the intensity)
data_CTL = [readInt(path_CTL_200224), readInt(path_CTL_180924),
            readInt(path_CTL_270924), readInt(path_CTL_111024)]

#4% samples
# Defining the path names
path_4percent_200224 = 'C:/Users/inagle/OneDrive - Delft University of Technology/Rheomicroscopy/Confocal_PBlab/20240220 - MDAMB231 in collagen I 4 mgmL - confocal/1%_MDAMB231_in_collagenI_4mgmL/Results_SUB_MaskLifeact_Polymerisation_1%cells_1h30_dt=10s.csv'
path_4percent_061124 = 'C:/Users/inagle/OneDrive - Delft University of Technology/Rheomicroscopy/Confocal_PBlab/20241106 - 4 percent MDAMb231 in atelocoll 4 mgmL/Results_20241106_4percent_MDAMb231_polym_dt20s_MeanThreshold.csv'
path_4percent_071124 = 'C:/Users/inagle/OneDrive - Delft University of Technology/Rheomicroscopy/Confocal_PBlab/20241107 - 4 percent MDAMB231 in atelocoll 4 mgmL - with or with 0.2% DMSO/Results_SubMeanThres_GBlur2_20241107_4percent_MDAMb231_polym_dt20s_sample2.csv'
path_4percent_181224 = 'C:/Users/inagle/OneDrive - Delft University of Technology/Rheomicroscopy/Confocal_PBlab/20241218 - MDAMB231 in atelocollagen I 4 mgmL/Results_SUB_20241218_MDAMB231_4percent_polymerization_dt20s - sample2-1.csv'

# Table containing all the samples the 4% condition (each sample is composed of two columns, one for time and one for the intensity)
# Sample from 20/02/2024 removed because the intensity curve did not look good
data_4percent = [ readInt(path_4percent_061124), readInt(path_4percent_071124), readInt(path_4percent_181224)]

# Dat processing to make sure that all the samples have the same size and picking delta = 20s, by removing the points taken every 10s
# Function to check if time difference between consecutive points is 10 seconds
def check_time_difference(arr):
    dt = np.diff(arr[0, :])  # Get the time differences
    # print(dt)
    return np.all(dt == 10)  # Check if all differences are 10

processed_data_4_percent = []
for arr in data_4percent:
    processed_time = []
    processed_intensity = []
    if check_time_difference(arr):  # Check if the time difference is 10
        # Slice the array to keep only one row out of two
        processed_data_4_percent.append(np.array([np.array(arr[0][::2]),np.array(arr[1][::2])]))
    else:
        processed_data_4_percent.append(np.array([np.array(arr[0][:]),np.array(arr[1][:])]))

pl.figure(figsize= (8,6))
plotInt_norm(data_CTL,'CTL','grey','o')
plotInt_norm(processed_data_4_percent,'4%','b','o')
pl.ylabel(r'$I \,  / \, I_{max}$ (-)',fontsize=24, labelpad = 10) 
# pl.savefig('C:/Users/inagle/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Figure_onset_time_rheoconfocal/Normalized intensity over time CTL 4percent.png', dpi=300, bbox_inches='tight')
pl.show()

# %%
## Plot of polymerization for all rheoconfocal curves CTL and 4%

# pl.figure(figsize=(8, 6))
# plotPolymerisation_normpeak(CTL_pol_comp_PPE, 'Storage Modulus', 'CTL', 'grey', 'o')
# plotPolymerisation_normpeak(MDA_4percent_comp_PPE,'Storage Modulus', '4%', 'b', 'o')
# pl.ylabel(r'$G^{\prime} \, / \, G^{\prime}_{max}$ (-)',fontsize=24, labelpad = 10) 
# pl.savefig('C:/Users/inagle/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Figure_onset_time_rheoconfocal/Storage modulus over time CTL 4percent.png', dpi=300, bbox_inches='tight')
# pl.show()

# Plot of polymerization over time of the 4% conditions in the rheoconfocal 
# Line width modified to 5 in the plotPolymerisation_normpeakfunction : pl.plot(ms = 5 instead of 1)
pl.figure(figsize=(13, 6))
plotPolymerisation_normpeak(MDA_4percent_comp_PPE,'Storage Modulus', '4%', 'b', 'o')
pl.xlabel(r'$t$ $(s)$',fontsize=30, labelpad = 10) # Labelpad increases the space between axis numbers and label
pl.ylabel(r'$G^{\prime} / G^{\prime}_{max}$ (-)',fontsize=30, labelpad = 10) 
pl.xticks(fontsize=26)
pl.yticks(fontsize=26)
for spine in pl.gca().spines.values():
    spine.set_linewidth(1.5)

# Adjust the padding for tick labels
pl.tick_params(axis='both', pad=20)  # Increase spacing for both axes
# Adjust the tick size
pl.tick_params(axis='both', which='major', length=7, width=1)

pl.savefig('C:/Users/inagle/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Figure 2/Polymerization_curve_rheoconfocal_4percentMDA_all.png', dpi=300, bbox_inches='tight')
pl.show()



#%% Plot of the onset time from the intensity curves
sigma = 10
print(onset_time_intensity(data_CTL,sigma))
print(onset_time_intensity(data_4percent,sigma))




#%% Plot of the onset time box plots 
# Plotting the different onset times
sigma = 10  # Value used for the gaussian smothing
CTL_pol_comp_PPE_onset_time = onset_time(CTL_pol_comp_PPE, sigma)
MDA_4percent_comp_PPE_onset_time = onset_time(MDA_4percent_comp_PPE, sigma)

CTL_onset_time_intensity= onset_time_intensity(data_CTL,sigma)
MDA_4percent_onset_time_intensity = onset_time_intensity(data_4percent,sigma)

all = [CTL_pol_comp_PPE_onset_time, MDA_4percent_comp_PPE_onset_time, CTL_onset_time_intensity, MDA_4percent_onset_time_intensity]
# Custom color palette
palette_colors = [
    color_blind_palette["grey"],  # CTL - rheology
    color_blind_palette["b"],     # 4% - rheology
    color_blind_palette["grey"],  # CTL - intensity
    color_blind_palette["b"],     # 4% - intensity
]

# # Perform pairwise Mann-Whitney U tests (Note:Comparing only CTL with other conditions with cells!!!)
# p_values = []
# for i in range(1, len(all)):  # Start from 1 to skip comparing CTL with itself
#     _, p = mannwhitneyu(all[0], all[i], alternative='two-sided')
#     p_values.append(p)

# # Function to map p-value to annotation text


# def get_p_value_annotation(p_value):
#     if p_value < 0.001:
#         return "***"
#     elif p_value < 0.01:
#         return "**"
#     elif p_value < 0.05:
#         return "*"
#     else:
#         return "ns"
# print(p_values)


# Compute means and standard deviations
means = [arr.mean() for arr in all]
print(means)
stds = [arr.std() for arr in all]



# Create figure
pl.figure(figsize=(8, 6))

# Seaborn scatter plot to display individual points
sns.swarmplot(data=all, color=".25",size = 6)

# Plot mean and standard deviation manually
for i, (mean, std, color) in enumerate(zip(means, stds, palette_colors)):
    pl.errorbar(i, mean, yerr=std, fmt='o', color=color, capsize=8, markersize=12,elinewidth=3, capthick = 3)

# Add labels
pl.xticks(range(len(all)), ['CTL', '4%', 'CTL', '4%'], fontsize=22)
pl.ylabel(r'$t_{onset}$ (s)', fontsize=24)
pl.yticks(fontsize=18)
pl.ylim(0, 2000)


# # Annotate statistical significance
# # Find the maximum value in the dataset
# y_max = max([np.max(data) for data in all])
# line_height = y_max + 50  # Starting height for annotation lines
# spacing = 25  # Spacing between annotation lines

# for i, p_value in enumerate(p_values):
#     annotation = get_p_value_annotation(p_value)
#     x1, x2 = 0, i + 1  # Positions of the two groups being compared
#     y = line_height + i * spacing  # Adjust line height for each comparison
#     ax.plot([x1, x1, x2, x2], [y, y + 0.5, y + 0.5, y],
#             color="black", lw=1.5)  # Draw line
#     ax.text((x1 + x2) / 2, y + 0.5, annotation, ha="center",
#             va="bottom", fontsize=15, color="black")  # Add annotation
#     line_height += spacing  # Increment the base line height for the next pair

# Finalize plot
pl.tight_layout()
pl.savefig('C:/Users/inagle/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Figure_onset_time_rheoconfocal/Onsettime_rheoconfoncal.png',
            dpi=300, bbox_inches='tight')
pl.show()
