# -*- coding: utf-8 -*-
"""
Created on Fri Jan 10 16:06:36 2025

@author: mtavasso
"""

# -*- coding: utf-8 -*-
"""
Created on Tue Sep 24 14:23:34 2024

@author: mtavasso
"""

import numpy as np
import matplotlib
import matplotlib.pyplot as pl
import pandas
import io

import codecs
from apMethods_ReadRheo_Datacollection import *

#This function calculates the average for n samples. 
# It is first required to construct an array containing all the arrays of each sample.
# The function has as variables :
# 1) data that is an array containing the of n x l x c, with n the number of different samples to average,
# l the number or lines in the array, or number of measuring points, and c the number of columns. 
# 2) Title corresponds to the title of the column we would like to plot. 
# For example, Title = Storage Modulus or Title = Loss Modulus
def average (data,Title:str):
    # Title = Type + " Modulus"   
    #n number of samples averaged
    n = np.size(data,0)
    S = np.zeros(np.size(data,1))
    for i in range (n) :
        S = S + np.array(data[i][Title], dtype='float64')
    avg = S/n
    return avg 

def standard_deviation (data,Title:str):
    # Title = Type + " Modulus" 
    #n number of samples averaged
    n = np.size(data,0)
    #l number of lines or measuring points per sample
    l = np.size(data,1)
    Q = np.zeros(l)
    for i in range (n) :
        Q = Q + np.square(np.array(data[i][Title], dtype='float64')-average(data,Title))
    SD = np.sqrt(Q/n)
    return SD 

# Plots 
#Plot polymerization curve from data. Data has to be an array of all polymerisation measurements, corresponding to a 12 x 1080 array. (1h30 polymerization and point every 5s)
#G' ( Title = 'Storage Modulus') or G" (Title = 'Loss Modulus') as a function of time.
#Curve and points of given color c, given shape s, and a given label l 
def plotPolymerisation(data,Title:str,l:str,c:str,s:str):
    time_pol = np.array(data[0]['Time'].str.replace(',', ''), dtype='float64') ## adding .str.replace(',', '') removed the commas
    # if np.size(data,0)==1:
    #     pl.plot(time_pol, np.array(data[0][Title],dtype='float64'), ls='-', color = c, label=l,marker=s, ms=1,linewidth=2)
    
    data_avg = average(data,Title)
    data_SD = standard_deviation(data,Title)
    pl.plot(time_pol, data_avg, ls='-', color = c, label=l,marker=s, ms=1, linewidth = 2)
    pl.errorbar(time_pol, data_avg, ls = 'None', yerr = data_SD, ecolor=c,elinewidth=0.05)
    
    pl.xlabel(r'Time (s)')
    if Title == 'Storage Modulus':
        pl.ylabel(r'$G^{\prime}$ $(Pa)$')
        pl.ylim (0,130)
    elif Title == 'Loss Modulus' :
        pl.ylabel(r'$G^{\prime\prime}$ $(Pa)$') 
        pl.ylim (0,30)
    pl.legend(loc='lower right')
    # pl.tight_layout()
    pl.xlim(0,6000)
    

    
#Plot frequency sweep from data. G' ( Title = 'Storage Modulus') or G" (Title = 'Loss Modulus') as a function of frequency.
# Here data plotted until the 6th point corrrsponding to 1 Hz. 
# Curve and points of given color c, given shape s, and a given label l

def plotFreqSweep (data,Title:str,l:str,c:str, s:str):
    data_avg = average(data,Title)
    data_SD = standard_deviation(data,Title)
    freq = np.array(data[0]['Frequency'],dtype='float64')
    pl.plot(freq[0:6], data_avg[0:6],ls='-', color = c, label=l,marker=s, ms=5, linewidth = 1)
    pl.errorbar(freq[0:6], data_avg[0:6], ls = 'None', yerr = data_SD[0:6], ecolor=c,elinewidth=0.8) 
    pl.loglog()
    pl.xlabel(r'Frequency (Hz)')
    if Title == 'Storage Modulus':
        pl.ylabel(r'$G^{\prime}$ $(Pa)$')
    elif Title == 'Loss Modulus' :
        pl.ylabel(r'$G^{\prime}$, $G^{\prime\prime}$ $(Pa)$')
    pl.legend(loc='upper right')
    pl.tight_layout()
    pl.ylim(1,150)
    
  

#Plot stress ramp from data. Strain as a function of the applied stress.
#Curve and points of given color c, given shape s, and a given label l    
def plotStressRamp (data,l:str,c:str, s:str):
    data_avg = average(data,'Strain')
    data_SD = standard_deviation(data,'Strain')
    stress = np.array(data[0]['Shear Stress'],dtype='float64')
    pl.plot(data_avg[0:40],stress[0:40],ls='-', color = c, label=l,marker=s, ms=5, linewidth = 1)
    pl.errorbar(data_avg[0:40],stress[0:40], ls = 'None', yerr = data_SD[0:40], ecolor=c,elinewidth=0.8) 
    pl.xlabel(r'Strain (%)')
    pl.ylabel(r'Shear Stress (Pa)')
    pl.legend(loc='best')
    pl.tight_layout()
    pl.xlim(0,25)
    pl.ylim(0,70)
    pl.xlabel(r'Strain (%)')#,fontsize=14
    pl.ylabel(r'Shear Stress (Pa)') #,fontsize=14
    pl.legend(loc='best')
    pl.tight_layout()
    pl.xlim(0,40)
    pl.ylim(0,30)
    

def plotDiffMod(data, l: str, c: str, s: str):
    all_diffMod = []
    all_stress = []
    all_strain = []
    
    for dataset in data:  # Assuming data is a list of dictionaries
        strain = np.array(dataset['Strain'], dtype='float64')
        stress = np.array(dataset['Shear Stress'], dtype='float64')

        # Calculate differential modulus for each dataset
        diffMod = np.gradient(stress, strain / 100.0)
        
        all_diffMod.append(diffMod)
        all_stress.append(stress)
        all_strain.append(strain)

    # Convert lists to arrays for easy averaging
    all_diffMod = np.array(all_diffMod)
    all_stress = np.array(all_stress)
    all_strain = np.array(all_strain)

    # Calculate average and standard deviation across datasets
    avg_diffMod = np.mean(all_diffMod, axis=0)
    diffMod_SD = np.std(all_diffMod, axis=0)

    # Plot the average differential modulus
    pl.plot(np.mean(all_stress, axis=0), avg_diffMod, ls='-', color=c, label=l, marker=s, ms=5, linewidth=1)
    # pl.plot(np.mean(all_strain, axis=0), avg_diffMod, ls='-', color=c, label=l, marker=s, ms=5, linewidth=1)

    # Plot the standard deviation as error bars
    # pl.errorbar(np.mean(all_stress, axis=0), avg_diffMod, yerr=diffMod_SD, fmt='none', ecolor=c, elinewidth=0.8)
    # pl.errorbar(np.mean(all_strain, axis=0), avg_diffMod, yerr=diffMod_SD, fmt='none', ecolor=c, elinewidth=0.8)

    pl.xlabel(r'Stress (Pa)') 
    pl.ylabel(r'K (Pa)') 
    pl.loglog()
    pl.legend(loc='best')
    pl.tight_layout()
    pl.xlim(0.1, 100)
    pl.ylim(10, 300)
   
    
   
def normalize_data(data, Title, reference_data):
    """
    Normalizes the data based on the final value of the reference_data.

    Parameters:
    - data: List of datasets to be normalized.
    - Title: Column to extract (e.g., 'Storage Modulus').
    - reference_data: The dataset used as the normalization reference.

    Returns:
    - Normalized dataset.
    """
    # Use the last value of the reference dataset for normalization
    reference_final_value = np.array(reference_data[0][Title], dtype='float64')[-1]
    normalized_data = []

    for dataset in data:
        # Normalize each dataset
        normalized = np.array(dataset[Title], dtype='float64') / reference_final_value
        dataset_normalized = dataset.copy()
        dataset_normalized[Title] = normalized
        normalized_data.append(dataset_normalized)

    return normalized_data

def plotPolymerisationWithNormalization(data, Title, l, c, s, reference_data):
    """
    Plots polymerization curves with normalization applied.

    Parameters:
    - data: List of datasets to be plotted.
    - Title: Column to extract (e.g., 'Storage Modulus').
    - l: Label for the plot.
    - c: Color for the plot.
    - s: Style for the plot markers.
    - reference_data: The dataset used as the normalization reference.
    """
    # Normalize the data
    normalized_data = normalize_data(data, Title, reference_data)

    # Extract time from the reference dataset
    time_pol = np.array(reference_data[0]['Time'].str.replace(',', ''), dtype='float64')

    # Compute averages and standard deviations for the normalized data
    data_avg = average(normalized_data, Title)
    data_SD = standard_deviation(normalized_data, Title)

    # Plot the normalized data
    pl.plot(time_pol, data_avg, ls='-', color=c, label=l, marker=s, ms=1, linewidth=2)
    pl.errorbar(time_pol, data_avg, ls='None', yerr=data_SD, ecolor=c, elinewidth=0.05)

    # Add labels and legend
    pl.xlabel(r'Time (s)')
    pl.ylabel(r'Normalized $G^{\prime}$')
    pl.legend(loc='lower right')

    # Set axis limits
    pl.xlim(0, 6000)
    pl.ylim(-0.1, 1.1)

    # Finalize layout
    pl.tight_layout()


##Plot of polymerization, Frequency sweep and stress ramp for CTL, 0.4, 4, 20% cells

# plotPolymerisation(CTL_pol, 'Storage Modulus', 'CTL', 'c', 'o')       ##Includes also the new batch data 
# plotPolymerisation(MDA_0pt1percent_pol , 'Loss Modulus','0.4% cells', 'y', 'o')
# plotPolymerisation(MDA_1percent_pol , 'Storage Modulus','4% cells', 'g', 'o')

# plotPolymerisation(MDA_20percent_new_pol, "Storage Modulus", '20% cells', 'red', 'o') ## here only new formulation array is used!!!



## Plot polymerization and stress ramp for 4% and 20% cells with or without incubation with 10 ug/mL anti-beta1 integrin blockin antibody
# plotPolymerisation(CTL_pol_integblock, 'Storage Modulus','CTL- p5d2', 'cadetblue', 'o')
# plotPolymerisation(MDA_20percent_new_pol, 'Storage Modulus','20% cells', 'r', 'o')
# plotPolymerisation(MDA_20percent_integblock_pol, 'Storage Modulus','20% cells - p5d2', 'orange', 'o')


# plotPolymerisation(MDA_1percent_pol , 'Storage Modulus','4% cells', 'g', 'o')
# plotPolymerisation(MDA_4percent_integblock_pol, 'Storage Modulus','4% cells - p5d2', 'lightgreen', 'o')

# Normalize and plot the curves
# CTL curve will be at 1
plotPolymerisationWithNormalization(CTL_pol_integblock,'Storage Modulus','CTL- p5d2','cadetblue','o', CTL_pol_integblock)

plotPolymerisationWithNormalization(MDA_20percent_integblock_pol,'Storage Modulus','20% cells - p5d2', 'orange', 'o', CTL_pol_integblock)

plotPolymerisationWithNormalization(MDA_4percent_integblock_pol,'Storage Modulus','4% cells - p5d2','lightgreen', 'o', CTL_pol_integblock)




