# -*- coding: utf-8 -*-
"""
Created on Tue Sep 24 14:23:34 2024

@author: inagle
"""

import numpy as np
import matplotlib
import matplotlib.pyplot as pl
import pandas
import io

import codecs
from apMethods_ReadRheo_Datacollection import *


# Martin Krzywinski's 24-color, color-blind-friendly palette
color_blind_palette = {
    "orange": "#E69F00",  # Orange
    "skyblue":"#56B4E9",  # Sky Blue
    "g": "#009E73",  # Green
    "y":"#F0E442",  # Yellow
    "b": "#0072B2",  # Blue
    "darkorange":"#D55E00",  # Dark Orange
    "pink":"#CC79A7",  # Pink
    "grey":"#999999",  # Grey
    "lightyellow":"#F7CB4D",  # Light Yellow
    "lightblue":"#ADD8E6",  # Light Blue
    "lightgreen":"#90EE90",  # Light Green
    "lightpink":"#FFB6C1",  # Light Pink
    "darkred":"#A52A2A",  # Brown
    "purple":"#800080",  # Purple
    "olive":"#808000",  # Olive
    "orange":"#FFA500",  # Orange
    "steelblue":"#4682B4",  # Steel Blue
    "brown":"#8B4513",  # Saddle Brown
    "cadetblue": "#5F9EA0",  # Cadet Blue
    "r":"#CD5C5C"   # Indian Red
}


# Set default color cycle
def resolve_color(color):
    """Resolve shorthand color names to Tableau hex codes."""
    return color_blind_palette.get(color, color)  # Default to original if shorthand not found

# Override the default plot function to use custom color
original_plot = pl.plot

def patched_plot(*args, **kwargs):
    if 'color' in kwargs:
        kwargs['color'] = resolve_color(kwargs['color'])
    return original_plot(*args, **kwargs)

pl.plot = patched_plot 




#This function calculates the average for n samples. 
# It is first required to construct an array containing all the arrays of each sample.
# The function has as variables :
# 1) data that is an array containing the of n x l x c, with n the number of different samples to average,
# l the number or lines in the array, or number of measuring points, and c the number of columns. 
# 2) Title corresponds to the title of the column we would like to plot. 
# For example, Title = Storage Modulus or Title = Loss Modulus

        
def average (data,Title:str):
    # Title = Type + " Modulus"   
    #n number of samples averaged
    n = np.size(data,0)
    S = np.zeros(np.size(data,1))
    for i in range (n) :
        S = S + np.array(data[i][Title], dtype='float64')
    avg = S/n
    return avg 

def standard_deviation (data,Title:str):
    # Title = Type + " Modulus" 
    #n number of samples averaged
    n = np.size(data,0)
    #l number of lines or measuring points per sample
    l = np.size(data,1)
    Q = np.zeros(l)
    for i in range (n) :
        Q = Q + np.square(np.array(data[i][Title], dtype='float64')-average(data,Title))
    SD = np.sqrt(Q/n)
    return SD 

# Plots 
#Plot polymerization curve from data. Mean +/- SD. Data has to be an array of all polymerisation measurements, corresponding to a 12 x 1080 array. (1h30 polymerization and point every 5s)
#G' ( Title = 'Storage Modulus') or G" (Title = 'Loss Modulus') as a function of time.
#Curve and points of given color c, given shape s, and a given label l 
def plotPolymerisation(data,Title:str,l:str,c:str,s:str):
    time_pol = np.array(data[0]['Time'].str.replace(',', ''), dtype='float64') ## adding .str.replace(',', '') removed the commas
    # if np.size(data,0)==1:
    #     pl.plot(time_pol, np.array(data[0][Title],dtype='float64'), ls='-', color = c, label=l,marker=s, ms=1,linewidth=2)
    
    data_avg = average(data,Title)
    data_SD = standard_deviation(data,Title)
    pl.plot(time_pol, data_avg, ls='-', color = c, label=l,marker=s, ms=1, linewidth = 3)
    pl.errorbar(time_pol, data_avg, ls = 'None', yerr = data_SD, ecolor=c,elinewidth=0.05)
    
    pl.xlabel(r'Time (s)', fontsize=22)
    if Title == 'Storage Modulus':
        pl.ylabel(r'$G^{\prime}$ $(Pa)$', fontsize=22)
        pl.ylim (0,130)
    elif Title == 'Loss Modulus' :
        pl.ylabel(r'$G^{\prime\prime}$ $(Pa)$', fontsize=22) 
        pl.ylim (0,30)
        

    pl.xticks(fontsize=18)
    pl.yticks(fontsize=18)
    pl.legend(loc='lower right',fontsize=12, frameon=False)
    # pl.tight_layout()
    pl.xlim(0,6000)

    
# Plots 
#Plot polymerization curve from data. Plots all the individual curves.
#Data has to be an array of all polymerisation measurements, corresponding to a 12 x 1080 array. (1h30 polymerization and point every 5s)
#G' ( Title = 'Storage Modulus') or G" (Title = 'Loss Modulus') as a function of time.
#Curve and points of given color c, given shape s, and a given label l 
def plotPolymerisation_all(data,Title:str,l:str,c:str,s:str):
    time_pol = np.array(data[0]['Time'].str.replace(',', ''), dtype='float64') ## adding .str.replace(',', '') removed the commas
    
    for i in range (np.size(data,0)):
        pl.plot(time_pol, np.array(data[i][Title], dtype='float64'), color = c,marker=s, ms=0,linewidth = 1)
        
    if Title == 'Storage Modulus':
        pl.xlabel(r'Time (s)')
        pl.tight_layout()
        pl.ylabel(r'$G^{\prime}$ $(Pa)$')
        pl.ylim (0,150)
       
    elif Title == 'Loss Modulus' :
        pl.ylabel(r'$G^{\prime\prime}$ $(Pa)$') 
        pl.ylim (0,30)
        pl.legend(loc='lower right')
    pl.xlim(0,6000)  
      
    
    
# Plots 
#Returns the final value of G' of G'' at the end of polymerization. Data has to be an array of all polymerisation measurements, corresponding to a 12 x 1080 array. (1h30 polymerization and point every 5s)
#G' ( Title = 'Storage Modulus') or G" (Title = 'Loss Modulus')
#Curve and points of given color c, given shape s, and a given label l 
def finalPol(data,Title:str):
    time_pol = np.array(data[0]['Time'].str.replace(',', ''), dtype='float64') ## adding .str.replace(',', '') removed the commas
    # if np.size(data,0)==1:
    #     pl.plot(time_pol, np.array(data[0][Title],dtype='float64'), ls='-', color = c, label=l,marker=s, ms=1,linewidth=2)
    
    data_avg = average(data,Title)
    final_value = data_avg[len(data_avg)-1]
    #final value of the Storage of teh Loss modulus 
    return final_value


# Plots 
#Plot polymerization normalized curve from data1 (mean and SD). Normalized data by the average final G' or G" value of data2. Data has to be an array of all polymerisation measurements, corresponding to a 12 x 1080 array. (1h30 polymerization and point every 5s)
#G' ( Title = 'Storage Modulus') or G" (Title = 'Loss Modulus') as a function of time.
#Curve and points of given color c, given shape s, and a given label l 
def plotPolymerisation_norm(data1,data2,Title:str,l:str,c:str,s:str):
    time_pol = np.array(data1[0]['Time'].str.replace(',', ''), dtype='float64') ## adding .str.replace(',', '') removed the commas
    # if np.size(data,0)==1:
    #     pl.plot(time_pol, np.array(data[0][Title],dtype='float64'), ls='-', color = c, label=l,marker=s, ms=1,linewidth=2)
    
    data_avg = average(data1,Title)
    data_SD = standard_deviation(data1,Title)
    data_avg_norm = data_avg/finalPol(data2, Title)
    data_SD_norm = data_SD/finalPol(data2, Title)
    
    pl.plot(time_pol, data_avg_norm, ls='-', color = c, label=l,marker=s, ms=1, linewidth = 2)
    pl.errorbar(time_pol, data_avg_norm, ls = 'None', yerr = data_SD_norm, ecolor=c,elinewidth=0.05)
    
    pl.xlabel(r'Time (s)')
    if Title == 'Storage Modulus':
        pl.ylabel(r'$G^{\prime}$/$G^{\prime}$(CTL)')
        pl.ylim (0,1.2)
    elif Title == 'Loss Modulus' :
        pl.ylabel(r'$G^{\prime\prime}$/$G^{\prime\prime}$(CTL)')
        pl.ylim (0,1.2)
    # pl.legend(loc='lower right')
    # pl.tight_layout()
    pl.xlim(0,6000)
    
    

    
#Plot frequency sweep from data. G' ( Title = 'Storage Modulus') or G" (Title = 'Loss Modulus') as a function of frequency.
# Here data plotted until the 6th point corrrsponding to 1 Hz. 
# Curve and points of given color c, given shape s, and a given label l

def plotFreqSweep (data,Title:str,l:str,c:str, s:str):
    data_avg = average(data,Title)
    data_SD = standard_deviation(data,Title)
    freq = np.array(data[0]['Frequency'],dtype='float64')
    pl.plot(freq[0:6], data_avg[0:6],ls='-', color = c, label=l,marker=s, ms=5, linewidth = 1)
    pl.errorbar(freq[0:6], data_avg[0:6], ls = 'None', yerr = data_SD[0:6], ecolor=c,elinewidth=0.8) 
    pl.loglog()
    pl.xlabel(r'Frequency (Hz)')
    if Title == 'Storage Modulus':
        pl.ylabel(r'$G^{\prime}$ $(Pa)$')
    elif Title == 'Loss Modulus' :
        pl.ylabel(r'$G^{\prime}$, $G^{\prime\prime}$ $(Pa)$')
    pl.legend(loc='upper right')
    pl.tight_layout()
    pl.ylim(1,150)
    
  

#Plot stress ramp from data. Strain as a function of the applied stress.
#Curve and points of given color c, given shape s, and a given label l    
def plotStressRamp (data,l:str,c:str, s:str):
    data_avg = average(data,'Strain')
    data_SD = standard_deviation(data,'Strain')
    stress = np.array(data[0]['Shear Stress'],dtype='float64')
    pl.plot(data_avg[0:40],stress[0:40],ls='-', color = c, label=l,marker=s, ms=5, linewidth = 1)
    pl.errorbar(data_avg[0:40],stress[0:40], ls = 'None', yerr = data_SD[0:40], ecolor=c,elinewidth=0.8) 
    pl.xlabel(r'Strain (%)')
    pl.ylabel(r'Shear Stress (Pa)')
    pl.legend(loc='best')
    pl.tight_layout()
    pl.xlim(0,25)
    pl.ylim(0,70)
    pl.xlabel(r'Strain (%)')#,fontsize=14
    pl.ylabel(r'Shear Stress (Pa)') #,fontsize=14
    pl.legend(loc='best')
    pl.tight_layout()
    pl.xlim(0,40)
    pl.ylim(0,30)
    
#Plot differential modulus from data as a function of the stress (or strain). Mean (+/- SD).
#Curve and points of given color c, given shape s, and a given label l    
def plotDiffMod(data, l: str, c: str, s: str):
    all_diffMod = []
    all_stress = []
    all_strain = []
    
    for dataset in data:  # Assuming data is a list of dictionaries
        strain = np.array(dataset['Strain'], dtype='float64')
        stress = np.array(dataset['Shear Stress'], dtype='float64')

        # Calculate differential modulus for each dataset
        diffMod = np.gradient(stress, strain / 100.0)
        
        all_diffMod.append(diffMod)
        all_stress.append(stress)
        all_strain.append(strain)

    # Convert lists to arrays for easy averaging
    all_diffMod = np.array(all_diffMod)
    all_stress = np.array(all_stress)
    all_strain = np.array(all_strain)

    # Calculate average and standard deviation across datasets
    avg_diffMod = np.mean(all_diffMod, axis=0)
    diffMod_SD = np.std(all_diffMod, axis=0)

    # fig, ax = pl.subplots(figsize=(8, 6)) 
    # Plot the average differential modulus
    pl.plot(np.mean(all_stress, axis=0), avg_diffMod, ls='-', color=c, label=l, marker=s, ms=6, linewidth=3)
    # pl.plot(np.mean(all_strain, axis=0), avg_diffMod, ls='-', color=c, label=l, marker=s, ms=5, linewidth=1)

    # Plot the standard deviation as error bars
    # pl.errorbar(np.mean(all_stress, axis=0), avg_diffMod, yerr=diffMod_SD, fmt='none', ecolor=c, elinewidth=0.8)
    # pl.errorbar(np.mean(all_strain, axis=0), avg_diffMod, yerr=diffMod_SD, fmt='none', ecolor=c, elinewidth=0.8)

    pl.xlabel(r'Stress (Pa)', fontsize=22) 
    pl.ylabel(r'K (Pa)',fontsize=22) 
    pl.loglog()
    pl.xticks(fontsize=18)
    pl.yticks(fontsize=18)
    pl.legend(loc='upper left',fontsize=12, frameon=False)
    pl.tight_layout()
    pl.xlim(0.1, 100)
    pl.ylim(10, 300)
    
    

#Plot differential modulus from data as a function of the stress (or strain). Plot all individual curves.
#Curve and points of given color c, given shape s, and a given label l 
def plotDiffMod_all(data, l: str, c: str, s: str):
    all_diffMod = []
    all_stress = []
    all_strain = []
    
    for dataset in data:  # Assuming data is a list of dictionaries
        strain = np.array(dataset['Strain'], dtype='float64')
        stress = np.array(dataset['Shear Stress'], dtype='float64')

        # Calculate differential modulus for each dataset
        diffMod = np.gradient(stress, strain / 100.0)
        
        all_diffMod.append(diffMod)
        all_stress.append(stress)
        all_strain.append(strain)

    # Convert lists to arrays for easy averaging
    all_diffMod = np.array(all_diffMod)
    all_stress = np.array(all_stress)
    all_strain = np.array(all_strain)
    
    fig, ax = pl.subplots(figsize=(8, 6)) 
    for i in range (np.size(data,0)):
        pl.plot(all_stress[i],all_diffMod[i], ls='-', color=c, marker=s, ms=2, linewidth=1)


    ax.set_xlabel(r'Stress (Pa)',fontsize=22)   # Changing x-axis title font size
    ax.set_ylabel(r'K (Pa)',fontsize=22)    # Changing y-axis title font size
    ax.xaxis.set_tick_params(labelsize=18)   # Changing x-axis ticks font size
    ax.yaxis.set_tick_params(labelsize=18)   # Changing y-axis ticks font size
    # pl.xlabel(r'Stress (Pa)') 
    # pl.ylabel(r'K (Pa)') 
    pl.loglog()
    # pl.legend(loc='best')
    pl.tight_layout()
    pl.xlim(0.1, 100)
    pl.ylim(10, 1000)
    
    
# This function returns the average value of the differential modulus of one condition (data containing all the arrays of each sample of the condition)
# at 0.1 Pa stress. (point 11 of the array)   
def DiffMod_init (data):
    all_diffMod = []
    all_stress = []
    all_strain = []
    
    for dataset in data:  # Assuming data is a list of dictionaries
        strain = np.array(dataset['Strain'], dtype='float64')
        stress = np.array(dataset['Shear Stress'], dtype='float64')

        # Calculate differential modulus for each dataset
        diffMod = np.gradient(stress, strain / 100.0)
        
        all_diffMod.append(diffMod)
        all_stress.append(stress)
        all_strain.append(strain)

    # Convert lists to arrays for easy averaging
    all_diffMod = np.array(all_diffMod)
    all_stress = np.array(all_stress)
    all_strain = np.array(all_strain)

    # Calculate average and standard deviation across datasets
    avg_diffMod = np.mean(all_diffMod, axis=0)
    diffMod_init = avg_diffMod[11]
    return diffMod_init 

#Plot differential modulus from data1 as a function of the stress. Differential modulus of data1 normalized by the average value at the initial value 0.1 Pa (point 11) of data 2.
#Curve and points of given color c, given shape s, and a given label l    
def plotDiffMod_norm(data1, data2, l: str, c: str, s: str):
    all_diffMod = []
    all_stress = []
    all_strain = []
    
    for dataset in data1:  # Assuming data is a list of dictionaries
        strain = np.array(dataset['Strain'], dtype='float64')
        stress = np.array(dataset['Shear Stress'], dtype='float64')

        # Calculate differential modulus for each dataset
        diffMod = np.gradient(stress, strain / 100.0)
        
        all_diffMod.append(diffMod)
        all_stress.append(stress)
        all_strain.append(strain)

    # Convert lists to arrays for easy averaging
    all_diffMod = np.array(all_diffMod)
    all_stress = np.array(all_stress)
    all_strain = np.array(all_strain)

    # Calculate average and standard deviation across datasets
    avg_diffMod = np.mean(all_diffMod, axis=0)
    diffMod_SD = np.std(all_diffMod, axis=0)

    avg_diffMod_norm = np.mean(all_diffMod, axis=0)/DiffMod_init(data2)
    diffMod_SD_norm = np.std(all_diffMod, axis=0)/DiffMod_init(data2)
    

    # Plot the average differential modulus
    pl.plot(np.mean(all_stress, axis=0), avg_diffMod_norm, ls='-', color=c, label=l, marker=s, ms=5, linewidth=1)
    # pl.plot(np.mean(all_strain, axis=0), avg_diffMod_norm, ls='-', color=c, label=l, marker=s, ms=5, linewidth=1)

    # Plot the standard deviation as error bars
    # pl.errorbar(np.mean(all_stress, axis=0), avg_diffMod_norm, yerr=diffMod_SD_norm, fmt='none', ecolor=c, elinewidth=0.8)
    # pl.errorbar(np.mean(all_strain, axis=0), avg_diffMod_norm, yerr=diffMod_SD_norm, fmt='none', ecolor=c, elinewidth=0.8)

    pl.xlabel(r'Stress (Pa)') 
    pl.ylabel(r'K / K(CTL)') 
    pl.loglog()
    pl.legend(loc='best')
    pl.tight_layout()
    pl.xlim(0.1, 50)
    pl.ylim(0.1, 10)


##Plot of frequency sweeps and stress ramp, static vs 1% 1Hz during the 1h30 polymerisation 
# plotFreqSweep(CTL_freqsweep,'Storage Modulus','CTL','darkgreen','o')
# plotFreqSweep(CTL_static_freqsweep,'Storage Modulus','CTL - static','limegreen','v')

# plotFreqSweep(MDA_10percent_freqsweep ,'Storage Modulus','10%','darkred','o')
# plotFreqSweep(MDA_10percent_static_freqsweep,'Storage Modulus','10% - static','r','v')

# plotStressRamp(CTL_stressramp,'CTL','darkgreen','o') 
# plotStressRamp(CTL_static_stressramp,'CTL - static','limegreen','v')

# plotStressRamp(MDA_10percent_stressramp,'10%','darkred','o') 
# plotStressRamp(MDA_10percent_static_stressramp,'10% - static','r','v')


##Plot of polymerisation, frequency sweeps and stress ramp for the different gaps 

# plotPolymerisation(CTL_pol, 'Storage Modulus', 'CTL - gap 0.16 mm', 'darkgreen', 'o')
# plotPolymerisation(CTL_g032_pol, 'Storage Modulus', 'CTL - gap 0.32 mm', 'limegreen', 'o')
# plotPolymerisation(CTL_g048_pol, 'Storage Modulus', 'CTL - gap 0.48 mm', 'palegreen', 'o')

# plotFreqSweep(CTL_freqsweep, 'Loss Modulus', 'CTL - gap 0.16 mm', 'darkred', 'o')
# plotFreqSweep(CTL_g032_freqsweep, 'Loss Modulus', 'CTL - gap 0.32 mm', 'r', 'o')
# plotFreqSweep(CTL_g048_freqsweep, 'Loss Modulus', 'CTL - gap 0.48 mm', 'lightcoral', 'o')

# plotStressRamp(CTL_stressramp, 'CTL - gap 0.16 mm', 'darkgreen', 'o')
# plotStressRamp(CTL_g032_stressramp, 'CTL - gap 0.32 mm', 'g', 'o')
# plotStressRamp(CTL_g048_stressramp, 'CTL - gap 0.48 mm', 'palegreen', 'o')

# plotPolymerisation(MDA_10percent_g016_pol, 'Loss Modulus', '10% - gap 0.16 mm','darkred', 'o')
# plotPolymerisation(MDA_10percent_g032_pol, 'Loss Modulus', '10% - gap 0.32 mm', 'r', 'o')
# plotPolymerisation(MDA_10percent_g048_pol, 'Loss Modulus', '10% - gap 0.48 mm', 'lightcoral', 'o')

# plotFreqSweep(MDA_10percent_g016_freqsweep, 'Storage Modulus', '10% - gap 0.16 mm','darkred', 'o')
# plotFreqSweep(MDA_10percent_g032_freqsweep, 'Storage Modulus', '10% - gap 0.32 mm','r', 'o')
# plotFreqSweep(MDA_10percent_g048_freqsweep, 'Storage Modulus', '10% - gap 0.48 mm','lightcoral', 'o')

# plotStressRamp(MDA_10percent_g016_stressramp, '10% - gap 0.16 mm','darkred', 'o')
# plotStressRamp(MDA_10percent_g032_stressramp, '10% - gap 0.32 mm','r', 'o')
# plotStressRamp(MDA_10percent_g048_stressramp, '10% - gap 0.48 mm','lightcoral', 'o')

##  Plot of polymerisation,frequency sweeps and stress ramp for 1% cells in 10 uM Batimastat for 3h in suspension - Experiment on 30/01/2024 and 06/02/2024
# plotPolymerisation(MDA_1percent_pol , 'Storage Modulus','1% cells', 'g', 'o')
# plotPolymerisation(MDA_1percent_10uMBat_pol , 'Loss Modulus','1% cells + 10 uM Batimastat for 3h in suspension', 'c', 'o')

# plotFreqSweep(MDA_1percent_freqsweep , 'Storage Modulus','1% cells', 'g', 'o')
# plotFreqSweep(MDA_1percent_10uMBat_freqsweep , 'Storage Modulus','1% cells + 10 uM Batimastat for 3h in suspension', 'c', 'o')

# plotStressRamp(MDA_1percent_stressramp,'1% cells','g','o') 
# plotStressRamp(MDA_1percent_10uMBat_stressramp,'1% cells + 10 uM Batimastat for 3h in suspension','c','o')


##Plot of polymerisation and frequency sweeps for 1% cells and 1% cells left for 3h in suspension - Experiment on 13/02/2024

# plotPolymerisation(MDA_1percent_pol , 'Storage Modulus','1% cells', 'g', 'o')
# plotPolymerisation(MDA_1percent_3hSusp_pol, 'Storage Modulus','1% cells for 3h in suspension', 'm', 's')

# plotFreqSweep(MDA_1percent_freqsweep, 'Loss Modulus','1% cells', 'g', 'o')
# plotFreqSweep(MDA_1percent_3hSusp_freqsweep, 'Loss Modulus','1% cells for 3h in suspension', 'm', 's')

# plotStressRamp(MDA_1percent_stressramp,'1% cells','g','o') 
# plotStressRamp(MDA_1percent_3hSusp_stressramp,'1% cells for 3h in suspension', 'm', 's')


##Plot of polymerisation and frequency sweeps for 1% cells and 10% cells with 10uM Batimastat for 3h in 2D - Experiment on 19/02/2024 and 05/03/2024

# plotPolymerisation(MDA_1percent_pol , 'Loss Modulus','4% cells', 'g', 'o')
# plotPolymerisation(MDA_1percent_10uMBat2D_pol, 'Loss Modulus','4% cells + 10 uM Batimastat', 'limegreen', 's')
# plotFreqSweep(MDA_1percent_freqsweep, 'Storage Modulus','1% cells', 'g', 'o')
# plotFreqSweep(MDA_1percent_freqsweep, 'Loss Modulus','1% cells', 'g', 'v')
# plotFreqSweep(MDA_1percent_10uMBat2D_freqsweep, 'Storage Modulus','1% cells + 10 uM Batimastat', 'limegreen', 'o')
# plotFreqSweep(MDA_1percent_10uMBat2D_freqsweep, 'Loss Modulus','1% cells + 10 uM Batimastat', 'limegreen', 'v')

# plotStressRamp(MDA_1percent_stressramp,'1% cells','g','o') 
# plotStressRamp(MDA_1percent_10uMBat2D_stressramp,'1% cells + 10 uM Batimastat', 'limegreen', 's') 

# plotStressRamp(MDA_1percent_stressramp,'4% cells','g','o') 
# plotStressRamp(MDA_1percent_10uMBat2D_stressramp,'4% cells + 10 uM Batimastat', 'limegreen', 's') 
# plotDiffMod(MDA_1percent_stressramp,'4% cells','g','o') 
# plotDiffMod(MDA_1percent_10uMBat2D_stressramp,'4% cells + 10 uM Batimastat', 'limegreen', 's') 


# plotPolymerisation(MDA_10percent_pol , 'Loss Modulus','10% cells', 'r', 'o')
# plotPolymerisation(MDA_10percent_10uMBat2D_pol, 'Loss Modulus','10% cells + 10 uM Batimastat', 'lightcoral', 's')
# plotFreqSweep(MDA_10percent_freqsweep , 'Storage Modulus','10% cells', 'r', 'o')
# plotFreqSweep(MDA_10percent_freqsweep , 'Loss Modulus','10% cells', 'r', 'v')
# plotFreqSweep(MDA_10percent_10uMBat2D_freqsweep, 'Storage Modulus','10% cells + 10 uM Batimastat', 'lightcoral', 'o')
# plotFreqSweep(MDA_10percent_10uMBat2D_freqsweep, 'Loss Modulus','10% cells + 10 uM Batimastat', 'lightcoral', 'v')

# plotStressRamp(MDA_10percent_stressramp,'10% cells','r','o') 
# plotStressRamp(MDA_10percent_10uMBat2D_stressramp,'10% cells + 10 uM Batimastat', 'lightcoral', 's') 


##Plot of polymerization, Frequency sweep and stress ramp for CTL, 0.4, 4, 20% cells
# pl.figure(figsize=(8, 6))
# plotPolymerisation(CTL_pol, 'Storage Modulus', 'CTL', 'grey', 'o')       ##Includes also the new batch data 
# plotPolymerisation(CTL_pol_newbatch, 'Storage Modulus', 'CTL - new batch', 'c', 'o')
# plotPolymerisation(MDA_0pt1percent_pol , 'Storage Modulus','0.4%', 'g', 'o')
# plotPolymerisation( MDA_0pt1percent_pol_new , 'Storage Modulus','0.4% - new', 'gold', 'o')
# plotPolymerisation(MDA_0pt1percent_pol_tot, 'Storage Modulus','0.4%', 'g', 'o')
# plotPolymerisation(MDA_1percent_pol , 'Storage Modulus','4%', 'b', 'o')
# plotPolymerisation(MDA_10percent_g016_pol, 'Storage Modulus', '20% cells','red', 'o')
# plotPolymerisation(MDA_20percent_new_pol, "Storage Modulus", '20%', 'r', 'o') ## here only new formulation array is used!!!
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Figure 1/Gprime-MDAMB231.png', dpi=300, bbox_inches='tight')

# plotPolymerisation(CTL_pol, 'Loss Modulus', 'CTL', 'grey', 'o')
# plotPolymerisation(MDA_0pt1percent_pol , 'Loss Modulus','0.4%', 'g', 'o')
# # plotPolymerisation(MDA_0pt1percent_pol_tot, 'Loss Modulus','0.4%', 'g', 'o')
# plotPolymerisation(MDA_1percent_pol , 'Loss Modulus','4%', 'b', 'o')
# # plotPolymerisation(MDA_A_10percent_g016_pol, 'Storage Modulus', '20% cells','red', 'o')
# plotPolymerisation(MDA_20percent_new_pol, "Loss Modulus", '20%', 'r', 'o') 
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Figure 1/Gdoubleprime-MDAMB231 - old 0.4%.png', dpi=300, bbox_inches='tight')

# plotFreqSweep(CTL_freqsweep, 'Storage Modulus', 'CTL', 'c', 'o')
# plotFreqSweep(MDA_0pt1percent_freqsweep , 'Storage Modulus','0.1% cells', 'y', 'o')
# plotFreqSweep(MDA_1percent_freqsweep , 'Storage Modulus','4% cells', 'g', 'o')
# plotFreqSweep(MDA_10percent_g016_freqsweep, 'Storage Modulus', '20% cells','red', 'o')

# plotFreqSweep(CTL_freqsweep, 'Storage Modulus', 'CTL', 'c', 'v')
# plotFreqSweep(CTL_freqsweep, 'Loss Modulus', 'CTL', 'c', 'v')
# plotFreqSweep(MDA_0pt1percent_freqsweep , 'Loss Modulus','0.1% cells', 'y', 'v')
# plotFreqSweep(MDA_1percent_freqsweep , 'Loss Modulus','4% cells', 'g', 'v')
# plotFreqSweep(MDA_10percent_g016_freqsweep, 'Loss Modulus', '20% cells','red', 'v')

pl.figure(figsize=(8, 6))
# plotStressRamp(CTL_stressramp, 'CTL', 'grey', 'o')
# plotStressRamp(CTL_stressramp_newbatch, 'CTL - new batch', 'grey', 'o')
plotDiffMod(CTL_stressramp, 'CTL', 'grey', 'o')
# plotDiffMod(CTL_stressramp_newbatch, 'CTL - new batch ', 'grey', 'o')
# plotStressRamp(MDA_0pt1percent_stressramp_tot , '0.4% cells', 'y', 'o')
# plotStressRamp(MDA_1percent_stressramp ,'4% cells', 'g', 'o')
# plotStressRamp(MDA_10percent_g016_stressramp, '20% cells','red', 'o')
# plotStressRamp(MDA_20percent_new_stressramp, '20% cells','red', 'o') ## here only new formulation array is used!!!
plotDiffMod(MDA_0pt1percent_stressramp, '0.4%', 'g', 'o')
plotDiffMod(MDA_1percent_stressramp ,'4%', 'b', 'o')
# plotDiffMod(MDA_1percent_10uMBleb2D_stressramp, '4% + 10 uM Blebbistatin', 'deeppink', 'o')
plotDiffMod(MDA_20percent_new_stressramp, '20%','r', 'o') ## here only new formulation array is used!!!
# plotDiffMod(MDA_20percent_DMSO_stressramp ,'20% - 0.2% DMSO', 'orange', 'o')
# plotDiffMod(MDA_20percent_10uMBleb2D_stressramp,'20% + 10 uM Blebbistatin', 'k', 'o')
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Figure 1/K-MDAMB231.png', dpi=300, bbox_inches='tight')

#Plot of polymerization, Frequency sweep and stress ramp for 1% CTL and 1% with 10 uM Blebbistatin for 3 hours in 2D
# plotPolymerisation(CTL_pol, 'Storage Modulus','CTL', 'c', 'o')
# plotPolymerisation(MDA_1percent_pol, 'Loss Modulus','4% cells', 'g', 'o')
# plotPolymerisation(MDA_1percent_10uMBleb2D_pol, 'Storage Modulus', '4% - 10 uM Blebbistatin', 'deeppink', 'o')

# plotPolymerisation(MDA_1percent_10uMBleb2D_pol, 'Loss Modulus', '4% - 10 uM Blebbistatin', 'deeppink', 'o')
# plotPolymerisation(MDA_1percent_CTLBleb_pol, 'Storage Modulus','4% cells', 'g', 'o')


# plotFreqSweep(MDA_1percent_10uMBleb2D_freqsweep, 'Storage Modulus', '1% + 10 uM Blebbistatin', 'deeppink', 'o')
# plotFreqSweep(MDA_1percent_10uMBleb2D_freqsweep, 'Loss Modulus', '1% + 10 uM Blebbistatin', 'deeppink', 'v')

# plotFreqSweep(MDA_1percent_CTLBleb_freqsweep,'Loss Modulus', '1%', 'g', 'o')

# plotFreqSweep(MDA_1percent_freqsweep, 'Storage Modulus','1%', 'g', 'o')
# plotFreqSweep(MDA_1percent_freqsweep, 'Loss Modulus','1%', 'g', 'v')

# plotStressRamp(CTL_stressramp, 'CTL', 'c', 'o')
# plotStressRamp(MDA_1percent_stressramp, '4%', 'g', 'o')
# plotStressRamp(MDA_1percent_10uMBleb2D_stressramp, '4% + 10 uM Blebbistatin', 'deeppink', 'o')
# plotStressRamp(MDA_1percent_CTLBleb_stressramp, '4%', 'g', 'o')
# plotDiffMod(MDA_1percent_10uMBleb2D_stressramp, '4% + 10 uM Blebbistatin', 'deeppink', 'o')


# #Plot of polymerization curves for 20% cells in atelocollagen 4 mg/mL with 0.2% DMSO or 10 uM Blebbistatin for 3h in 2D
# plotPolymerisation(MDA_20percent_new_pol , 'Loss Modulus', '20%', 'r', 'o')
# plotPolymerisation(MDA_20percent_DMSO_pol , 'Loss Modulus', '20% - 0.2% DMSO', 'orange', 'o')
# plotPolymerisation(MDA_20percent_10uMBleb2D_pol , 'Storage Modulus', '20% - 10 uM Blebbistatin', 'k', 'o')
# plotPolymerisation(MDA_20percent_10uMBleb2D_pol , 'Loss Modulus', '20% - 10 uM Blebbistatin', 'orange', 'k')


# plotStressRamp(MDA_20percent_new_stressramp ,'20%', 'r', 'o')
# plotDiffMod(MDA_20percent_DMSO_stressramp ,'20% - 0.2% DMSO', 'orange', 'o')
# plotStressRamp(MDA_20percent_10uMBleb2D_stressramp,'20% - 10 uM Blebbistatin', 'k', 'o')
# plotDiffMod(MDA_20percent_10uMBleb2D_stressramp,'20% + 10 uM Blebbistatin', 'k', 'o')



##Plot of polymerization curves and stress ramp for CTL (+EDTA), 4%, 20% cells in atelocollagen 4 mg/mL 5mM EDTA for 10 min 
# plotPolymerisation(CTL_EDTA_pol, 'Storage Modulus', 'CTL - 5mM EDTA', 'steelblue', 'o')
# plotPolymerisation(MDA_4percent_EDTA_pol, 'Storage Modulus', '4% - 5mM EDTA', 'plum', 'o')
# plotPolymerisation(MDA_20percent_EDTA_pol , 'Storage Modulus', '20% - 5mM EDTA', 'purple', 'o')

##Plot of polymerization, and stress ramp for 4% and 4% EDTA
# plotPolymerisation(CTL_pol, 'Storage Modulus','CTL', 'c', 'o')
# plotPolymerisation(MDA_1percent_pol, 'Storage Modulus','4% cells', 'g', 'o')
# plotPolymerisation(MDA_4percent_EDTA_pol, 'Storage Modulus', '4% - 5mM EDTA', 'plum', 'o')

# plotDiffMod(CTL_stressramp,'CTL', 'c', 'o')
# plotDiffMod(MDA_1percent_stressramp,'4% cells', 'g', 'o')
# plotDiffMod(MDA_4percent_EDTA_stressramp, '4% - 5mM EDTA', 'plum', 'o')

##Plot of polymerization, and stress ramp for 20% and 20% EDTA
# plotPolymerisation(CTL_pol, 'Storage Modulus','CTL', 'c', 'o')
# plotPolymerisation(MDA_20percent_pol, 'Storage Modulus','20% cells', 'r', 'o')
# plotPolymerisation(MDA_20percent_EDTA_pol, 'Storage Modulus', '20% - 5mM EDTA', 'purple', 'o')

# plotDiffMod(CTL_stressramp,'CTL', 'c', 'o')
# plotDiffMod(MDA_20percent_stressramp,'20% cells', 'r', 'o')
# plotDiffMod(MDA_20percent_EDTA_stressramp, '20% - 5mM EDTA', 'purple', 'o')



# plotStressRamp(CTL_EDTA_stressramp,'CTL - 5mM EDTA', 'steelblue', 'o')
# plotStressRamp(MDA_4percent_EDTA_stressramp,  '4% - 5mM EDTA', 'purple', 'o')
# plotStressRamp(MDA_20percent_EDTA_stressramp , '20% - 5mM EDTA', 'orange', 'o')
# plotDiffMod(CTL_EDTA_stressramp,'CTL - 5Mm EDTA', 'steelblue', 'o')
# plotDiffMod(MDA_4percent_EDTA_stressramp,  '4% - 5mM EDTA', 'purple', 'o')
# plotDiffMod(MDA_20percent_EDTA_stressramp , '20% - 5mM EDTA', 'orange', 'o')


##Plots with PAA beads (4 and 20%) and relative CTL of 12/11 and 14/11
# plotPolymerisation(CTL_beads_pol, 'Storage Modulus', 'CTL', 'c', 'o')
# plotPolymerisation(PAbeads_4percent_pol, 'Storage Modulus', '4% PAA beads', 'teal', 'o')
# plotPolymerisation(PAbeads_20percent_pol, 'Storage Modulus', '20% PAA beads', 'brown', 'o')



# plotStressRamp(PAbeads_20percent_stressramp, '20% PAA beads', 'brown', 'o')
# plotStressRamp(PAbeads_4percent_stressramp, '4% PAA beads', 'teal', 'o')

# plotDiffMod(CTL_beads_stressramp, 'CTL', 'c', 'o')
# plotDiffMod(PAbeads_4percent_stressramp, '4% PAA beads', 'teal', 'o')
# plotDiffMod(PAbeads_20percent_stressramp, '20% PAA beads', 'brown', 'o')



##Plots with HDF 0.4, 4 and CTL with HDF media 
# pl.figure(figsize= (8,6))
# plotPolymerisation(CTL_HDF_pol, 'Storage Modulus', 'CTL', 'grey', 'o')
# plotPolymerisation(HDF_04percent_pol, 'Storage Modulus', ' 0.4%', 'g', 'o')
# plotPolymerisation(HDF_4percent_pol, 'Storage Modulus', '4% ', 'b', 'o')
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Figure 1/Gprime-HDF.png', dpi=300, bbox_inches='tight')


# plotStressRamp(CTL_HDF_stressramp, 'CTL - HDF', 'indigo', 'o')
# plotStressRamp(HDF_04percent_stressramp, ' 0.4% HDF', 'orchid', 'o')
# plotStressRamp(HDF_4percent_stressramp, '4% HDF', 'palevioletred', 'o')

# plotDiffMod(CTL_HDF_stressramp, 'CTL', 'grey', 'o')
# plotDiffMod(HDF_04percent_stressramp, ' 0.4%', 'g', 'o')
# plotDiffMod(HDF_4percent_stressramp, '4%', 'b', 'o')
# pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Figure 1/K-HDF.png', dpi=300, bbox_inches='tight')
##Less used plots (Chondrocytes, TELOCollagen, PPE lab, 2mg/mL collagen)

##Plot of polymerization Frequency sweep and stress ramp for CTL collagen - PPE lab
# plotPolymerisation(CTL_pol_PPE, 'Storage Modulus','CTL - DSR', 'skyblue', 'v')
# plotFreqSweep(CTL_freqsweep_PPE, 'Storage Modulus', 'CTL- DSR', 'skyblue', 'o')
# plotFreqSweep(CTL_freqsweep_PPE, 'Loss Modulus', 'CTL- DSR', 'skyblue', 'v')
# plotStressRamp(CTL_stressramp_PPE, 'CTL - DSR', 'skyblue', 'v')


##Plot of polymerization Frequency sweep and stress ramp for 1% cells - PPE  lab
# plotPolymerisation(MDA_1percent_pol_PPE, 'Storage Modulus','1% cells - DSR', 'lightgreen', 'v')
# plotFreqSweep(MDA_1percent_freqsweep_PPE, 'Loss Modulus', '1% cells - DSR', 'lightgreen', 'v')
# plotStressRamp(MDA_1percent_stressramp_PPE, '1% cells - DSR', 'lightgreen', 'v')


# #Plot of polymerization, Frequency sweep and stress ramp for CTL, 0.1, 1, 10% cells - Bovine chondrocytes
# plotPolymerisation(CTL_BC_pol, 'Loss Modulus', 'CTL', 'c', 'o')
# plotPolymerisation(BC_0pt1percent_pol, 'Loss Modulus','0.1% cells', 'y', 'o')
# plotPolymerisation(BC_1percent_pol , 'Loss Modulus','1% cells', 'g', 'o')
# plotPolymerisation(BC_10percent_pol, 'Loss Modulus', '10% cells','red', 'o')

# plotFreqSweep(CTL_BC_freqsweep, 'Loss Modulus', 'CTL', 'c', 'o')
# plotFreqSweep(BC_0pt1percent_freqsweep, 'Loss Modulus','0.1% cells', 'y', 'o')
# plotFreqSweep(BC_1percent_freqsweep, 'Loss Modulus','1% cells', 'g', 'o')
# plotFreqSweep(BC_10percent_freqsweep, 'Loss Modulus', '10% cells','red', 'o')


##Plot of polymerization, Frequency sweep and stress ramp for Telocollagen CTL, 4% and 20% cells 
# plotPolymerisation(CTL_telo_pol, 'Storage Modulus', 'CTL-telo', 'b', 'o')
# plotPolymerisation(MDA_4percent_telo_pol, 'Storage Modulus', '4% cells- Telo', 'y', 'o')
# plotPolymerisation(MDA_20percent_telo_pol, 'Storage Modulus', '20% cells- Telo', 'salmon', 'o')

# plotFreqSweep(CTL_telo_freqsweep, 'Storage Modulus', 'CTL-telo', 'b', 'o')
# plotFreqSweep(CTL_telo_freqsweep, 'Loss Modulus', 'CTL-telo', 'b', 'v')
# plotFreqSweep(MDA_4percent_telo_freqsweep, 'Storage Modulus', '4% cells- Telo', 'y', 'o')
# plotFreqSweep(MDA_percent_telo_freqsweep, 'Loss Modulus', '4% cells- Telo', 'y', 'v')
# plotFreqSweep(MDA_20percent_telo_freqsweep, 'Storage Modulus', '20% cells- Telo', 'salmon', 'o')
# plotFreqSweep(MDA_20percent_telo_freqsweep, 'Loss Modulus', '20% cells- Telo', 'salmon', 'v')

# plotStressRamp(CTL_telo_stressramp, 'CTL- telo', 'b', 'o')
# plotStressRamp(MDA_4percent_telo_stressramp, '4% cells- Telo', 'y', 'o')
# plotStressRamp(MDA_20percent_telo_stressramp, '20% cells- Telo', 'salmon', 'o')
# plotDiffMod(CTL_telo_stressramp, 'CTL- telo', 'b', 'o') 

#Plot of polymerization, Frequency sweep and stress ramp for 20% NEW FORMULATION vs 10% OLD FORMULATION
# plotPolymerisation(MDA_20percent_new_pol, 'Storage Modulus', '20% new formulation', 'orange', 'o')
# plotPolymerisation(MDA_10percent_pol, 'Storage Modulus', '10% old formulation', 'red', 'o')
# plotFreqSweep(MDA_20percent_new_freqsweep, 'Storage Modulus', '20% new formulation', 'orange', 'o')
# plotFreqSweep(MDA_10percent_freqsweep, 'Storage Modulus', '10% old formulation', 'red', 'o')
# plotStressRamp(MDA_20percent_new_stressramp, '20% new formulation', 'orange', 'o')
# plotStressRamp(MDA_10percent_stressramp, '10% old formulation', 'red', 'o')


#Plot of polymerization, Frequency sweep and stress ramp for CTL, 4%,20% atelocollagen 2 mg/mL
# plotPolymerisation(CTL_2mgmL_pol, 'Loss Modulus', 'CTL', 'c', 'o')
# plotPolymerisation(MDA_0pt1percent_pol , 'Loss Modulus','0.1% cells', 'y', 'o')
# plotPolymerisation(MDA_4percent_2mgmL_pol , 'Loss Modulus','4% cells', 'g', 'o')
# plotPolymerisation(MDA_20percent_2mgmL_pol, 'Loss Modulus', '20% cells','red', 'o')
# plotStressRamp(CTL_2mgmL_stressramp, 'CTL', 'c', 'o')
# # plotPolymerisation(MDA_0pt1percent_pol , 'Loss Modulus','0.1% cells', 'y', 'o')
# plotStressRamp(MDA_4percent_2mgmL_stressramp ,'4% cells', 'g', 'o')
# plotStressRamp(MDA_20percent_2mgmL_stressramp,'20% cells','red', 'o')


# #Plot of individual polymerization curves for 4% cells in atelocollagen 2 mg/mL
# plotPolymerisation_one(data_250624_rheo1[0] , 'Storage Modulus','4% cells', 'g', 'o')
# plotPolymerisation_one(data_250624_rheo1[6] , 'Storage Modulus','4% cells', 'g', 'o')
# plotPolymerisation_one(data_250624_rheo2[0] , 'Storage Modulus','4% cells', 'g', 'o')
# plotPolymerisation_one(data_250624_rheo2[6] , 'Storage Modulus','4% cells', 'g', 'o')


## Plot polymerization for 4% and 20% cells with or without incubation with 10 ug/mL anti-beta1 integrin blockin antibody
##and the corresponding CTL of those days


#plotPolymerisation(CTL_pol, 'Storage Modulus','CTL', 'c', 'o')
# plotPolymerisation(CTL_pol_integblock, 'Storage Modulus','CTL- p5d2', 'mediumblue', 'o')

# plotPolymerisation(CTL_pol, 'Storage Modulus','CTL', 'c', 'o')
# plotPolymerisation(CTL_pol_integblock, 'Storage Modulus','CTL - p5d2', 'mediumblue', 'o')


# plotPolymerisation(MDA_4percent_integblock_pol, 'Storage Modulus','4% cells - p5d2', 'lightgreen', 'o')
# plotPolymerisation(MDA_1percent_pol , 'Storage Modulus','4% cells', 'g', 'o')

# plotPolymerisation(MDA_20percent_integblock_pol, 'Storage Modulus','20% cells - p5d2', 'orange', 'o')
# plotPolymerisation(MDA_20percent_new_pol, 'Storage Modulus','20% cells', 'r', 'o')

## Plot polymerization for 4% and 20% cells with or without incubation with 10 ug/mL anti-beta1 integrin blockin antibody
##normalized by the final value of the corresponding CTL of those days

# plotPolymerisation_norm(CTL_pol,CTL_pol, 'Storage Modulus','CTL - p5d2', 'c', 'o')
# plotPolymerisation_norm(CTL_pol_integblock, CTL_pol_integblock, 'Storage Modulus','CTL - p5d2', 'mediumblue', 'o')

# plotPolymerisation_norm(MDA_1percent_pol, CTL_pol, 'Storage Modulus','4% cells', 'g', 'o')
# plotPolymerisation_norm(MDA_4percent_integblock_pol, CTL_pol_integblock, 'Storage Modulus','4% cells - p5d2', 'lightgreen', 'o')

# plotPolymerisation_norm(MDA_20percent_new_pol, CTL_pol, 'Storage Modulus','20% cells', 'r', 'o')
# plotPolymerisation_norm(MDA_20percent_integblock_pol, CTL_pol_integblock, 'Storage Modulus','20% cells - p5d2', 'orange', 'o')




#Plot of differential modulus. For control conditions and P5D2, 4% and 20%.
# plotDiffMod(CTL_stressramp,'CTL', 'skyblue', 'o')
# plotDiffMod(CTL_stressramp_integblock,'CTL - p5d2', 'pink', 'o')
# plotDiffMod(MDA_1percent_stressramp,'4% cells', 'purple', 'o')
# plotDiffMod(MDA_4percent_integblock_stressramp,'4% cells - p5d2', 'orange', 'o')
# plotDiffMod(MDA_20percent_stressramp,'20% cells', 'r', 'o')
# plotDiffMod(MDA_20percent_integblock_stressramp,'20% cells - p5d2','orange', 'o')


#Plot of differential modulus normalized by differential modulus at 0.1 Pa of internal CTL. For control conditions and P5D2, 4% and 20%.
# plotDiffMod_norm(CTL_stressramp,CTL_stressramp,'CTL', 'c', 'o')
# plotDiffMod_norm(CTL_stressramp_integblock,CTL_stressramp_integblock,'CTL - p5d2', 'b', 'o')
# plotDiffMod_norm(MDA_1percent_stressramp,CTL_stressramp,'4% cells', 'green', 'o')
# plotDiffMod_norm(MDA_4percent_integblock_stressramp,CTL_stressramp_integblock,'4% cells - p5d2', 'lightgreen', 'o')
# plotDiffMod_norm(MDA_20percent_stressramp,CTL_stressramp,'20% cells', 'r', 'o')
# plotDiffMod_norm(MDA_20percent_integblock_stressramp,CTL_stressramp_integblock,'20% cells - p5d2','orange', 'o')

#Plot all individual curves for Polymerization curves
#plotPolymerisation_all(CTL_pol, 'Storage Modulus','CTL', 'c', 'o')
# plotPolymerisation_all(MDA_0pt1percent_pol_tot, 'Storage Modulus','CTL', 'y', 'o')
# plotPolymerisation_all(MDA_1percent_pol, 'Storage Modulus','CTL', 'g', 'o')
# plotPolymerisation_all(MDA_20percent_new_pol, 'Storage Modulus','CTL', 'r', 'o')


#Plot all individual curves for Differential modulus curves
# plotDiffMod_all(CTL_stressramp,'CTL', 'g', 'o') 

# plotDiffMod_all(CTL_stressramp,'CTL', 'c', 'o') 
# plotDiffMod(CTL_stressramp,'CTL', 'c', 'o')

#Plot of differential modulus normalized by differential modulus at 0.1 Pa of internal CTL and 0.4% old, new and old + new 

# plotDiffMod_norm(MDA_0pt1percent_stressramp,MDA_0pt1percent_stressramp,'0.4% cells ', 'y', 'o')
# plotDiffMod_norm(MDA_0pt1percent_stressramp_new,MDA_0pt1percent_stressramp_new,'0.4% cells - new  ', 'goldenrod', 'o')
# plotDiffMod_norm(MDA_0pt1percent_stressramp_tot,MDA_0pt1percent_stressramp_tot,'0.4% cells - all  ', 'gold', 'o')
