# -*- coding: utf-8 -*-
"""
Created on Fri Aug  9 13:08:50 2024

@author: inagle
"""

import numpy as np

# For one sample, this function finds the maximum of the polymerization curve on the storage modulus.
# Then computes the percentage of decrease by substracting the maximum value by the finale value,
#dividing by the maximal value and multiplying by 100 to get the percentage.
# If the percentage of decrease is below a certain percentage set by "value", 
# then the function gives 0 as an output, otherwise it gives 1.
def percent_decrease (data,value):
    data = np.array(data[:]['Storage Modulus'], dtype='float64')
    percentage = ((max(data)- data[np.size(data)-1])/max(data))*100
    out = 0
    if percentage>value:
        out = 1
    else :
        out = 0
    return out 

# This function uses the percent_decrease function to assess if there is a decrease on the polymerization curve
# on the storage modulus above a certain percentage value (value) on the collection of all samples from
# one condition, grouped in an array. 
# The output is a table of 0 and 1. Each number corresponds to one sample
# 1 corresponds to a decrease higher than the value percentage for that sample
# 0 corresponds to a decrease lower than the value percentage for that sample
def polym_decrease (array,value):
    result = np.zeros(np.size(array,0))
    for i in range (np.size(array,0)-1):
        result[i] = percent_decrease(array[i][:][:],value)
    return result