# -*- coding: utf-8 -*-
"""
Created on Thu Jan 16 12:34:35 2025

@author: mtavasso
"""

import numpy as np
import matplotlib.pyplot as pl
import seaborn as sns
from scipy.optimize import fsolve
from scipy.ndimage import gaussian_filter1d
from scipy import stats
from scipy.stats import mannwhitneyu
from scipy.stats import ttest_ind

# from apMethods_Pol_FreqSweep_StressRamp_V5 import *
from apMethods_ReadRheo_Datacollection import *


def gamma_onset(data):
    all_strain_onset = []
    all_stress_onset = []

    for dataset in data:  # Assuming data is a list of dictionaries
        strain = np.array(dataset['Strain'], dtype='float64')
        stress = np.array(dataset['Shear Stress'], dtype='float64')

        # Calculate differential modulus (differential stress per differential strain)
        diffMod = np.gradient(stress, strain)
        
        # Apply smoothing to the differential modulus to reduce noise
        diffMod_smooth = gaussian_filter1d(diffMod, 2)
        
        # Calculate the first and second derivatives of the smooth differential modulus
        dK = np.gradient(diffMod_smooth)
        d2K = np.gradient(dK)
        
        # Finding the first maximum of the differential modulus curve (second derivative positive)
        ind = 14
        for j in range(ind, len(d2K)):  # Start at index 14 (around 0.2 Pa)
            if d2K[j-1] < d2K[j] :  # Finding the maximum
                ind = j
            if  d2K[j]>d2K[j+1]:
                break #Stop at the first local maximum
        strain_onset = strain[ind]
        stress_onset = stress[ind]
        all_strain_onset.append(strain_onset)
        all_stress_onset.append(stress_onset)

        # Plotting the differential modulus and the onset point
        pl.figure(figsize=(8, 6))
        pl.plot(stress, d2K, color='c', label="Differential Modulus")
        pl.plot(stress_onset, d2K[ind], marker='o', color='y', label="Onset Point")
        
        pl.xscale('log')
        pl.yscale('log')
        pl.xlabel('Stress (Pa)')
        pl.ylabel('Modulus / Derivative')
        pl.title('Differential Modulus and Onset Point')
        pl.legend()
        pl.show()

    return (all_strain_onset, all_stress_onset)

# Example of how you would call this function with your data
print(gamma_onset(MDA_1percent_stressramp))
# print(f"Onset Point: Stress = {stress_onset}, Strain = {strain_onset}")