# -*- coding: utf-8 -*-
"""
Created on Tue Feb 25 14:12:36 2025

@author: mtavasso
"""

# -*- coding: utf-8 -*-
import numpy as np
import matplotlib.pyplot as pl
import seaborn as sns
from scipy.optimize import fsolve
from scipy.ndimage import gaussian_filter1d
from scipy import stats
from scipy.stats import mannwhitneyu
from scipy.stats import ttest_ind

import pandas
import io

import codecs
from apMethods_ReadRheo_Datacollection_mt import *

import matplotlib
import matplotlib.pyplot as pl

# Martin Krzywinski's 24-color, color-blind-friendly palette
color_blind_palette = {
    "orange": "#E69F00",  # Orange
    "skyblue":"#56B4E9",  # Sky Blue
    "g": "#009E73",  # Green
    "y":"#F0E442",  # Yellow
    "b": "#0072B2",  # Blue
    "darkorange":"#D55E00",  # Dark Orange
    "pink":"#CC79A7",  # Pink
    "grey":"#999999",  # Grey
    "lightyellow":"#F7CB4D",  # Light Yellow
    "lightblue":"#ADD8E6",  # Light Blue
    "lightgreen":"#90EE90",  # Light Green
    "lightpink":"#FFB6C1",  # Light Pink
    "darkred":"#A52A2A",  # Brown
    "purple":"#800080",  # Purple
    "olive":"#808000",  # Olive
    "orange":"#FFA500",  # Orange
    "steelblue":"#4682B4",  # Steel Blue
    "brown":"#8B4513",  # Saddle Brown
    "cadetblue": "#5F9EA0",  # Cadet Blue
    "r":"#CD5C5C"   # Indian Red
}

# Set default color cycle
def resolve_color(color):
    """Resolve shorthand color names to Tableau hex codes."""
    return color_blind_palette.get(color, color)  # Default to original if shorthand not found

# Override the default plot function to use custom color
original_plot = pl.plot

def patched_plot(*args, **kwargs):
    if 'color' in kwargs:
        kwargs['color'] = resolve_color(kwargs['color'])
    return original_plot(*args, **kwargs)

pl.plot = patched_plot 



# Function that gives as an output the list of onset times for an array containing all the arrays of samples for one condition.
# The onset time is defined by the intersection between the tangent of the curve were the first derivative is maximal and the t axis. 
# The input arguments are "data" the array containing all the arrays of each sample of the condition and sigma the value used for the gaussian filtering.
# The function can plot the storage modulus as a function of time, the first derivative as a function of time, the second derivative as a function of time
# with the onset time point visible to verify the computation. 
def onset_time(data,sigma):
    #Defining the time axis
    time = np.array(data[0][:]['Time'], dtype='float64')
    dt = time[1]-time[0]
    # Initializing the list that will contain all teh calculated onset times
    onset_time = np.zeros(np.size(data,0))
    for i in range (np.size(data,0)) :
        #Load the data
        sample = np.array(data[i][:]['Storage Modulus'], dtype='float64')
        #Smooth the curve
        sample_smooth = gaussian_filter1d(sample, sigma)
        #Compute first derivative
        dy = np.gradient(sample_smooth)
        #Compute second derivative
        d2y = np.gradient(dy)
        # Find the max of the first derivative and the corresponding index ind_tM and time tM
        ind_tM = np.argmax(dy)
        tM = ind_tM *dt
        #Equation of the tangent to the curve where the first derivative is maximal.
        tang = dy[ind_tM]/dt*(time-tM)+sample_smooth[ind_tM]
        #Finding the intersection of that tangent with the abcissa axis (t axis). It gives a value t1 corresponding to the onset time.
        ind_t1=0
        for j in range (1,len(dy)):
            if tang [j-1]*tang[j]< 0:  # Check if the product of consecutive y values is negative
                ind_t1 = j-1
        t1 = ind_t1*dt
        onset_time[i] = t1
    
        # Plotting the curve with the tangent, its intersection to the t axis and with the tangent to the Plateau. 
        # pl.figure()
        # pl.plot(time,sample_smooth)
        # pl.plot(time,tang, 'k--', lw = 1)
        # pl.title('Sample ' + str(i))
        # # pl.plot(time_pol,tang2, 'k--', lw = 1)
        # pl.plot(tM,sample_smooth[ind_tM],marker='o', color = 'y')
        # pl.plot(t1,0,marker='o', color = 'g')
        # # pl.plot(t2,tang[ind_t2],marker='o', color = 'r')
        # pl.ylim(0,120)
        # pl.xlabel(r'Time (s)')
        # pl.ylabel(r'$G^{\prime}$ $(Pa)$')
      
        # # Plotting the first derivative and the max of the first derivative at tM
        # pl.figure()
        # pl.plot(time,dy)
        # pl.plot(tM,dy[ind_tM],marker='o', color = 'y')

        # # Plotting the second derivative and the max of the first derivative at tM corresponding to where the second derivative is equal to zero
        # pl.figure()
        # pl.plot(time,d2y)
        # pl.plot(tM,d2y[ind_tM],marker='o', color = 'y')
        
    return onset_time
            


sigma = 2
CTL_onset_time = onset_time(CTL_pol,sigma)
CTL_onset_time = CTL_onset_time[2:] 
##MT: Noticed that the sigma is only for CTL and 20%! Does it change things? To check. I added it to the other ones, the boxplots change a bit 
# print(CTL_onset_time)


MDA_1percent_10uMBleb2D_onset_time = onset_time(MDA_1percent_10uMBleb2D_pol,sigma)

MDA_20percent_10uMBleb2D_onset_time = onset_time(MDA_20percent_10uMBleb2D_pol,sigma)


all = [CTL_onset_time, MDA_1percent_10uMBleb2D_onset_time, MDA_20percent_10uMBleb2D_onset_time]
# Custom color palette
palette_colors = [
    color_blind_palette["grey"],  # CTL
    color_blind_palette["skyblue"],     # 0.4%
    color_blind_palette["orange"]      # 20%
]
    
#Perform pairwise Mann-Whitney U tests (Note:Comparing only CTL with other conditions with cells!!!)
p_values = []
for i in range(1, len(all)):  # Start from 1 to skip comparing CTL with itself
    _, p = mannwhitneyu(all[0], all[i], alternative='two-sided')
    p_values.append(p)

# Function to map p-value to annotation text
def get_p_value_annotation(p_value):
    if p_value < 0.001:
        return "***"
    elif p_value < 0.01:
        return "**"
    elif p_value < 0.05:
        return "*"
    else:
        return "ns"

# Plot
pl.figure(figsize=(8, 6))
boxprops = {'linewidth': 2}         # Thicker box lines
whiskerprops = {'linewidth': 2}     # Thicker whisker lines
capprops = {'linewidth': 3}         # Thicker cap lines
medianprops = {'color': 'black', 'linewidth': 3}  # Thicker median line
for spine in pl.gca().spines.values():
        spine.set_linewidth(1.5)
        

# Create the boxplot and swarmplot
ax = sns.boxplot(data=all, palette= palette_colors, width=0.5, 
                  boxprops=boxprops, whiskerprops=whiskerprops, 
                  capprops=capprops, medianprops=medianprops, 
                  showfliers=False)
sns.swarmplot(data=all, color="black", size = 6)

# Add labels
ax.set_xticks(range(len(all)))  # Set tick positions
ax.set_xticklabels(['CTL', '4%-bleb', '20%-bleb'], fontsize=30)
ax.set_ylabel(r'$t_{onset}$ (s)', fontsize=30)
pl.yticks(fontsize=24)
pl.tick_params(axis='both', which='major', length=8, width=2)  # Make ticks longer and thicker
pl.tick_params(axis='both', which='minor', length=6, width=2) 
#Set the range of the plot
ax.set_ylim(0,1000)

# Annotate statistical significance
y_max = max([np.max(data) for data in all])  # Find the maximum value in the dataset
line_height = y_max + 1  # Starting height for annotation lines
spacing = 20  # Spacing between annotation lines

for i, p_value in enumerate(p_values):
    annotation = get_p_value_annotation(p_value)
    x1, x2 = 0, i + 1  # Positions of the two groups being compared
    y = line_height + i * spacing  # Adjust line height for each comparison
    ax.plot([x1, x1, x2, x2], [y, y, y, y], color="black", lw=1.5)  # Draw line
    ax.text((x1 + x2) / 2, y - 20, annotation, ha="center", va="bottom", fontsize=18, color="black")  # Add annotation
    line_height += spacing  # Increment the base line height for the next pair
    
# Finalize plot
pl.tight_layout()
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 4/Onsettime-Bleb.png', dpi=300, bbox_inches='tight')
pl.show()



