# -*- coding: utf-8 -*-
"""
Created on Mon Feb 17 15:04:39 2025

@author: mtavasso
"""

# -*- coding: utf-8 -*-
"""
Created on Thu Jan 16 10:19:59 2025

@author: mtavasso
"""

import numpy as np
import matplotlib.pyplot as pl
import seaborn as sns
import matplotlib.ticker as ticker
from scipy.optimize import fsolve
from scipy.ndimage import gaussian_filter1d
from scipy import stats
from scipy.stats import mannwhitneyu
from scipy.stats import ttest_ind

from apMethods_ReadRheo_Datacollection_mt import *


import matplotlib
import matplotlib.pyplot as pl

# Martin Krzywinski's 24-color, color-blind-friendly palette
color_blind_palette = {
    "orange": "#E69F00",  # Orange
    "skyblue":"#56B4E9",  # Sky Blue
    "g": "#009E73",  # Green
    "y":"#F0E442",  # Yellow
    "b": "#0072B2",  # Blue
    "darkorange":"#D55E00",  # Dark Orange
    "pink":"#CC79A7",  # Pink
    "grey":"#999999",  # Grey
    "lightyellow":"#F7CB4D",  # Light Yellow
    "lightblue":"#ADD8E6",  # Light Blue
    "lightgreen":"#90EE90",  # Light Green
    "lightpink":"#FFB6C1",  # Light Pink
    "darkred":"#A52A2A",  # Brown
    "purple":"#800080",  # Purple
    "olive":"#808000",  # Olive
    "orange":"#FFA500",  # Orange
    "steelblue":"#4682B4",  # Steel Blue
    "brown":"#8B4513",  # Saddle Brown
    "cadetblue": "#5F9EA0",  # Cadet Blue
    "r":"#CD5C5C"   # Indian Red
}


# Set default color cycle
def resolve_color(color):
    """Resolve shorthand color names to Tableau hex codes."""
    return color_blind_palette.get(color, color)  # Default to original if shorthand not found

# Override the default plot function to use custom color
original_plot = pl.plot

def patched_plot(*args, **kwargs):
    if 'color' in kwargs:
        kwargs['color'] = resolve_color(kwargs['color'])
    return original_plot(*args, **kwargs)

pl.plot = patched_plot 


# Returns the strain and stress rupture values defined as the maximum of the differential modulus from one given condition.
# Data contains all the stress ramps of one condition.
def gamma_rupture(data):
    all_diffMod = []
    all_strain = []
    all_stress = []
    
    all_strain_rupture = []
    all_stress_rupture = []
    i = 0
    
    for dataset in data:  # Assuming each argument is a list of dictionaries
        strain = np.array(dataset['Strain'], dtype='float64')
        stress = np.array(dataset['Shear Stress'], dtype='float64')

        # Calculate differential modulus for each dataset
        diffMod = np.gradient(stress, strain / 100.0)
        diffMod_smooth = gaussian_filter1d(diffMod, 2)
            
        # Finding the maximum of the differential modulus curve and the corresponding index from 0.2 Pa, corresponding to 14th point
        ind = 14
        for j in range(ind, np.size(diffMod_smooth)):
            if diffMod_smooth[j-1] < diffMod_smooth[j] :  # Finding the maximum
                ind = j
            # if  diffMod_smooth[j]>diffMod_smooth[j+1]:
            #     break #Stop at the first local maximum
            # if diffMod[j]>diffMod[j+1]: 
            #     break   #Stop at the first local maximum
        strain_rupture = strain[ind]
        stress_rupture = stress[ind]
           
        all_strain_rupture.append(strain_rupture)
        all_stress_rupture.append(stress_rupture)
        
        
        # if diffMod[j] > diffMod[j - 1] and diffMod[j] > diffMod[j + 1]:
        #         ind = j
        #         break  # Stop at the first local maximum
        # # Plotting the differential modulus and the max of the differential modulus
        # pl.figure()
        # pl.plot(stress, diffMod_smooth)
        # pl.plot(stress_rupture, diffMod_smooth[ind], marker='o', color='y')
        # pl.loglog()
        # pl.xlabel('Stress (Pa)')
        # pl.ylabel('Differential modulus (Pa)')
        # pl.title(str(i))
        # i = i+1

    return (all_strain_rupture, all_stress_rupture)


#%%
##Figure 1 Supplementary - stress and strain at rupture for HDFs
M = 3
# Samples 
data1 = CTL_HDF_stressramp
data2 = HDF_04percent_stressramp
data3 = HDF_4percent_stressramp

# Making one large array of all the conditions
all = [data1, data2, data3,]
# Name of the conditions
labels = ['CTL', '0.4%', '4%']
# Colors of the plots 
colors = [
    color_blind_palette["grey"],  # CTL
    color_blind_palette["g"],     # 0.4%
    color_blind_palette["b"],     # 4%
    ]

A = gamma_rupture(all[0])
# Calculating all the strains and stresses at rupture for all the conditions. 
#The results are gathered in the strain_data and stress_data arrays. Each line corresponds to one condition with all the corresponding strains or stresses
strain_data =[]
stress_data=[]
result =[]
for i in range(M):
    result = gamma_rupture(all[i]) # Calculating the strain and stress values at rupture for all the samples among one condition 
    strain_data.append(result[0][:])
    stress_data.append(result[1][:])
    
    
##Perform pairwise Mann-Whitney U tests for strain (Note:Comparing only CTL with other conditions)
p_values = []
for i in range(1, len(all)):  # Start from 1 to skip comparing CTL with itself
    _, p = mannwhitneyu(strain_data[0], strain_data[i], alternative='two-sided')
    p_values.append(p)

# Function to map p-value to annotation text
def get_p_value_annotation(p_value):
    if p_value < 0.001:
        return "***"
    elif p_value < 0.01:
        return "**"
    elif p_value < 0.05:
        return "*"
    else:
        return "ns"
print(p_values)

# Plotting strain at rupture
pl.figure(figsize=(8, 6))

for i, data in enumerate(strain_data):
    # Scatter plot of individual data points
    pl.scatter([i] * len(data), data, color=colors[i], s=60, alpha=0.7)

    # Mean and SD
    mean = np.mean(data)
    std = np.std(data)

    # Mark the mean with a larger circle in the corresponding color
    pl.scatter(i, mean, color=colors[i], marker='o', s=200, zorder=3)

    # Error bar to show SD
    pl.errorbar(i, mean, yerr=std, color=colors[i], capsize=10, linestyle='', linewidth=2)

# Customize tick parameters and axis scales
pl.tick_params(axis='both', which='major', length=8, width=2)  # Major ticks: longer and thicker
pl.tick_params(axis='both', which='minor', length=6, width=2)  # Minor ticks
pl.yscale('log')
pl.ylim(10, 101)
pl.gca().yaxis.set_major_locator(ticker.LogLocator(base=10.0, numticks=2))

# Set axis labels and tick formatting
pl.xticks(ticks=range(M), labels=labels, fontsize=30)
pl.yticks(fontsize=24)
pl.ylabel(r'$\gamma_r$ (%)', fontsize=30, labelpad=15)  # Labelpad increases the space between Y-axis numbers and label
    
# y_max = max([np.max(data) for data in strain_data])  # Find the maximum value in the dataset
# line_height = y_max * 1  # Starting height for annotation lines
# spacing_factor = 1.2 # Multiplicative spacing factor for log scale

# for i, p_value in enumerate(p_values):
#     annotation = get_p_value_annotation(p_value)
#     x1, x2 = 0, i + 1  # Positions of the two groups being compared

#     # Adjust y position for log scale
#     y = line_height * (spacing_factor ** i)

#     # Draw comparison line
#     pl.plot([x1, x1, x2, x2], [y, y, y, y], color="black", lw=1.5)

#     # Display annotation slightly above the line for clarity
#     pl.text((x1 + x2) / 2, y * 1.1, annotation, ha="center", va="bottom", fontsize=18, color="black")

# Adjust spine thickness
for spine in pl.gca().spines.values():
    spine.set_linewidth(1.5)
        
pl.tight_layout()
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/Strain at rupture - HDF.png', dpi=300, bbox_inches='tight')
pl.show()

# %%
# # Plotting stress at rupture
#Perform pairwise Mann-Whitney U tests for stress(Note:Comparing only CTL with other conditions)
p_values_stress = []
for i in range(1, len(all)):  # Start from 1 to skip comparing CTL with itself
    _, p = mannwhitneyu(stress_data[0], stress_data[i], alternative='two-sided')
    p_values_stress.append(p)

# Function to map p-value to annotation text
def get_p_value_annotation(p_value):
    if p_value < 0.001:
        return "***"
    elif p_value < 0.01:
        return "**"
    elif p_value < 0.05:
        return "*"
    else:
        return "ns"
print(p_values_stress)

# # Annotate statistical significance
# y_max = max([np.max(data) for data in stress_data])  # Find the maximum value in the dataset
# line_height = y_max + 15  # Starting height for annotation lines
# spacing = 25  # Spacing between annotation lines

# Plotting stress at rupture
pl.figure(figsize=(8, 6))

for i, data in enumerate(stress_data):
    # Scatter plot of individual data points
    pl.scatter([i] * len(data), data, color=colors[i], s=60, alpha=0.7)

    # Mean and SD
    mean = np.mean(data)
    std = np.std(data)

    # Mark the mean with a larger circle in the corresponding color
    pl.scatter(i, mean, color=colors[i], marker='o', s=200, zorder=3)

    # Error bar to show SD
    pl.errorbar(i, mean, yerr=std, color=colors[i], capsize=10, linestyle='', linewidth=2)

# Add labels and axis formatting
pl.xticks(ticks=range(M), labels=labels, fontsize=30)
pl.tick_params(axis='both', which='major', length=8, width=2)  # Major ticks: longer and thicker
pl.tick_params(axis='both', which='minor', length=6, width=2)  # Minor ticks
pl.yscale('log')
pl.ylim(10, 200)
pl.yticks(fontsize=24)
pl.ylabel(r'$\sigma_r$ (Pa)', fontsize=30, labelpad=15) # Labelpad increases the space between Y-axis numbers and label))

# y_max = max([np.max(data) for data in stress_data])  # Find the maximum value in the dataset
# line_height = y_max * 1.3  # Starting height for annotation lines
# spacing_factor = 1.5  # Multiplicative spacing factor for log scale

# for i, p_value in enumerate(p_values_stress):
#     annotation = get_p_value_annotation(p_value)
#     x1, x2 = 0, i + 1  # Positions of the two groups being compared

#     # Adjust y position for log scale
#     y = line_height * (spacing_factor ** i)

#     # Draw comparison line
#     pl.plot([x1, x1, x2, x2], [y, y, y, y], color="black", lw=1.5)

#     # Display annotation slightly above the line for clarity
#     pl.text((x1 + x2) / 2, y * 1.1, annotation, ha="center", va="bottom", fontsize=18, color="black")

for spine in pl.gca().spines.values():
        spine.set_linewidth(1.5)
pl.tight_layout()
pl.savefig('C:/Users/mtavasso/OneDrive - Delft University of Technology/Rheomicroscopy/Paper/Figures/Supplementary figures/Supplementary - Figure 1/Stress at rupture - HDF.png', dpi=300, bbox_inches='tight')
pl.show()